using System;
using System.IO;
using de.osci.osci12.common;
using System.Security.Cryptography;
using Org.BouncyCastle.Crypto;
using Org.BouncyCastle.Crypto.Signers;
using Org.BouncyCastle.Crypto.Parameters;
using Org.BouncyCastle.Security;
using Org.BouncyCastle.Math;
using Org.BouncyCastle.Crypto.Engines;

public interface IThreadRunnableMore {
	void Run();
}
	/// <summary>
	/// Zusammenfassung fr MoreSupport.
	/// </summary>
public class MoreSupport {

	public static byte[] ToByteArray(string sourceString) {
		return ToByteArray(sourceString, Constants.CHAR_ENCODING);
	}

	public static byte[] ToByteArray(string sourceString,string encoding) {
		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding); 
		return encoder.GetBytes(sourceString);
	}
	
	public static void ToStream(SupportClass.DigestOutputSupport str, 
		                         string sourceString, 
		                         string encoding) {

		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding); 
		byte[] bytes = encoder.GetBytes(sourceString);
		str.Write(bytes,0,bytes.Length);
	}
	
	public static void ToStream(Stream str,
		                         string sourceString, 
		                         string encoding) {

		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding); 
		byte[] bytes = encoder.GetBytes(sourceString);
		str.Write(bytes,0,bytes.Length);
	}
	
	public static String NewString(byte [] bytes, int off, int len) {
		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(Constants.CHAR_ENCODING); 
		return encoder.GetString(bytes, off, len);
	}
	
	public static String NewString(byte [] bytes, string encoding) {
		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding); 
		return encoder.GetString(bytes);
	}

	public static String NewString(Stream str) {
		return NewString( str,"UTF-8");
	}

	public static String NewString(Stream str, string encoding) {
		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding);
        System.IO.MemoryStream memStream = new System.IO.MemoryStream();
        byte[] bytes = new byte[1024];
        int anz = 0;
        while ((anz = str.Read(bytes, 0, bytes.Length)) > 0)
            memStream.Write(bytes, 0, anz);
        return MoreSupport.NewString(memStream.GetBuffer(), 0, (int)memStream.Length);
    }

    public static HashAlgorithm createMessageDigest(String digAlgo)
    {
        if (digAlgo.Equals(Constants.DIGEST_ALGORITHM_SHA1))
            return new SHA1Managed();
        else if (digAlgo.Equals(Constants.DIGEST_ALGORITHM_SHA256))
            return new SHA256Managed();
        else if (digAlgo.Equals(Constants.DIGEST_ALGORITHM_SHA512))
            return new SHA512Managed();
        else if (digAlgo.Equals(Constants.DIGEST_ALGORITHM_RIPEMD160))
            return new RIPEMD160Managed();
        throw new Exception("Illegal digest algorithm: " + digAlgo);
    }

    public static int getKeySize(java.security.cert.X509Certificate cert)
    {
        AsymmetricKeyParameter akp = cert.GetPublicKey();
        if (akp is ECPublicKeyParameters)
            return ((ECPublicKeyParameters)akp).Parameters.N.BitLength;
        else // RSA
            return ((RsaKeyParameters)akp).Modulus.BitLength;
    }

    public static bool checkSignature(java.security.cert.X509Certificate cert, byte[] data, byte[] signature, String sigAlg)
    {
        AsymmetricKeyParameter akp = cert.GetPublicKey();
        String digAlg = null;
        if (akp is ECPublicKeyParameters)
        {
            if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_ECDSA_SHA256))
                digAlg = "SHA-256";
            else if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_ECDSA_SHA512))
                digAlg = "SHA-512";
            ECDsaSigner signer = new ECDsaSigner();
            signer.Init(false, cert.GetPublicKey());
            byte[] digest = DigestUtilities.CalculateDigest(digAlg, data);
            return signer.VerifySignature(digest, new BigInteger(1, signature, 0, signature.Length / 2), 
                new BigInteger(1, signature, signature.Length / 2, signature.Length / 2));
        }
        else // RSA
        {
            if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_SHA1))
                digAlg = "SHA-1";
            else if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_SHA256)
                || sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_SHA256_PSS))
                digAlg = "SHA-256";
            else if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_SHA512)
                || sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_SHA512_PSS))
                digAlg = "SHA-512";
            else if (sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_RIPEMD160)
                || sigAlg.Equals(Constants.SIGNATURE_ALGORITHM_RSA_RIPEMD160_RFC4051))
                digAlg = "RIPEMD-160";

            IDigest id = DigestUtilities.GetDigest(digAlg);
            ISigner rsads;
            if (sigAlg.Contains("MGF"))
                rsads = new PssSigner(new RsaEngine(), id);
            else
                rsads = new RsaDigestSigner(id);
            rsads.Init(false, cert.GetPublicKey());
            rsads.BlockUpdate(data, 0, data.Length);
            return rsads.VerifySignature(signature);
        }
    }

}