using System;
using System.IO;
using System.Text;
using java.security.cert;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using de.osci.osci12.common;
//using Microsoft.Cryptography;

public interface IThreadRunnable {
	void Run();
}

public class SupportClass {

	static public System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(Constants.CHAR_ENCODING); 

	public static string ToString(string data, string encoding) {

		Encoder enc;
		switch (encoding) {
			case "UTF-8":
				enc =  Encoding.UTF8.GetEncoder();
				break;

			default:
				throw new InvalidOperationException("CHAR_Encoding (" + encoding + ") wird nicht untersttzt.");
				break;
		}

		Char[] chars = data.ToCharArray();
		int byteCount = enc.GetByteCount(chars, 0, chars.Length, true);
		Byte[] bytes = new Byte[byteCount];
		int bytesEncodedCount = enc.GetBytes(chars, 0, chars.Length, bytes, 0, true);

		return bytes.ToString();
	}
	
	/// <summary>
	/// Converts an array of sbytes to an array of bytes
	/// </summary>
	/// <param name="sbyteArray">The array of sbytes to be converted</param>
	/// <returns>The new array of bytes</returns>
	public static byte[] ToByteArray(sbyte[] sbyteArray) {
		byte[] byteArray = new byte[sbyteArray.Length];
		for(int index=0; index < sbyteArray.Length; index++)
			byteArray[index] = (byte) sbyteArray[index];
		return byteArray;
	}

	// ein einfacher Cast sollte reichen:
	//	/// <summary>
	//	/// pr ,gemacht damit die Aufrufe keine Fehler liefern wenn byte[] bergeben wird
	//	/// </summary>
	//	/// <param name="sbyteArray"></param>
	//	/// <returns></returns>
	//	public static byte[] ToByteArray(byte[] byteArray) {
	//		return byteArray;
	//	}

	/// <summary>
	/// Converts a string to an array of bytes
	/// </summary>
	/// <param name="sourceString">The string to be converted</param>
	/// <returns>The new array of bytes</returns>
	public static byte[] ToByteArray(string sourceString) {
		byte[] byteArray = new byte[sourceString.Length];
		for (int index=0; index < sourceString.Length; index++)
			byteArray[index] = (byte) sourceString[index];
		return byteArray;
	}

	/*******************************/
	/// <summary>Reads a number of characters from the current source Stream and writes the data to the target array at the specified index.</summary>
	/// <param name="sourceStream">The source Stream to read from</param>
	/// <param name="target">Contains the array of characteres read from the source Stream.</param>
	/// <param name="start">The starting index of the target array.</param>
	/// <param name="count">The maximum number of characters to read from the source Stream.</param>
	/// <returns>The number of characters read. The number will be less than or equal to count depending on the data available in the source Stream.</returns>
	public static System.Int32 ReadInput(System.IO.Stream sourceStream, ref sbyte[] target, int start, int count) {
		byte[] receiver = new byte[target.Length];
		int bytesRead   = sourceStream.Read(receiver, start, count);
			
		for(int i = start; i < start + bytesRead; i++)
			target[i] = (sbyte)receiver[i];
			
		if (bytesRead == 0) return - 1;
		return bytesRead;
	}

	/*******************************/
	/// <summary>Reads a number of characters from the current source Stream and writes the data to the target array at the specified index.</summary>
	/// <param name="sourceStream">The source Stream to read from</param>
	/// <param name="target">Contains the array of characteres read from the source Stream.</param>
	/// <param name="start">The starting index of the target array.</param>
	/// <param name="count">The maximum number of characters to read from the source Stream.</param>
	/// <returns>The number of characters read. The number will be less than or equal to count depending on the data available in the source Stream.</returns>
	public static System.Int32 ReadInput(System.IO.Stream sourceStream, ref byte[] target, int start, int count) {
		byte[] receiver = new byte[target.Length];
		int bytesRead   = sourceStream.Read(receiver, start, count);
			
		for(int i = start; i < start + bytesRead; i++)
			target[i] = (byte)receiver[i];
			
		if (bytesRead == 0) return - 1;
		return bytesRead;
	}

	/// <summary>Reads a number of characters from the current source TextReader and writes the data to the target array at the specified index.</summary>
	/// <param name="sourceTextReader">The source TextReader to read from</param>
	/// <param name="target">Contains the array of characteres read from the source TextReader.</param>
	/// <param name="start">The starting index of the target array.</param>
	/// <param name="count">The maximum number of characters to read from the source TextReader.</param>
	/// <returns>The number of characters read. The number will be less than or equal to count depending on the data available in the source TextReader.</returns>
	public static System.Int32 ReadInput(System.IO.TextReader sourceTextReader, ref sbyte[] target, int start, int count) {
		char[] charArray = new char[target.Length];
		int bytesRead = sourceTextReader.Read(charArray, start, count);

		for(int index=start; index<start+bytesRead; index++)
			target[index] = (sbyte)charArray[index];

		return bytesRead;
	}

	/*******************************/
	public static System.String Stream2String(System.IO.Stream str, string encoding) {

		// ggfls. Standard-Encoding verwenden:
		if(encoding == null || encoding.Length < 1) encoding = Constants.CHAR_ENCODING;
		System.Text.Encoding encoder = System.Text.Encoding.GetEncoding(encoding);
		System.Text.StringBuilder buf = new System.Text.StringBuilder();

		int count;
		byte[] bytes = new byte[1024];
		while ((count = str.Read(bytes, 0, bytes.Length)) >0)
			buf.Append(encoder.GetString(bytes, 0, count));
		return buf.ToString();
	}

	/*******************************/
	// Generic function using HashAlgorithm 
	public static string HashData(HashAlgorithm hashAlg, string str ) {
		byte[] rawBytes = System.Text.UTF8Encoding.UTF8.GetBytes( str );
		byte[] hashData = hashAlg.ComputeHash( rawBytes );
		//		log.debug("hash: " + BitConverter.ToString( hashData ) );
		return BitConverter.ToString( hashData );
	}

	public static string HashMD5(string str) {
		MD5 md5 = new System.Security.Cryptography.MD5CryptoServiceProvider();
		return HashData( md5, str); // Sample of the MD5 algorithm 
	}

	public static string HashSHA1(string str) {
		SHA1 sha1 = new System.Security.Cryptography.SHA1Managed(); 
		return HashData( sha1, str);
	}

	/// <summary>
	///  liefert die Differenz zw. Stream.Length und Stream.Position
	/// </summary>
	/// <param name="stream"></param>
	/// <returns></returns>
	public static long available(System.IO.Stream stream) {
		return stream.Length - stream.Position;
	}

	/*******************************/
	public static sbyte[] ToSByteArray(byte[] byteArray) {
		sbyte[] sbyteArray = new sbyte[byteArray.Length];
		for(int index=0; index < byteArray.Length; index++)
			sbyteArray[index] = (sbyte) byteArray[index];
		return sbyteArray;
	}

	/*******************************/
	/// <summary>
	/// Converts an array of sbytes to an array of chars
	/// </summary>
	/// <param name="sByteArray">The array of sbytes to convert</param>
	/// <returns>The new array of chars</returns>
	public static char[] ToCharArray(sbyte[] sByteArray) {
		char[] charArray = new char[sByteArray.Length];	   
		sByteArray.CopyTo(charArray, 0);
		return charArray;
	}

	/// <summary>
	/// Converts an array of bytes to an array of chars
	/// </summary>
	/// <param name="byteArray">The array of bytes to convert</param>
	/// <returns>The new array of chars</returns>
	public static char[] ToCharArray(byte[] byteArray) {
		char[] charArray = new char[byteArray.Length];	   
		byteArray.CopyTo(charArray, 0);
		return charArray;
	}

	/*******************************/
	public static void WriteStackTrace(System.Exception throwable, System.IO.TextWriter stream) {
		stream.Write(throwable.StackTrace);
		stream.Flush();
	}

	/*******************************/
	/// <summary>
	/// This class is a holder for two keys, a private key and a public key.
	/// </summary>
	public class KeyPairSupport {
		private PrivateKeySupport privateKey;
		private PublicKeySupport  publicKey;
		
		/// <summary>
		/// Construct a new key pair object with the specified PublicKeySupport and PrivateKeySupport
		/// </summary>
		/// <param name="publicKey">The public key</param>
		/// <param name="privateKey">The private key</param>
		public KeyPairSupport(PublicKeySupport  publicKey, PrivateKeySupport privateKey) {
			this.publicKey  = publicKey;
			this.privateKey = privateKey;
		}

		/// <summary>
		/// A reference to the private key component of this key pair
		/// </summary>
		public PrivateKeySupport Private {
			get {
				return this.privateKey;
			}
		}

		/// <summary>
		/// A reference to the public key component of this key pair
		/// </summary>
		public PublicKeySupport Public {
			get {
				return this.publicKey;
			}
		}
	}

	/*******************************/
	/// <summary>
	/// This class offers support for all classes that use cryptographic private keys.
	/// </summary>
	public class PrivateKeySupport: KeySupport {
		/// <summary>
		/// Construct a new private key object
		/// </summary>
		/// 
		public string containerName;
		public string providerName;
		public int providerType;
		public int keySpec;

		public PrivateKeySupport() {
		}
	}

	/*******************************/
	/// <summary>
	/// This class offers support for all classes that use cryptographic keys.
	/// </summary>
	public class KeySupport {
		private System.Security.Cryptography.KeyedHashAlgorithm algorithm;
		public bool isMachineKeyset;

		/// <summary>
		/// Construct to new objects key
		/// </summary>
		public KeySupport() {
		}

		/// <summary>
		/// Construct to new objects key with the algorithm specified
		/// </summary>
		/// <param name="algorithm">the cryptographic algorithm</param>
		public KeySupport(System.Security.Cryptography.KeyedHashAlgorithm algorithm) {
			this.algorithm = algorithm;
		}

		/// <summary>
		/// The standard algorithm name for this key
		/// </summary>
		/// <returns>the keyed hash algorithm name</returns>
		public String GetAlgorithm() {
			return this.algorithm.ToString();
		}		

		/// <summary>
		/// The key to be used in the algorithm.
		/// </summary>
		public byte[] Key {
			get {
				return this.algorithm.Key;
			}
		}
	}

	/*******************************/
	/// <summary>
	/// This class offers support for all classes that use cryptographic public keys.
	/// </summary>
	public class PublicKeySupport: KeySupport {
		/// <summary>
		/// Construct a new public key object
		/// </summary>
		public PublicKeySupport() {
		}
	}

	/*******************************/
	/// <summary>
	/// This class uses a cryptographic Random Number Generator to provide support for
	/// strong pseudo-random number generation.
	/// </summary>
	public class SecureRandomSupport {
		//<konvert> pr fehlte </konvert>
		public static SecureRandomSupport getInstance(String SHA1PRNG){return new SecureRandomSupport();}
		public void nextBytes(byte[] b) {
			;
		}

		private System.Security.Cryptography.RNGCryptoServiceProvider generator;

		/// <summary>
		/// Initializes a new instance of the random number generator.
		/// </summary>
		public SecureRandomSupport() {
			this.generator = new System.Security.Cryptography.RNGCryptoServiceProvider();
		}

		/// <summary>
		/// Initializes a new instance of the random number generator with the given seed.
		/// </summary>
		/// <param name="seed">The initial seed for the generator</param>
		public SecureRandomSupport(byte[] seed) {
			this.generator = new System.Security.Cryptography.RNGCryptoServiceProvider(seed);
		}

		/// <summary>
		/// Returns an array of bytes with a sequence of cryptographically strong random values
		/// </summary>
		/// <param name="randomnumbersarray">The array of bytes to fill</param>
		public byte[] NextBytes(byte[] randomnumbersarray) {			
			this.generator.GetBytes(randomnumbersarray);
			return randomnumbersarray;
		}

		/// <summary>
		/// Returns the given number of seed bytes generated for the first running of a new instance 
		/// of the random number generator
		/// </summary>
		/// <param name="numberOfBytes">Number of seed bytes to generate</param>
		/// <returns>Seed bytes generated</returns>
		public static byte[] GetSeed(int numberOfBytes) {
			System.Security.Cryptography.RNGCryptoServiceProvider generatedSeed = new System.Security.Cryptography.RNGCryptoServiceProvider();
			byte[] seeds = new byte[numberOfBytes];
			generatedSeed.GetBytes(seeds);
			return seeds;
		}

		/// <summary>
		/// Creates a new instance of the random number generator with the seed provided by the user
		/// </summary>
		/// <param name="newSeed">Seed to create a new random number generator</param>
		public void SetSeed(byte[] newSeed) {
			this.generator = new System.Security.Cryptography.RNGCryptoServiceProvider(newSeed);
		}

		/// <summary>
		/// Creates a new instance of the random number generator with the seed provided by the user
		/// </summary>
		/// <param name="newSeed">Seed to create a new random number generator</param>
		public void SetSeed(long newSeed) {
			byte[] bytes = new byte[8];
			for (int index= 7; index > 0 ; index--) {
				bytes[index] = (byte)(newSeed - (long)((newSeed >> 8) << 8));
				newSeed  = (long)(newSeed >> 8);
			}			
			SetSeed(bytes);
		}
	}

	/*******************************/
	/// <summary>
	/// This method is used as a dummy method to simulate VJ++ behavior
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static long Identity(long literal) {
		return literal;
	}

	/// <summary>
	/// This method is used as a dummy method to simulate VJ++ behavior
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static ulong Identity(ulong literal) {
		return literal;
	}

	/// <summary>
	/// This method is used as a dummy method to simulate VJ++ behavior
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static float Identity(float literal) {
		return literal;
	}

	/// <summary>
	/// This method is used as a dummy method to simulate VJ++ behavior
	/// </summary>
	/// <param name="literal">The literal to return</param>
	/// <returns>The received value</returns>
	public static double Identity(double literal) {
		return literal;
	}

	/*******************************/
	public class ThreadClass:IThreadRunnable {
		private System.Threading.Thread threadField;

		public ThreadClass() {
			threadField = new System.Threading.Thread(new System.Threading.ThreadStart(Run));
		}

		public ThreadClass(System.Threading.ThreadStart p1) {
			threadField = new System.Threading.Thread(p1);
		}

		public virtual void Run() {
		}

		public virtual void Start() {
			threadField.Start();
		}

		public System.Threading.Thread Instance {
			get {
				return threadField;
			}
			set {
				threadField	= value;
			}
		}

		public System.String Name {
			get {
				return threadField.Name;
			}
			set {
				if (threadField.Name == null)
					threadField.Name = value; 
			}
		}

		public System.Threading.ThreadPriority Priority {
			get {
				return threadField.Priority;
			}
			set {
				threadField.Priority = value;
			}
		}

		public bool IsAlive {
			get {
				return threadField.IsAlive;
			}
		}

		public bool IsBackground {
			get {
				return threadField.IsBackground;
			} 
			set {
				threadField.IsBackground = value;
			}
		}

		public void Join() {
			threadField.Join();
		}

		public void Join(long p1) {
			lock(this) {
				threadField.Join(new System.TimeSpan(p1 * 10000));
			}
		}

		public void Join(long p1, int p2) {
			lock(this) {
				threadField.Join(new System.TimeSpan(p1 * 10000 + p2 * 100));
			}
		}

		public void Resume() {
			threadField.Resume();
		}

		public void Abort() {
			threadField.Abort();
		}

		public void Abort(System.Object stateInfo) {
			lock(this) {
				threadField.Abort(stateInfo);
			}
		}

		public void Suspend() {
			threadField.Suspend();
		}

		public override System.String ToString() {
			return "Thread[" + Name + "," + Priority.ToString() + "," + "" + "]";
		}

		public static ThreadClass Current() {
			ThreadClass CurrentThread = new ThreadClass();
			CurrentThread.Instance = System.Threading.Thread.CurrentThread;
			return CurrentThread;
		}
	}

	/*******************************/
	/// <summary>
	/// Encapsulates the functionality of message digest algorithms such as SHA-1 or MD5.
	/// </summary>
	public class MessageDigestSupport {
		private System.Security.Cryptography.HashAlgorithm algorithm;
		private byte[] data;
		private int position;
		private string algorithmName;

		/// <summary>
		/// The HashAlgorithm instance that provide the cryptographic hash algorithm
		/// </summary>
		public System.Security.Cryptography.HashAlgorithm Algorithm {
			get {
				return this.algorithm;
			}
			set {
				this.algorithm  = value;
			}
		}

		/// <summary>
		/// The digest data
		/// </summary>
		public byte[] Data {
			get {
				return this.data;
			}
			set {
				this.data  = value;
			}
		}

		/// <summary>
		/// The name of the cryptographic hash algorithm used in the instance
		/// </summary>
		public string AlgorithmName {
			get {
				return this.algorithmName;
			}
		}

		/// <summary>
		/// Creates a message digest using the specified name to set Algorithm property.
		/// </summary>
		/// <param name="algorithm">The name of the algorithm to use</param>
		public MessageDigestSupport(System.String algorithm) {			
			if (algorithm.Equals("SHA-1")) {
				this.algorithmName = "SHA";
			}
			else {
				this.algorithmName = algorithm;
			}
			this.Algorithm = (System.Security.Cryptography.HashAlgorithm) System.Security.Cryptography.CryptoConfig.CreateFromName(this.algorithmName);			
			this.position  = 0;
		}

		/// <summary>
		/// Computes the hash value for the internal data digest.
		/// </summary>
		/// <returns>The array of signed bytes with the resulting hash value</returns>
		public byte[] DigestData() {
			byte[] result = this.Algorithm.ComputeHash(this.data);
			this.Reset();
			return result;
		}

		/// <summary>
		/// Performs and update on the digest with the specified array and then completes the digest
		/// computation.
		/// </summary>
		/// <param name="newData">The array of bytes for final update to the digest</param>
		/// <returns>An array of signed bytes with the resulting hash value</returns>
		public byte[] DigestData(byte[] newData) {
			this.Update(newData);
			return this.DigestData();
		}

		/// <summary>
		/// Updates the digest data with the specified array of bytes by making an append
		/// operation in the internal array of data.
		/// </summary>
		/// <param name="newData">The array of bytes for the update operation</param>
		public void Update(byte[] newData) {
			if (position == 0) {
				this.Data = newData;
				this.position = this.Data.Length - 1;
			}
			else {
				byte[] oldData = this.Data;
				this.Data = new byte[newData.Length + position + 1];
				oldData.CopyTo(this.Data, 0);
				newData.CopyTo(this.Data, oldData.Length);
	            
				this.position = this.Data.Length - 1;
			}
		}
        
		/// <summary>
		/// Updates the digest data with the input byte by calling the method Update with an array.
		/// </summary>
		/// <param name="newData">The input byte for the update</param>
		public void Update(byte newData) {
			byte[] newDataArray = new byte[1];
			newDataArray[0] = newData;
			this.Update(newDataArray);
		}

		/// <summary>
		/// Updates the specified count of bytes with the input array of bytes starting at the
		/// input offset.
		/// </summary>
		/// <param name="newData">The array of bytes for the update operation</param>
		/// <param name="offset">The initial position to start from in the array of bytes</param>
		/// <param name="count">The number of bytes fot the update</param>
		public void Update(byte[] newData, int offset, int count) {
			byte[] newDataArray = new byte[count];
			System.Array.Copy(newData, offset, newDataArray, 0, count);
			this.Update(newDataArray);
		}
		
		/// <summary>
		/// Resets the digest data to the initial state.
		/// </summary>
		public void Reset() {
			this.data = null;
			this.position = 0;
		}

		/// <summary>
		/// Returns a string representation of the Message Digest
		/// </summary>
		/// <returns>A string representation of the object</returns>
		public override string ToString() {
			return this.Algorithm.ToString();
		}

		/// <summary>
		/// Generates a new instance of the MessageDigestSupport class using the specified algorithm
		/// </summary>
		/// <param name="algorithm">The name of the algorithm to use</param>
		/// <returns>A new instance of the MessageDigestSupport class</returns>
		public static MessageDigestSupport GetInstance(System.String algorithm) {
			return new MessageDigestSupport(algorithm);
		}
		
		/// <summary>
		/// Compares two arrays of signed bytes evaluating equivalence in digest data
		/// </summary>
		/// <param name="firstDigest">An array of signed bytes for comparison</param>
		/// <param name="secondDigest">An array of signed bytes for comparison</param>
		/// <returns>True if the input digest arrays are equal</returns>
		public static bool EquivalentDigest(System.SByte[] firstDigest, System.SByte[] secondDigest) {
			bool result = false;
			if (firstDigest.Length == secondDigest.Length) {
				int index = 0;
				result = true;
				while(result && index < firstDigest.Length) {
					result = firstDigest[index] == secondDigest[index];
					index++;
				}
			}
			
			return result;
		}
	}

	/*******************************/
	/// <summary>
	/// It allows to associate a MessageDigestSupport object with a stream.
	/// When the message digest is activated the write methods updates the message 
	/// digest before actually write any data to the stream.
	/// </summary>
	public class DigestOutputSupport {
		private MessageDigestSupport messageDigest;
		private bool activated;
		private System.IO.Stream stream;

		/// <summary>
		/// Creates a new object of type DigestOutputSupport with the specified MessageDigest an the specified stream
		/// </summary>
		/// <param name="stream">The output stream to use</param>
		/// <param name="md">The message digest to associate to the stream</param>
		public DigestOutputSupport(System.IO.Stream stream, MessageDigestSupport md) {
			this.stream = stream;
			this.MessageDigest = md;
		}
		
		/// <summary>
		/// pr 
		/// </summary>
		public System.IO.Stream BaseStream {
			get {
				return stream;
			}
		}
		/// <summary>
		/// The message digest associated to this object
		/// </summary>
		public MessageDigestSupport MessageDigest {
			get {
				return this.messageDigest;
			}
			set {
				this.messageDigest = value;
			}
		}

		/// <summary> 
		/// True if the message digest needs to be updated in every call to write 
		/// </summary>
		public System.Boolean Activated {
			get {
				return this.activated;
			}
			set {
				this.activated = value;
			}
		}

		/// <summary>
		/// If the message digest is activated then it is updated with the specified byte array and then 
		/// write the byte array to the stream
		/// </summary>
		/// <param name="data">The byte array</param>
		/// <param name="start">The start position in the array</param>
		/// <param name="count">The count of bytes to write</param>
		public void Write(byte[] data, int start, int count) {
			if (this.activated) {
				this.messageDigest.Update(data, start, count);
			}
			this.stream.Write(data, start, count);
		}

		/// <summary>
		/// If the message digest is activated then it is updated with the specified byte and then
		/// is written to the output stream.
		/// </summary>
		/// <param name="data">The specified byte to write to the output stream</param>
		public void Write(System.Int32 data) {
			byte databyte = System.Convert.ToByte(data);
			if (this.activated) {
				this.messageDigest.Update(databyte);
			}
			this.stream.WriteByte(databyte);
		}		
		
		/// <summary>
		/// Returns a string representation of the Digest output stream. Actually is just a representation
		/// of the message digest object associated to the digest output instance
		/// </summary>
		/// <returns>A string representation of the Digest output stream</returns>
		public override String ToString() {
			return this.MessageDigest.ToString();
		}
	}

	/*******************************/
	public static Object PutElement(System.Collections.Hashtable hashTable, Object key, Object newValue) {
		Object element = hashTable[key];
		hashTable[key] = newValue;
		return element;
	}

	/*******************************/
	public class CalendarManager {
		public const int YEAR = 0;
		public const int MONTH = 1;
		public const int DATE = 2;
		public const int HOUR = 3;
		public const int MINUTE = 4;
		public const int SECOND = 5;
		public const int MILLISECOND = 6;
		public const int DAY_OF_MONTH = 7;
		public const int HOUR_OF_DAY = 8;

		static public CalendarHashTable manager = new CalendarHashTable();

		public class CalendarHashTable:System.Collections.Hashtable {
			public System.DateTime GetDateTime(System.Globalization.Calendar calendar) {
				if (this[calendar] != null)
					return ((CalendarProperties) this[calendar]).dateTime;
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTime = System.DateTime.Now;
					this.Add(calendar, tempProps);
					return this.GetDateTime(calendar);
				}
			}
			public void SetDateTime(System.Globalization.Calendar calendar, System.DateTime date) {
				if (this[calendar] != null) {
					((CalendarProperties) this[calendar]).dateTime = date;
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTime = date;
					this.Add(calendar, tempProps);
				}
			}
			public void Set(System.Globalization.Calendar calendar, int field, int fieldValue) {
				if (this[calendar] != null) {
					System.DateTime tempDate = ((CalendarProperties) this[calendar]).dateTime;
					switch (field) {
						case CalendarManager.DATE:
							tempDate = tempDate.AddDays(fieldValue - tempDate.Day);
							break;
						case CalendarManager.HOUR:
							tempDate = tempDate.AddHours(fieldValue - tempDate.Hour);
							break;
						case CalendarManager.MILLISECOND:
							tempDate = tempDate.AddMilliseconds(fieldValue - tempDate.Millisecond);
							break;
						case CalendarManager.MINUTE:
							tempDate = tempDate.AddMinutes(fieldValue - tempDate.Minute);
							break;
						case CalendarManager.MONTH:
							//Month value is 0-based. e.g., 0 for January
							tempDate = tempDate.AddMonths(fieldValue - (tempDate.Month + 1));
							break;
						case CalendarManager.SECOND:
							tempDate = tempDate.AddSeconds(fieldValue - tempDate.Second);
							break;
						case CalendarManager.YEAR:
							tempDate = tempDate.AddYears(fieldValue - tempDate.Year);
							break;
						case CalendarManager.DAY_OF_MONTH:
							tempDate = tempDate.AddDays(fieldValue - tempDate.Day);
							break;
						case CalendarManager.HOUR_OF_DAY:
							tempDate = tempDate.AddHours(fieldValue - tempDate.Hour);
							break;

						default:
							break;
					}
					((CalendarProperties) this[calendar]).dateTime = tempDate;
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTime = System.DateTime.Now;
					this.Add(calendar, tempProps);
					this.Set(calendar, field, fieldValue);
				}
			}

			public void Set(System.Globalization.Calendar calendar, int year, int month, int day) {
				if (this[calendar] != null) {
					this.Set(calendar, CalendarManager.YEAR, year);
					this.Set(calendar, CalendarManager.MONTH, month);
					this.Set(calendar, CalendarManager.DATE, day);
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					//Month value is 0-based. e.g., 0 for January
					tempProps.dateTime = new System.DateTime(year, month + 1, day);
					this.Add(calendar, tempProps);
				}
			}

			public void Set(System.Globalization.Calendar calendar, int year, int month, int day, int hour, int minute) {
				if (this[calendar] != null) {
					this.Set(calendar, CalendarManager.YEAR, year);
					this.Set(calendar, CalendarManager.MONTH, month);
					this.Set(calendar, CalendarManager.DATE, day);
					this.Set(calendar, CalendarManager.HOUR, hour);
					this.Set(calendar, CalendarManager.MINUTE, minute);
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					//Month value is 0-based. e.g., 0 for January
					tempProps.dateTime = new System.DateTime(year, month + 1, day, hour, minute, 0);
					this.Add(calendar, tempProps);
				}
			}

			public void Set(System.Globalization.Calendar calendar, int year, int month, int day, int hour, int minute, int second) {
				if (this[calendar] != null) {
					this.Set(calendar, CalendarManager.YEAR, year);
					this.Set(calendar, CalendarManager.MONTH, month);
					this.Set(calendar, CalendarManager.DATE, day);
					this.Set(calendar, CalendarManager.HOUR, hour);
					this.Set(calendar, CalendarManager.MINUTE, minute);
					this.Set(calendar, CalendarManager.SECOND, second);
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					//Month value is 0-based. e.g., 0 for January
					tempProps.dateTime = new System.DateTime(year, month + 1, day, hour, minute, second);
					this.Add(calendar, tempProps);
				}
			}

			public int  Get(System.Globalization.Calendar calendar, int field) {
				if (this[calendar] != null) {
					switch (field) {
						case CalendarManager.DATE:
							return ((CalendarProperties) this[calendar]).dateTime.Day;
						case CalendarManager.HOUR:
							int tempHour = ((CalendarProperties) this[calendar]).dateTime.Hour;
							return tempHour > 12 ? tempHour - 12 : tempHour;

						case CalendarManager.MILLISECOND:
							return ((CalendarProperties) this[calendar]).dateTime.Millisecond;
						case CalendarManager.MINUTE:
							return ((CalendarProperties) this[calendar]).dateTime.Minute;
						case CalendarManager.MONTH:
							//Month value is 0-based. e.g., 0 for January
							return ((CalendarProperties) this[calendar]).dateTime.Month - 1;
						case CalendarManager.SECOND:
							return ((CalendarProperties) this[calendar]).dateTime.Second;
						case CalendarManager.YEAR:
							return ((CalendarProperties) this[calendar]).dateTime.Year;
						case CalendarManager.DAY_OF_MONTH:
							return ((CalendarProperties) this[calendar]).dateTime.Day;
						case CalendarManager.HOUR_OF_DAY:
							return ((CalendarProperties) this[calendar]).dateTime.Hour;

						default:
							return 0;
					}
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTime = System.DateTime.Now;
					this.Add(calendar, tempProps);
					return this.Get(calendar, field);
				}
			}

			public void SetTimeInMilliseconds(System.Globalization.Calendar calendar, long milliseconds) {
				if (this[calendar] != null) {
					((CalendarProperties) this[calendar]).dateTime = new System.DateTime(milliseconds);
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTime = new System.DateTime(System.TimeSpan.TicksPerMillisecond * milliseconds);
					this.Add(calendar, tempProps);
				}
			}
			
			public System.DayOfWeek GetFirstDayOfWeek(System.Globalization.Calendar calendar) {
				if (this[calendar] != null && ((CalendarProperties)this[calendar]).dateTimeFormat != null) {
					return ((CalendarProperties) this[calendar]).dateTimeFormat.FirstDayOfWeek;
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTimeFormat = new System.Globalization.DateTimeFormatInfo();
					tempProps.dateTimeFormat.FirstDayOfWeek = System.DayOfWeek.Sunday;
					this.Add(calendar, tempProps);
					return this.GetFirstDayOfWeek(calendar);
				}
			}

			public void SetFirstDayOfWeek(System.Globalization.Calendar calendar, System.DayOfWeek  firstDayOfWeek) {
				if (this[calendar] != null && ((CalendarProperties)this[calendar]).dateTimeFormat != null) {
					((CalendarProperties) this[calendar]).dateTimeFormat.FirstDayOfWeek = firstDayOfWeek;
				}
				else {
					CalendarProperties tempProps = new CalendarProperties();
					tempProps.dateTimeFormat = new System.Globalization.DateTimeFormatInfo();
					this.Add(calendar, tempProps);
					this.SetFirstDayOfWeek(calendar, firstDayOfWeek);
				}
			}

			public void Clear(System.Globalization.Calendar calendar) {
				if (this[calendar] != null)
					this.Remove(calendar);
			}

			public void Clear(System.Globalization.Calendar calendar, int field) {
				if (this[calendar] != null)
					this.Remove(calendar);
				else
					this.Set(calendar, field, 0);
			}

			class CalendarProperties {
				public System.DateTime dateTime;
				public System.Globalization.DateTimeFormatInfo dateTimeFormat;
			}
		}
	}

	/*******************************/
	/// <summary>
	/// Support for digital signatures
	/// </summary>
	public class DigitalSignature {
		private System.Security.Cryptography.AsymmetricSignatureFormatter formatter;
		private System.Security.Cryptography.AsymmetricSignatureDeformatter deformatter;
		private System.Security.Cryptography.HashAlgorithm hashAlgorithm;
		private int objective;
		private string algorithmName;
		private byte[] data;
		private int position;
/*
		private RSABase rsa;

		public void initVerify(java.security.cert.X509Certificate x) 
        {
//            rsa = new Microsoft.Cryptography.RSAManaged();
            rsa = new Microsoft.Cryptography.BouncyCastle.RSABouncyCastle();
            X509Certificate2 xnet = new X509Certificate2(x.GetRawCertData());
            rsa.ImportParameters(((RSA)xnet.PublicKey.Key).ExportParameters(false));	
			objective=2;
		}
*/
		/// <summary>
		/// Gets or sets the data to be signed
		/// </summary>
		public byte[] Data {
			get {
				return this.data;
			}
			set {
				this.data  = value;
			}
		}

		/// <summary>
		/// Gets the name of the cryptographic algorithm used in the instance
		/// </summary>
		public string AlgorithmName {
			get {
				return this.algorithmName;
			}
		}

		protected DigitalSignature() {}

		/// <summary>
		/// Generates a DigitalSignature instance with the specified algorithm
		/// </summary>
		/// <param name="algorithmName">Name of the algorithm for the new DigitalSignature instance</param>
		/// <returns>The new DigitalSignature instance</returns>
		public static DigitalSignature GetInstance(String algorithmName) {
			DigitalSignature signature = null;
            signature = new DigitalSignature();
            signature.formatter = new System.Security.Cryptography.RSAPKCS1SignatureFormatter();
            signature.deformatter = new System.Security.Cryptography.RSAPKCS1SignatureDeformatter();

            signature.algorithmName = (String)Constants.NET_MAP[algorithmName];
            if (signature.algorithmName == null)
                throw new System.Exception("Algorithm not supported");

			signature.objective = 0;

			return signature;
		}

		/// <summary>
		/// Sets the objective property to specify that the signature was created for signing
		/// </summary>
		public void Signing() {
			this.objective = 1;
		}

		/// <summary>
		/// Sets the objetive property to specify that the signature was created for verification
		/// </summary>
		public void Verification() {
			this.objective = 2;
		}

		/// <summary>
		/// Creates a signature from the updated data
		/// </summary>
		/// <returns>Signature bytes of the signing operation's result</returns>
		public byte[] Sign() {
			byte[] realSignature = null;
			if (this.objective == 1)
				realSignature = this.formatter.CreateSignature(this.Data);
			else
				throw new System.Exception("Object was not created for signing");
			this.Reset();
			return realSignature;
		}

		/// <summary>
		/// Verifies the given signature with the updated data
		/// </summary>
		/// <param name="signature">Signature bytes to be verified</param>
		/// <returns>True if the signature was verified, otherwise false</returns>
        //
        /*
		public bool Verify(byte[] signature) {
			bool result = false;
            if (this.objective == 2)
            {
                String s = MoreSupport.NewString(this.Data, "UTF-8");
                return rsa.VerifyData(this.Data, this.algorithmName, signature);
            }
            else
                throw new System.Exception("Object was not created for verification");
			this.Reset();
			return result;
		}
        */
		/// <summary>
		/// Updates the data with the specified array of bytes
		/// </summary>
		/// <param name="newData">The array of bytes to update the data with</param>
		public void Update(byte[] newData) {
            this.Data = newData;
		}
        
		/// <summary>
		/// Updates the data with the specified byte
		/// </summary>
		/// <param name="newData">The byte to update the data with</param>
		public void Update(byte newData) {
			byte[] newDataArray = new byte[1];
			newDataArray[0] = newData;
			this.Update(newDataArray);
		}

		/// <summary>
		/// Updates the data with a part of the specified array of bytes 
		/// </summary>
		/// <param name="newData">The array of bytes containing the part of bytes to update the data with</param>
		/// <param name="offset">The initial position of the part of bytes of the array of bytes</param>
		/// <param name="count">The number of bytes of the part of bytes</param>
		public void Update(byte[] newData, int offset, int count) {
			byte[] newDataArray = new byte[count];
			System.Array.Copy(newData, offset, newDataArray, 0, count);
			this.Update(newDataArray);
		}

		/// <summary>
		/// Resets the data to the initial state
		/// </summary>
		private void Reset() {
			this.data = null;
			this.position = 0;
		}

		/// <summary>
		/// String representation of the digital signature object
		/// </summary>
		/// <returns>String representation of the signature</returns>
		public override string ToString() {
			string result= "Instance of DigitalSignature for ";
			if (this.objective == 1)
				result += "signing ";
			else
				result += "verification ";
			result += "using " + this.AlgorithmName + " algorithm";
			return result;
		}

	}

	/*******************************/
	/// <summary>
	/// Removes the element with the specified key from a Hashtable instance.
	/// </summary>
	/// <param name="hashtable">The Hashtable instance</param>
	/// <param name="key">The key of the element to remove</param>
	/// <returns>The element removed</returns>  
	public static System.Object HashtableRemove(System.Collections.Hashtable hashtable, System.Object key) {
		System.Object element = hashtable[key];
		hashtable.Remove(key);
		return element;
	}

	public static byte[] ToByteArray(System.IO.Stream sourceStream) {
		using(System.IO.MemoryStream ms = new System.IO.MemoryStream()) {
			int i;
			byte[] tmp = new byte[1024];
			while ((i=sourceStream.Read(tmp, 0, tmp.Length)) > 0)
				ms.Write(tmp, 0, i);
            byte[] ret = ms.ToArray();
            sourceStream.Close();
            return ret;
		} // Using kmmert sich um das Schliessen des MemoryStream
	}

	public static byte[] ToByteArray(string sourceString,string encoding) {
		return MoreSupport.ToByteArray(sourceString, encoding);
	}

	public static void ToStream(SupportClass.DigestOutputSupport str,string sourceString,string encoding) {
		MoreSupport.ToStream(str, sourceString, encoding);
	}

	public static void ToStream(Stream str,string sourceString,string encoding) {
		MoreSupport.ToStream(str, sourceString, encoding);
	}

	public static String NewString(byte [] bytes,string encoding) {
		return MoreSupport.NewString(bytes, encoding);
	}

	/*******************************/
	public class ICollectionSupport {
		/// <summary>
		/// Adds a new element to the specified collection.
		/// </summary>
		/// <param name="c">Collection where the new element will be added.</param>
		/// <param name="obj">Object to add.</param>
		/// <returns>true</returns>
		public static bool Add(System.Collections.ICollection c, System.Object obj) {
			bool added = false;
			//Reflection. Invoke either the "add" or "Add" method.
			System.Reflection.MethodInfo method;
			try {
				//Get the "add" method for proprietary classes
				method = c.GetType().GetMethod("Add");
				if (method == null)
					method = c.GetType().GetMethod("add");
				int index = (int) method.Invoke(c, new System.Object[] {obj});
				if (index >=0)	
					added = true;
			}
			catch (System.Exception e) {
				throw e;
			}
			return added;
		}

		/// <summary>
		/// Adds all of the elements of the "c" collection to the "target" collection.
		/// </summary>
		/// <param name="target">Collection where the new elements will be added.</param>
		/// <param name="c">Collection whose elements will be added.</param>
		/// <returns>Returns true if at least one element was added, false otherwise.</returns>
		public static bool AddAll(System.Collections.ICollection target, System.Collections.ICollection c) {
			System.Collections.IEnumerator e = new System.Collections.ArrayList(c).GetEnumerator();
			bool added = false;

			//Reflection. Invoke "addAll" method for proprietary classes
			System.Reflection.MethodInfo method;
			try {
				method = target.GetType().GetMethod("addAll");

				if (method != null)
					added = (bool) method.Invoke(target, new System.Object[] {c});
				else {
					method = target.GetType().GetMethod("Add");
					while (e.MoveNext() == true) {
						bool tempBAdded =  (int) method.Invoke(target, new System.Object[] {e.Current}) >= 0;
						added = added ? added : tempBAdded;
					}
				}
			}
			catch (System.Exception ex) {
				throw ex;
			}
			return added;
		}

		/// <summary>
		/// Removes all the elements from the collection.
		/// </summary>
		/// <param name="c">The collection to remove elements.</param>
		public static void Clear(System.Collections.ICollection c) {
			//Reflection. Invoke "Clear" method or "clear" method for proprietary classes
			System.Reflection.MethodInfo method;
			try {
				method = c.GetType().GetMethod("Clear");

				if (method == null)
					method = c.GetType().GetMethod("clear");

				method.Invoke(c, new System.Object[] {});
			}
			catch (System.Exception e) {
				throw e;
			}
		}

		/// <summary>
		/// Determines whether the collection contains the specified element.
		/// </summary>
		/// <param name="c">The collection to check.</param>
		/// <param name="obj">The object to locate in the collection.</param>
		/// <returns>true if the element is in the collection.</returns>
		public static bool Contains(System.Collections.ICollection c, System.Object obj) {
			bool contains = false;

			//Reflection. Invoke "contains" method for proprietary classes
			System.Reflection.MethodInfo method;
			try {
				method = c.GetType().GetMethod("Contains");

				if (method == null)
					method = c.GetType().GetMethod("contains");

				contains = (bool)method.Invoke(c, new System.Object[] {obj});
			}
			catch (System.Exception e) {
				throw e;
			}

			return contains;
		}

		/// <summary>
		/// Determines whether the collection contains all the elements in the specified collection.
		/// </summary>
		/// <param name="target">The collection to check.</param>
		/// <param name="c">Collection whose elements would be checked for containment.</param>
		/// <returns>true id the target collection contains all the elements of the specified collection.</returns>
		public static bool ContainsAll(System.Collections.ICollection target, System.Collections.ICollection c) {						
			System.Collections.IEnumerator e =  c.GetEnumerator();

			bool contains = false;

			//Reflection. Invoke "containsAll" method for proprietary classes or "Contains" method for each element in the collection
			System.Reflection.MethodInfo method;
			try {
				method = target.GetType().GetMethod("containsAll");

				if (method != null)
					contains = (bool)method.Invoke(target, new Object[] {c});
				else {					
					method = target.GetType().GetMethod("Contains");
					while (e.MoveNext() == true) {
						if ((contains = (bool)method.Invoke(target, new Object[] {e.Current})) == false)
							break;
					}
				}
			}
			catch (System.Exception ex) {
				throw ex;
			}

			return contains;
		}

		/// <summary>
		/// Removes the specified element from the collection.
		/// </summary>
		/// <param name="c">The collection where the element will be removed.</param>
		/// <param name="obj">The element to remove from the collection.</param>
		public static bool Remove(System.Collections.ICollection c, System.Object obj) {
			bool changed = false;

			//Reflection. Invoke "remove" method for proprietary classes or "Remove" method
			System.Reflection.MethodInfo method;
			try {
				method = c.GetType().GetMethod("remove");

				if (method != null)
					method.Invoke(c, new System.Object[] {obj});
				else {
					method = c.GetType().GetMethod("Contains");
					changed = (bool)method.Invoke(c, new System.Object[] {obj});
					method = c.GetType().GetMethod("Remove");
					method.Invoke(c, new System.Object[] {obj});
				}
			}
			catch (System.Exception e) {
				throw e;
			}

			return changed;
		}

		/// <summary>
		/// Removes all the elements from the specified collection that are contained in the target collection.
		/// </summary>
		/// <param name="target">Collection where the elements will be removed.</param>
		/// <param name="c">Elements to remove from the target collection.</param>
		/// <returns>true</returns>
		public static bool RemoveAll(System.Collections.ICollection target, System.Collections.ICollection c) {
			System.Collections.ArrayList al = ToArrayList(c);
			System.Collections.IEnumerator e = al.GetEnumerator();

			//Reflection. Invoke "removeAll" method for proprietary classes or "Remove" for each element in the collection
			System.Reflection.MethodInfo method;
			try {
				method = target.GetType().GetMethod("removeAll");

				if (method != null)
					method.Invoke(target, new System.Object[] {al});
				else {
					method = target.GetType().GetMethod("Remove");
					System.Reflection.MethodInfo methodContains = target.GetType().GetMethod("Contains");

					while (e.MoveNext() == true) {
						while ((bool) methodContains.Invoke(target, new System.Object[] {e.Current}) == true)
							method.Invoke(target, new System.Object[] {e.Current});
					}
				}
			}
			catch (System.Exception ex) {
				throw ex;
			}
			return true;
		}

		/// <summary>
		/// Retains the elements in the target collection that are contained in the specified collection
		/// </summary>
		/// <param name="target">Collection where the elements will be removed.</param>
		/// <param name="c">Elements to be retained in the target collection.</param>
		/// <returns>true</returns>
		public static bool RetainAll(System.Collections.ICollection target, System.Collections.ICollection c) {
			System.Collections.IEnumerator e = new System.Collections.ArrayList(target).GetEnumerator();
			System.Collections.ArrayList al = new System.Collections.ArrayList(c);

			//Reflection. Invoke "retainAll" method for proprietary classes or "Remove" for each element in the collection
			System.Reflection.MethodInfo method;
			try {
				method = c.GetType().GetMethod("retainAll");

				if (method != null)
					method.Invoke(target, new System.Object[] {c});
				else {
					method = c.GetType().GetMethod("Remove");

					while (e.MoveNext() == true) {
						if (al.Contains(e.Current) == false)
							method.Invoke(target, new System.Object[] {e.Current});
					}
				}
			}
			catch (System.Exception ex) {
				throw ex;
			}

			return true;
		}

		/// <summary>
		/// Returns an array containing all the elements of the collection.
		/// </summary>
		/// <returns>The array containing all the elements of the collection.</returns>
		public static System.Object[] ToArray(System.Collections.ICollection c) {	
			int index = 0;
			System.Object[] objects = new System.Object[c.Count];
			System.Collections.IEnumerator e = c.GetEnumerator();

			while (e.MoveNext())
				objects[index++] = e.Current;

			return objects;
		}

		/// <summary>
		/// Obtains an array containing all the elements of the collection.
		/// </summary>
		/// <param name="objects">The array into which the elements of the collection will be stored.</param>
		/// <returns>The array containing all the elements of the collection.</returns>
		public static System.Object[] ToArray(System.Collections.ICollection c, System.Object[] objects) {	
			int index = 0;

			System.Type type = objects.GetType().GetElementType();
			System.Object[] objs = (System.Object[]) Array.CreateInstance(type, c.Count );

			System.Collections.IEnumerator e = c.GetEnumerator();

			while (e.MoveNext())
				objs[index++] = e.Current;

			//If objects is smaller than c then do not return the new array in the parameter
			if (objects.Length >= c.Count)
				objs.CopyTo(objects, 0);

			return objs;
		}

		/// <summary>
		/// Converts an ICollection instance to an ArrayList instance.
		/// </summary>
		/// <param name="c">The ICollection instance to be converted.</param>
		/// <returns>An ArrayList instance in which its elements are the elements of the ICollection instance.</returns>
		public static System.Collections.ArrayList ToArrayList(System.Collections.ICollection c) {
			System.Collections.ArrayList tempArrayList = new System.Collections.ArrayList();
			System.Collections.IEnumerator tempEnumerator = c.GetEnumerator();
			while (tempEnumerator.MoveNext())
				tempArrayList.Add(tempEnumerator.Current);
			return tempArrayList;
		}
	}
	public class DateTimeFormatManager {
		static public DateTimeFormatHashTable manager = new DateTimeFormatHashTable();

		public class DateTimeFormatHashTable :System.Collections.Hashtable {
			public void SetDateFormatPattern(System.Globalization.DateTimeFormatInfo format, System.String newPattern) {
				if (this[format] != null)
					((DateTimeFormatProperties) this[format]).DateFormatPattern = newPattern;
				else {
					DateTimeFormatProperties tempProps = new DateTimeFormatProperties();
					tempProps.DateFormatPattern  = newPattern;
					Add(format, tempProps);
				}
			}

			public string GetDateFormatPattern(System.Globalization.DateTimeFormatInfo format) {
				if (this[format] == null)
					return "d-MMM-yy";
				else
					return ((DateTimeFormatProperties) this[format]).DateFormatPattern;
			}
			
			public void SetTimeFormatPattern(System.Globalization.DateTimeFormatInfo format, System.String newPattern) {
				if (this[format] != null)
					((DateTimeFormatProperties) this[format]).TimeFormatPattern = newPattern;
				else {
					DateTimeFormatProperties tempProps = new DateTimeFormatProperties();
					tempProps.TimeFormatPattern  = newPattern;
					Add(format, tempProps);
				}
			}

			public string GetTimeFormatPattern(System.Globalization.DateTimeFormatInfo format) {
				if (this[format] == null)
					return "h:mm:ss tt";
				else
					return ((DateTimeFormatProperties) this[format]).TimeFormatPattern;
			}

			class DateTimeFormatProperties {
				public string DateFormatPattern = "d-MMM-yy";
				public string TimeFormatPattern = "h:mm:ss tt";
			}
		}	
	}

	/*******************************/
	public static string FormatDateTime(System.Globalization.DateTimeFormatInfo format, System.DateTime date) {
		string timePattern = DateTimeFormatManager.manager.GetTimeFormatPattern(format);
		string datePattern = DateTimeFormatManager.manager.GetDateFormatPattern(format);
		return date.ToString(datePattern + " " + timePattern, format);            
	}

	/*******************************/
	public class Tokenizer {
		private System.Collections.ArrayList elements;
		private string source;
		//The tokenizer uses the default delimiter set: the space character, the tab character, the newline character, and the carriage-return character
		private string delimiters = " \t\n\r";		

		public Tokenizer(string source) {			
			this.elements = new System.Collections.ArrayList();
			this.elements.AddRange(source.Split(this.delimiters.ToCharArray()));
			this.RemoveEmptyStrings();
			this.source = source;
		}

		public Tokenizer(string source, string delimiters) {
			this.elements = new System.Collections.ArrayList();
			this.delimiters = delimiters;
			this.elements.AddRange(source.Split(this.delimiters.ToCharArray()));
			this.RemoveEmptyStrings();
			this.source = source;
		}

		public int Count {
			get {
				return (this.elements.Count);
			}
		}

		public bool HasMoreTokens() {
			return (this.elements.Count > 0);			
		}

		public string NextToken() {			
			string result;
			if (source == "") throw new System.Exception();
			else {
				this.elements = new System.Collections.ArrayList();
				this.elements.AddRange(this.source.Split(delimiters.ToCharArray()));
				RemoveEmptyStrings();		
				result = (string) this.elements[0];
				this.elements.RemoveAt(0);				
				this.source = this.source.Remove(this.source.IndexOf(result),result.Length);
				this.source = this.source.TrimStart(this.delimiters.ToCharArray());
				return result;					
			}			
		}

		public string NextToken(string delimiters) {
			this.delimiters = delimiters;
			return NextToken();
		}

		private void RemoveEmptyStrings() {
			//VJ++ does not treat empty strings as tokens
			for (int index=0; index < this.elements.Count; index++)
				if ((string)this.elements[index]== "") {
					this.elements.RemoveAt(index);
					index--;
				}
		}
	}
}
