namespace de.osci.helper {

	using System;
	using System.IO;
	using System.Text;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <summary> <p> Einfacher MIME-Parser. Die Funktion beschrnkt
	/// sich auf die Anforderungen der OSCI 1.2 Transportbibliothek.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class MIMEParser {

		private static Log log;
		private MIMEPartInputStream currentStream;
		internal Stream in_Renamed;
		internal String boundary;
		internal byte[] buffer;
		internal int bufferPointer;
		

		
		virtual public MIMEPartInputStream getNextStream() {
																	 String contentType;
																	 String charset;
																	 String encoding;
		String contentID;
		String contentLength;
		long length = -1;

		String bound;
		while ((bound = readLine().Trim()).Equals("")) {}

			if (bound.Equals("--" + boundary + "--"))
			{
				in_Renamed.Close();
				return null;
			}
			else if (!bound.Equals("--" + boundary))
				throw new IOException(DialogHandler.text.getString("msg_format_error"));
		System.Collections.Hashtable header = readHeaders();

		if (header == null) {
//			System.Console.WriteLine("\nCLOSE HH "+in_Renamed);
			in_Renamed.Close();

		return null;
	}

	if (header["content-type"] == null)
	throw new IOException(DialogHandler.text.getString("msg_format_error"));

	System.Collections.Hashtable cntTyp = parseHeader("content-type", (String) header["content-type"]);
	contentType = (String) cntTyp["content-type"];

	if (cntTyp["charset"] != null) {
	charset = (String) cntTyp["charset"];

	if (!charset.ToUpper().Equals(Constants.CHAR_ENCODING))
		throw new Exception(DialogHandler.text.getString("invalid_charset") +charset);
}

	encoding = (String) header["content-transfer-encoding"];
	contentID = (String) header["content-id"];

	if (contentID.StartsWith("<"))
	contentID = contentID.Substring(1, contentID.Length - 2);

	contentLength = (String) header["content-length"];

	if (contentLength != null)
	length = long.Parse(contentLength);

	currentStream = new MIMEPartInputStream(this, contentType, encoding, contentID, length);
	currentStream.mime_headers = header;

	return currentStream;
}

  private System.Collections.Hashtable readHeaders()
  {
    bool end = false;
    System.Collections.Hashtable headers = new System.Collections.Hashtable();
    String header;
	String nextHeader;

	  //      header = readLine();
      while ((header = readLine().Trim()).Equals("")){}

      while (!header.Equals(""))
      {
		nextHeader = readLine();
  	 
		while ((nextHeader.Length > 0) && Char.IsWhiteSpace(nextHeader[0]))
		{
			header = header + nextHeader;
			nextHeader = readLine();
		}        
		int i=header.IndexOf(':');
		SupportClass.PutElement(headers,header.Substring(0, i).ToLower().Trim(),
			header.Substring(i + 1).Trim());
        header = nextHeader.Trim();
      }
	  if (headers.Count == 0)
		  return null;

	  return headers;
  }
  private String readLine()
  {
    int b = 0;
    System.IO.MemoryStream baos = new System.IO.MemoryStream();
    byte[] ba = new byte[1];
    bool end = false;
//	System.Text.StringBuilder sb=new System.Text.StringBuilder();
    while (!end)
    {
      b = readFromInput();

      if (b == -1)
        return null;

      if (b == 0x0D)
      {
        b = readFromInput();

        if (b == 0x0A)
          end = true;
        else if (Char.IsWhiteSpace((char) (b)))
          continue;
      }

		ba[0] = (byte)b;
		if (!end)
			baos.Write(ba,0,1);

    }
	
	  byte[] tb = baos.ToArray();
	  String txt = MoreSupport.NewString(tb, 0, tb.Length);

	  if (log.DebugEnabled)
		log.debug("ZEILE: " + txt);

		return txt;
  }

	private int readFromInput(){
			int j = 0;
		if ((buffer != null) && (bufferPointer < buffer.Length)) {
			j = buffer[bufferPointer++];

			if (bufferPointer >= buffer.Length) {
			buffer = null;
			bufferPointer = 0;
		}
			return j;
	}
	else{
	 byte[] buf=new byte[1];
		 j = in_Renamed.Read(buf,j,1);
			return (int)buf[0];
		}
	}
/*  private int readFromInput()
  {
    int j = 0;

    if ((buffer != null) && (bufferPointer < buffer.Length))
    {
      j = buffer[bufferPointer++];

      if (bufferPointer >= buffer.Length)
      {
        buffer = null;
        bufferPointer = 0;
      }
    }
    else
      j = in_Renamed.Read(buffer,j,1);

    return j;
  }
*/
  private System.Collections.Hashtable parseHeader(String field_name, String header)
  {
    System.Collections.Hashtable lparams = new System.Collections.Hashtable();
    //StringTokenizer ct = new StringTokenizer(header, "; ");
	 String[] ct=header.Split(';');
	  //	  SupportClass.PutElement(lparams,field_name, ct.nextToken().Trim());
	  	  SupportClass.PutElement(lparams,field_name, ct[0].Trim());

    String[] tmp;

	char[] sep = new char[]{'='};
	for(int i=1;i<ct.Length;i++)
	{
		tmp = ct[i].Split(sep, 2);
		if (tmp[1].StartsWith("\"") && tmp[1].EndsWith("\""))
			tmp[1] = tmp[1].Substring(1, tmp[1].Length - 2);
		SupportClass.PutElement(lparams,tmp[0].Trim(), tmp[1].Trim());
	}

    return lparams;
  }

virtual public MIMEPartInputStream getNextStream0(){
			System.String contentType;
			System.String charset;
			System.String encoding;
			System.String contentID;
			System.String contentLength;
			int length = - 1; // muss berprft werden (evtl. auf 0 setzen, da ReturnCodes in Read-Methode anstelle von -1 die 0 in .NET zurckgeben)
			contentType = eval("Content-Type: ", "\r\n");
			if (contentType == null)
				return null;
			string [] st=contentType.Split(';');
			contentType =st[0];

			System.Collections.Hashtable param = new System.Collections.Hashtable();
			String[] tmp;
			int z=1;
			while (z< st.Length) {
				tmp=st[z].Split('=');
				param[tmp[0].Trim()] = tmp[1].Trim();
				z++;
			}

			if (param["charset"] != null) {
				charset = (String) param["charset"];

				if (!charset.ToUpper().Equals(Constants.CHAR_ENCODING))
					throw new Exception(DialogHandler.text.getString("invalid_charset") +
						charset);
			}

			if ((contentType.ToLower().Equals("x-osci/attachment")) && (param["type"] != null))
				contentType = (String) param["type"];

			encoding = eval("Content-Transfer-Encoding: ", "\r\n");
			contentID = eval("Content-ID: <", ">");
			contentLength = eval("\r\n", "\r\n");
				
			if (contentLength.Length > 0) {
				int i;
					
				if ((i = contentLength.IndexOf("Content-Length: ")) > - 1)
					length = System.Int32.Parse(contentLength.Substring(i + 16));
				eval("\r\n", null);
			}
				
			currentStream = new MIMEPartInputStream(this, contentType, encoding, contentID, length);
				
			return currentStream;
		}
	/*	
		public MIMEParser(System.IO.Stream in_Renamed) {
			this.in_Renamed = in_Renamed;
			boundary = eval("boundary=", ";");
			
			if (boundary == null)
				throw new System.IO.IOException("Keine MIME-Trenner Definition gefunden !");
		}
*/
		public MIMEParser(Stream in_Renamed){
			this.in_Renamed = in_Renamed;

		System.Collections.Hashtable header = readHeaders();

		if ((header["mime-version"] != null) && (!header["mime-version"].Equals("1.0")))
		throw new IOException(DialogHandler.text.getString("msg_format_error"));

		System.Collections.Hashtable contentType = parseHeader("content-type", (String) header["content-type"]);

		if ((boundary = (String) contentType["boundary"]) == null)
		throw new IOException(DialogHandler.text.getString("msg_format_error"));

	}

		private System.String eval(System.String start, System.String stop) {
			byte[] txt = SupportClass.encoder.GetBytes(start);

			int i;
			int j = 0;
			
			do {
				for (i = 0; i < txt.Length; i++) {
					if ((buffer != null) && (bufferPointer < buffer.Length)){
						j = buffer[bufferPointer++];
						if (j == 0x22)	j = buffer[bufferPointer++];}
					else{
						j = in_Renamed.ReadByte();
							if (j == 0x22) j = in_Renamed.ReadByte();}
					if (txt[i] != (byte) j) {
						break;
					}
				}
			}
			while ((i < txt.Length) && (j > 0));
			
			if (j <1) {
				return null; // Stream enthlt nicht den gesuchten String
			}
			
			if (stop != null) {
				System.IO.MemoryStream sb = new System.IO.MemoryStream();
				txt = SupportClass.encoder.GetBytes(stop);

				int len = txt.Length;
				j = 0;
				
				do {
					for (i = 0; i < txt.Length; i++) {
						if ((buffer != null) && (bufferPointer < buffer.Length)){
							j = buffer[bufferPointer++];
							if (j == 0x22)	j = buffer[bufferPointer++];}
						else{
							j = in_Renamed.ReadByte();
							if (j == 0x22) j = in_Renamed.ReadByte();}
						sb.WriteByte((byte) j);
						
						if (txt[i] != (byte) j) {
							break;
						}
					}
				}
				while ((i < txt.Length) && (j > 0));
				
				if (j <1) {
					throw new System.ArgumentException("Stream enthlt nicht den gesuchten String: " + stop);
				} 
				else {
					txt = sb.ToArray();
				}

				char[] tmpChar;
				tmpChar = new char[txt.Length];
				tmpChar = SupportClass.ToCharArray(txt);

				System.String data = new System.String(tmpChar, 0, txt.Length - len);
				log.debug("RETURN " + data);

				return data;
			} 
			else {
				return null;
			}
		}
		
		[STAThread]
		public static void  Main0() {
			String filename = "./../../../osci-bibliothek-net-test/Daten/Testdaten/StoreDelivery2.osci";
			System.IO.FileInfo fi = new System.IO.FileInfo(filename);
			System.IO.FileStream fs = fi.OpenRead();
			MIMEParser mp = new MIMEParser(fs);
			de.osci.helper.MIMEPartInputStream inp;
				
			while ((inp = mp.getNextStream()) != null) {
				byte[] tmp = new byte[8096];
				int i;
				System.IO.MemoryStream baos = new System.IO.MemoryStream();
					
				while ((i = SupportClass.ReadInput(inp, ref tmp, 0, tmp.Length)) > 0) {
					baos.Write(tmp, 0, i);
				}
					
				char[] tmpChar;
				tmpChar = SupportClass.ToCharArray(baos.ToArray());
	//			System.Console.Out.WriteLine("\nSTREAM:\n" + new System.String(tmpChar));
			}
		}

		static MIMEParser() {
			log = new Log(typeof(MIMEParser));
		}
	}
}