namespace de.osci.helper {

	using System;
	using de.osci.osci12.dotcommon;
	using org.apache.commons.logging;
	 
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class StoreInputStream:InputStream {

		private static Log log;
		private System.IO.Stream baseStream; // InputStream
		private System.IO.Stream copyStream; // OutputStream
		private System.IO.MemoryStream buffer;
		private bool closed = false, save = false;
		private int s;

		public StoreInputStream(System.IO.Stream inStream, System.IO.Stream copyStream):base() {
			this.baseStream = inStream;
			this.copyStream = copyStream;
			buffer = new System.IO.MemoryStream();
		}

		virtual public bool Save {
			set {
//				if (value)
//					return ;
				
				lock (baseStream) {
					this.save = value;
					if (value) {
						buffer.Close();
						byte[] temp_byteArray = buffer.ToArray();
						copyStream.Write(temp_byteArray, 0, temp_byteArray.Length);
					}
					buffer = null;
				}
			}
		}

		public int Read() {
			if (closed)
				return - 1;
			s = (baseStream).ReadByte();
			if (s == - 1) {
				return - 1;//Close();
			}
			else {
				if (buffer != null)
					buffer.WriteByte((System.Byte) s);
				else if (save)
					copyStream.WriteByte((System.Byte) s);
			}
			return s;
		}
		
		public override int Read(byte[] b, int off, int len) {
			if (closed)
				return - 1;
			s = SupportClass.ReadInput(baseStream, ref b, off, len);
			
			if (s == - 1) {
				return - 1;//Close();
			}
			else {
				if (buffer != null)
					buffer.Write(b, off, s);
				else if (save)
					copyStream.Write(b, off, s);
			}
			return s;
		}
				
		public override void  Close() {
			if (closed)
				return ;
			(baseStream).Close();
			if (buffer != null) {
				buffer.Close();
				byte[] temp_byteArray = buffer.ToArray();
				copyStream.Write(temp_byteArray, 0, temp_byteArray.Length);
			}
			copyStream.Close();
			closed = true;
		}

		static StoreInputStream() {
			log = LogFactory.getLog(typeof(StoreInputStream));
		}
	}
}