namespace de.osci.helper {

	using System;
	using org.apache.commons.logging;
	using javax.crypto;
	using de.osci.osci12.common;
	using de.osci.osci12.encryption;
	using System.Security.Cryptography;
	using de.osci.osci12.dotcommon;

	/// <summary> Diese Erweiterung der Klasse java.io.FilterInputStream fhrt eine 2-Key-TripleDES-
	/// Ver-/Entschlsselung der gelesenen Daten im CBC-Mode durch. Der Initialisierungsvektor
	/// wird gem der XML-Encryption-Spezifikation den Inhaltsdaten hinzugefgt bzw.
	/// entnommen.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.helper.SymCipherOutputStream">
	/// </seealso>
	public class SymCipherInputStream:InputStream {
		private static Log log;
		private System.Security.Cryptography.SymmetricAlgorithm cipher;
		private String cipherName;
		private int count = 0;
		internal byte[] iv;
		private bool encrypt;
		private  CryptoStream baseStream;
		private byte[] b = new byte[1];
			
		public SymCipherInputStream(System.IO.Stream inStream, SecretKey symKey, bool encrypt):base() {
			this.encrypt = encrypt;
			if (inStream == null)
				throw new System.ArgumentException("inStream darf nicht null sein !");
			if (symKey == null)
				throw new System.ArgumentException("Es wurde kein symmetrischer Schlssel bergeben (symKey ist null).");
			cipher = symKey.GetAlgo();
			if (encrypt) {
				cipher.GenerateIV();
				iv = cipher.IV;
				if (symKey == null)
					symKey = de.osci.osci12.encryption.Crypto.createSymKey("http://www.w3.org/2001/04/xmlenc#tripledes-cbc");
			} else {
				if ((symKey.GetAlgoString().StartsWith("DESede")) | ("http://www.w3.org/2001/04/xmlenc#tripledes-cbc".Equals(symKey.GetAlgoString())))
					iv = new byte[8];
				else
					iv = new byte[16];
				inStream.Read(iv, 0, iv.Length);				
			}
			System.Security.Cryptography.ICryptoTransform ct= cipher.CreateDecryptor(symKey.Encoded(), iv);
			baseStream = new CryptoStream(inStream, ct, CryptoStreamMode.Read);			
		}

		public   int Read() {
//			byte[] b = new byte[1];
			if (Read( b, 0, b.Length) == 0)
				return 0;
			return b[0];
		}
		
		public override int Read(byte[] b, int off, int len) {
			if (encrypt && (count < iv.Length)) {
				if (len < iv.Length - count) {
					Array.Copy(iv, count,b, off, len);
					count += len;
					return len;
				} else {
					Array.Copy(iv, count,b, off, iv.Length - count);
					int ret = SupportClass.ReadInput((System.IO.Stream) baseStream, ref b, off + iv.Length - count, len - iv.Length + count);
					ret += iv.Length - count;
					count = iv.Length;
					return ret;
				}
			} else {
				return baseStream.Read( b, off, len);
			}
		}

		static SymCipherInputStream() {
			log = LogFactory.getLog(typeof(SymCipherInputStream));
		}
	}
}