namespace de.osci.helper {

	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.encryption;
	using org.apache.commons.logging;
	using javax.crypto;
	using System.Security.Cryptography;
	using de.osci.osci12.dotcommon;

	/// <summary> Diese Erweiterung der Klasse java.io.FilterOutputStream fhrt eine 2-Key-TripleDES-
	/// Ver-/Entschlsselung der geschriebenen Daten im CBC-Mode durch. Der Initialisierungsvektor
	/// wird gem der XML-Encryption-Spezifikation den Inhaltsdaten hinzugefgt bzw.
	/// entnommen.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.helper.SymCipherInputStream">
	/// </seealso>
	public class SymCipherOutputStream:OutputStream {
		virtual public byte[] Iv {			
			get {
				return iv;
			}
		}

		private static Log log;
		private System.Security.Cryptography.SymmetricAlgorithm cipher;
		private int index = 0;
		private byte[] iv;
		private SecretKey symKey;
		private bool encrypt;
		private sbyte[] tmp;

		private System.Security.Cryptography.CryptoStream baseStream;
		private System.IO.Stream outStream;

		public SymCipherOutputStream(System.IO.Stream outStream, SecretKey symKey, byte[] iv):this(outStream, symKey, true, iv) {
		}
		
		public SymCipherOutputStream(System.IO.Stream outStream, SecretKey symKey, bool encrypt):this(outStream, symKey, encrypt, null) {
		}
		
		internal SymCipherOutputStream(System.IO.Stream outStream, SecretKey symKey, bool encrypt, byte[] iv) {//:base(outStream)
			this.symKey = symKey;
			this.encrypt = encrypt;
			this.iv = iv;
			if (encrypt) 
			{			
				if (symKey == null) 
				{
					symKey = de.osci.osci12.encryption.Crypto.createSymKey("DESede");
				}
				SymmetricAlgorithm sa=de.osci.osci12.encryption.Crypto.createSymmetricAlgorithm(symKey.GetAlgoString());
				if (this.iv == null) {						
					sa.GenerateIV();
					this.iv = sa.IV;
				}
				outStream.Write(this.iv, 0, this.iv.Length);
				baseStream = new CryptoStream(outStream, sa.CreateEncryptor(symKey.Encoded(), this.iv), CryptoStreamMode.Write);
			} else {
				if (symKey.GetAlgoString().StartsWith(Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC))
					this.iv = new byte[8];
				else
					this.iv = new byte[16];
			}
		}

		public void  write(int b) {
			Write(new byte[] { (byte) b }, 0, 1);
		}
		
		public  override void  Write(System.Byte[] b, int off, int len) {
			if ((off | len | (b.Length - (len + off)) | (off + len)) < 0)
				throw new System.IndexOutOfRangeException();
			
			if ((!encrypt) && (index < iv.Length)) {
				if (len < (iv.Length - index)) {
					for (int i = 0; i < len; i++)
						iv[index++] = b[off + i];
				} else {
					int i;
					
					for (i = 0; i < (iv.Length - index); i++)
						iv[i + index] = b[off + i];
					
					index += i;
					
					baseStream = new CryptoStream(outStream, cipher.CreateDecryptor(symKey.Encoded(), iv), CryptoStreamMode.Write);
					baseStream.Write(b, off + i, len - i);
				}
			} else {
				baseStream.Write(b, off, len);
			}
		}
				
		public override void Close() {	
			baseStream.FlushFinalBlock();
		}	
	
		static SymCipherOutputStream() {
			log = LogFactory.getLog(typeof(SymCipherOutputStream));
		}
	}
}