namespace de.osci.helper {

	using System;
	using System.IO;
	using org.apache.commons.logging;
	using java.security.cert;
	using java.security;
 
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Tools {
		private static Log log;


		public static bool compareByteArrays(byte[] a, byte[] b)
		{
			if (a.Length != b.Length)
				return false;

			for (int i = 0; i < a.Length; i++)
			{
				if (a[i] != b[i])
					return false;
			}

			return true;
		}


		/// <summary> Erzeugen eines Zertifikates.
		/// </summary>
		/// <param name="certificate">Zertifikate in seiner codierten Form als Stream
		/// </param>
		/// <returns> X509Certificate
		/// @throws CryptographicException
		/// </returns>
		public static X509Certificate createCertificate(System.IO.Stream certificate) 
		{
			if (certificate == null) {
				log.debug("Kein Input Stream! (null)");
			}
			return new X509Certificate(SupportClass.ToByteArray(certificate));
		} 
		/// <summary> Erzeugen eines Zertifikates.
		/// </summary>
		/// <param name="certificate">Zertifikate in seiner codierten Form als byte[]
		/// </param>
		/// <returns> X509Certificate
		/// @throws CryptographicException
		/// </returns>
		public static X509Certificate createCertificate(byte[] certificate) {
				// MW:
				using (System.IO.Stream strm = new System.IO.MemoryStream(certificate)) {
					return createCertificate(strm);
				} // kmmert sich um das Schliessen des Streams

		} // createCertificate(...)

		/// <summary> Erzeugen eines Zertifikates.
		/// </summary>
		/// <param name="path">Path des einzulesenden Zertifikates
		/// </param>
		/// <returns> X509Certificate
		/// @throws CryptographicException
		/// </returns>
		public static X509Certificate createCertificate(String path) 
		{
				using (System.IO.FileStream fs = new System.IO.FileStream(path, System.IO.FileMode.Open, System.IO.FileAccess.Read)) {
					return createCertificate(fs);
				} // kmmert sich um das Schliessen des FileStream

		} // createCertificate(...)

		public static java.security.cert.KeyStore createKeyStore(System.IO.Stream keyStoreStream, char[] password) {
   			if (keyStoreStream == null || SupportClass.available(keyStoreStream) == 0) {
					throw new System.Exception("InputStream ist nicht vorhanden");
				}
				
				java.security.cert.KeyStore keyStore = KeyStore.getInstance("PKCS12");
				keyStore.load(keyStoreStream, null);
				return keyStore;
		}
		
		public static SupportClass.KeyPairSupport createKeyPair() {
				KeyPairGenerator keyGen = KeyPairGenerator.getInstance("DSA");
				keyGen.initialize(1024); 
				SupportClass.KeyPairSupport keyPair = keyGen.genKeyPair();
				return keyPair;
		}
		
		public static byte[] readBytes(System.IO.Stream in_Renamed) {
			System.IO.MemoryStream bos = new System.IO.MemoryStream();
			byte[] bytes = new byte[1024];
			int count = 0;
			while ((count = SupportClass.ReadInput(in_Renamed, ref bytes, 0, bytes.Length)) > 0) {
				bos.Write(bytes, 0, count);
			}
            in_Renamed.Close();
            return bos.ToArray();
		}
		
		public static byte[] createRandom(System.String algorithm, int byteCount) {
			byte[] bytes = null;
			bytes = new byte[byteCount];
			SupportClass.SecureRandomSupport.getInstance(algorithm).nextBytes(bytes);
			return bytes;
		}
				
		public static System.String createRandom(int length) {
			SupportClass.SecureRandomSupport random = SupportClass.SecureRandomSupport.getInstance("SHA1PRNG");
			byte[] bytes = new byte[length];
			bytes = random.NextBytes(bytes);
			return Base64.encode(bytes);
		}

		public static void showCertificate(X509Certificate certificate) 
        {
			try {
				System.Console.WriteLine("----------------------------------------------------------------------");
                System.Console.WriteLine("Cert: Subject                         " + certificate.SubjectDN);
                System.Console.WriteLine("Cert: Issuer                          " + certificate.IssuerDN());
                System.Console.WriteLine("----------------------------------------------------------------------");

			} catch(Exception ex) {
				log.error("Fehler: ", ex);			
			}
		} // showCertificate(...)

		static Tools() {
			log = LogFactory.getLog(typeof(Tools));
		}
	}
}