namespace de.osci.osci12.common {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.extinterfaces;

	/// <summary> Diese Klasse stellt die Standard-Implementierung der abstrakten
	/// OSCIDataSource-Klasse dar. Diese Implementierung puffert Inhaltsdaten
	/// bis zu einer konfigurierbaren Anzahl von Bytes im Arbeitsspeicher.
	/// Wird diese Anzahl berschritten, werden die gepufferten Bytes wie alle
	/// folgenden Bytes in eine temporre Datei geschrieben.<p></p><p></p>
	/// Zur Dokumentation der Methoden s. OSCIDataSource.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.extinterfaces.OSCIDataSource">
	/// </seealso>
	public class SwapBuffer:OSCIDataSource {

		/// <summary>
		/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
		/// <p>Erstellt von Governikus GmbH & Co.  KG

		/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
		/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
		/// unter der URL www.osci.de abgerufen oder bei der
		/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
		/// 28195 Bremen schriftlich angefordert werden.</p>
		/// 
		/// <p>Author: P. Ricklefs, N. Bngener</p> 
		/// <p>Version: 1.6</p>
		/// </summary>
		public class MStream:System.IO.MemoryStream {
			public MStream():base() {
			}

			public override void Close() {
				this.Seek(0,System.IO.SeekOrigin.Begin);
			}
		}
		
		private static Log log;
		private MStream os;
		private bool isFirstCall = true;

		public SwapBuffer() {
			os = new MStream();
		}

		override public System.IO.Stream OutputStream {
			get {
				return os;
			}
		}

		override public System.IO.Stream InputStream {
			get {
				return os;
			}			
		}

		/// <summary> Liefert die Anzahl der (momentan) gespeicherten Bytes.
		/// </summary>
		/// <value> Anzahl der Bytes
		/// </value>
		override public long Length {
			get {
				return os.Length;
			}
		}
	
		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		override public System.String Vendor {
			get {
				return "BOS";
			}
		}

		/// <summary> Liefert die Versionsnummer
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		override public System.String Version {
			get {
				return "0.1";
			}
		}

		public override OSCIDataSource newInstance() {
			return new SwapBuffer();
		}
		public void close()
		{ 
			os.Close(); 
		} 
	}
}