//******************************************************************************
//
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
// EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
//
// Copyright (C) 1999- 2002.  Microsoft Corporation.  All rights reserved.
//
//******************************************************************************
//
// This is a C# sample that illustrates how to use features introduced in
// CAPICOM's bridging support to interop between CAPI and System.Security.xml.
//
// Note: for simplicity, this code does not handle exceptions.
//
//******************************************************************************

namespace de.osci.osci12.dotcommon {
	
	using System;
	using System.IO;
	using System.Security.Cryptography;
	using System.Text;
	using System.Security.Cryptography.X509Certificates;
	using System.Security.Cryptography.Xml;
	using System.Xml;
	using java.security.cert;
	using System.Runtime.InteropServices;

	public class RSAEncryption {
		#region "Private methods"

		private static byte[] ConvertIntToByteArray(uint dwInput) {
			// output of this routine is always big endian
			byte[] rgbTemp = new byte[4]; 
			uint t1, t2;

			if (dwInput == 0) return new byte[1]; 
			t1 = dwInput; 
			int i = 0;
			while (t1 > 0) {
				t2 = t1 % 256;
				rgbTemp[i] = (byte) t2;
				t1 = (t1 - t2)/256;
				i++;
			}
			// Now, copy only the non-zero part of rgbTemp and reverse
			byte[] rgbOutput = new byte[i];
			// copy and reverse in one pass
			for (int j = 0; j < i; j++) {
				rgbOutput[j] = rgbTemp[i-j-1];
			}
			return(rgbOutput);
		}
		
		#endregion 

		#region "Public methods"

        static public java.security.cert.X509Certificate GetX509Certificate(X509Certificate2 cert)
        {
            byte[] buffer = cert.GetRawCertData();
            return new java.security.cert.X509Certificate(buffer);
        }

        static public void GetRSACryptoServiceProviderPublicKey(out RSACryptoServiceProvider rsa, java.security.cert.X509Certificate cert)
        {
            X509Certificate2 netCert = new X509Certificate2(cert.GetRawCertData());

            if (!(netCert.PublicKey.Key is RSACryptoServiceProvider))
            {
                throw new Exception("Certificate must support RSA.");
            }

            rsa = (RSACryptoServiceProvider)netCert.PublicKey.Key;
		}

		public static void GetRSACryptoServiceProviderPrivateKey(out RSACryptoServiceProvider rsa, SupportClass.PrivateKeySupport pKeyInfo) {
			rsa = null;
				CspParameters csp = new CspParameters();
				csp.KeyContainerName = pKeyInfo.containerName;
				csp.ProviderName = pKeyInfo.providerName;
				csp.ProviderType = pKeyInfo.providerType;
				csp.KeyNumber = pKeyInfo.keySpec;
				if (pKeyInfo.isMachineKeyset)
					csp.Flags = CspProviderFlags.UseMachineKeyStore;
				rsa = new RSACryptoServiceProvider(csp);
		}

		public static void LoadFromStore(out X509Certificate2 x509Cert, 
            string certPath, 
            string pin, 
            bool privateKeyStore, 
            out SupportClass.PrivateKeySupport pKeyInfo) 
        {
		    LoadFromStore(out x509Cert, certPath,  pin,  privateKeyStore, out pKeyInfo, "MY", "BOS Capicom Sample", "choose certificate");
		}

		public static void LoadFromStore(out java.security.cert.X509Certificate x509Cert, 
            string certPath, 
            string pin, 
            bool privateKeyStore, 
            out SupportClass.PrivateKeySupport pKeyInfo) 
        {
            X509Certificate2 ret;
		    LoadFromStore(out ret, certPath,  pin,  privateKeyStore, out pKeyInfo, "MY", "BOS Capicom Sample", "choose certificate");
            x509Cert = GetX509Certificate(ret);
		}

		public static void LoadFromStore(out X509Certificate2 x509Cert, 
            string certPath, 
            string pin, 
            bool privateKeyStore,
            out SupportClass.PrivateKeySupport pKeyInfo,
            string StoreName,
            string Title,
            string Display) 
        {
			x509Cert = null;
            X509Certificate2 selectedCert = null;
            pKeyInfo = null;

			if(certPath.Length==0) 
            {
                X509Store st = new X509Store(StoreName,
                    StoreLocation.CurrentUser);
				
                st.Open(OpenFlags.MaxAllowed);

                X509Certificate2Collection selectedCerts =
                    X509Certificate2UI.SelectFromCollection(st.Certificates,
                        Title,
                        Display,
                        X509SelectionFlag.SingleSelection);

            	selectedCert = (X509Certificate2)selectedCerts[0];
			}
			else {
				selectedCert = new X509Certificate2();
                selectedCert.Import(certPath,
                    pin,
//                    X509KeyStorageFlags.UserKeySet);
                    X509KeyStorageFlags.Exportable);
            }
            String algo = selectedCert.GetKeyAlgorithm();
            
			// This only works with RSA keys
            if (!algo.Equals("RSA") && !algo.Equals("1.2.840.113549.1.1.1"))
            {
				throw new ApplicationException("choose certificate algorithm (rsa)");
			}

			x509Cert = selectedCert;

			if (privateKeyStore) {
				if(!selectedCert.HasPrivateKey)
					throw new ApplicationException("no private key");
				else 
                {
                    pKeyInfo = new SupportClass.PrivateKeySupport();
                    pKeyInfo.containerName = ((RSACryptoServiceProvider)selectedCert.PrivateKey).CspKeyContainerInfo.KeyContainerName;
                    pKeyInfo.providerName = ((RSACryptoServiceProvider)selectedCert.PrivateKey).CspKeyContainerInfo.ProviderName;
                    pKeyInfo.providerType = ((RSACryptoServiceProvider)selectedCert.PrivateKey).CspKeyContainerInfo.ProviderType;
                    pKeyInfo.keySpec = (int)((RSACryptoServiceProvider)selectedCert.PrivateKey).CspKeyContainerInfo.KeyNumber;
				}
			}
		}

		#endregion
	}
}
