namespace de.osci.osci12.encryption {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.extinterfaces;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class CipherReference 
	{

		// Transform ist optional und wird zunchst nicht untersttzt.
		private void  InitBlock() {
			transformList = new System.Collections.ArrayList();
		}

		virtual public System.IO.Stream ReferencedStream {
			get {return swapBuffer.InputStream;
			}
			set {
				log.debug("Der Referenced Stream wurde gesetzt.");
				System.IO.Stream out_Renamed = swapBuffer.OutputStream;
				byte[] bytes = new byte[1024];
				int anz = 0;
				while ((anz = SupportClass.ReadInput(value, ref bytes, 0, bytes.Length)) > 0) {
					out_Renamed.Write(bytes, 0, anz);
				}
			}			
		}

		virtual public System.String URI {
			get {
				return uri;
			}			
			set {
				this.uri = value;
			}
		}

		virtual public bool EncryptedStream {
			get {
				return encryptedStream;
			}
			set {
				this.encryptedStream = value;
			}
		}

		private static Log log;
		private System.Collections.ArrayList transformList;
		private System.String uri;
		private OSCIDataSource swapBuffer = null;
		private bool encryptedStream = false;
		
		public CipherReference(System.String uri) {
			InitBlock();
			this.uri = uri;
			log.debug("Konstruktor");
			swapBuffer = DialogHandler.NewDataBuffer;
		}
		
		public virtual void  addTransform(System.String transform) {
			transformList.Add(transform);
		}
		
		public virtual de.osci.osci12.signature.Transform getTransform(int i) {
			return (de.osci.osci12.signature.Transform) transformList[i];
		}
		
		public virtual void  writeXML(System.IO.Stream out_Renamed, String ds, String xenc) {
			MoreSupport.ToStream(out_Renamed,"<" + xenc + ":CipherReference URI=\"" + this.URI + "\">",de.osci.osci12.common.Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + xenc + ":Transforms><" + ds + ":Transform Algorithm=\"http://www.w3.org/2000/09/xmldsig#base64\"></" + ds + ":Transform></" + xenc + ":Transforms>",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"</" + xenc + ":CipherReference>",de.osci.osci12.common.Constants.CHAR_ENCODING);
		}
		
		// @todo diese Methode sollte heraus
		public override System.String ToString() {
			System.Text.StringBuilder buffer = new System.Text.StringBuilder();
			buffer.Append("<CipherReference URI=\"" + this.URI + "\" >");
			// Transformer auflisten
			buffer.Append("</CipherReference>");
			
			return buffer.ToString();
		}		
		
		static CipherReference() {
			log = LogFactory.getLog(typeof(CipherReference));
		}
	}
}