namespace de.osci.osci12.encryption {

	using System;
	using javax.crypto;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using System.Security.Cryptography;

    using Org.BouncyCastle.Crypto;
    using Org.BouncyCastle.Crypto.Encodings;
    using Org.BouncyCastle.Crypto.Engines;
    using Org.BouncyCastle.Crypto.Digests;
    using Org.BouncyCastle.Security;    

    /// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Crypto 
	{

		private static String hex = "0123456789abcdef";
		private static Log log;
		
		// Asymetrische Verfahren
		//public final static String RSA15 = "http://www.w3.org/2001/04/xmlenc#rsa-1_5";
		
		// Symetrische Verfahen
		//public final static String TDES ="http://www.w3.org/2001/04/xmlenc#tripledes-cbc";
		//public final static String AES128 ="http://www.w3.org/2001/04/xmlenc#aes128-cbc";
		//public final static String AES192 ="http://www.w3.org/2001/04/xmlenc#aes192-cbc";
		//public final static String AES256 ="http://www.w3.org/2001/04/xmlenc#aes256-cbc";

		public static System.Security.Cryptography.SymmetricAlgorithm createSymmetricAlgorithm(String keyname) {
			System.Security.Cryptography.SymmetricAlgorithm sa;
			if (keyname.Equals("http://www.w3.org/2001/04/xmlenc#tripledes-cbc")) {
			 sa = new TripleDESCryptoServiceProvider();}
			else if (keyname.Equals("http://www.w3.org/2001/04/xmlenc#aes128-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=128;}
			else if (keyname.StartsWith("http://www.w3.org/2001/04/xmlenc#aes192-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=192;}
			else if (keyname.StartsWith("http://www.w3.org/2001/04/xmlenc#aes256-cbc")) {
			 sa = new RijndaelManaged();sa.KeySize=256;}
			else{throw(new Exception());}
			return sa;
		}
		public static SecretKey createSymKey(String keyname) {
			System.Security.Cryptography.SymmetricAlgorithm sa=Crypto.createSymmetricAlgorithm(keyname);
			sa.GenerateKey();
			return new SecretKey(sa.Key,keyname);
		} 

		public static javax.crypto.SecretKey createSymKey(byte[] data, String algorithm) {
			return new SecretKey(data, algorithm);
		}

		public static String toHex(byte[] b) {
			System.Text.StringBuilder s = new System.Text.StringBuilder();
			int j;
			for (int i = 0; i < b.Length; i++) {
				// hi-nibble
				j = (b[i] >> 4) & 0xf;
				s.Append(hex[j]);
				// lo-nibble
				j = b[i] & 0xf;
				s.Append(hex[j]);
			}
			return s.ToString();
		}

        public static byte[] doRSADecryption(SupportClass.PrivateKeySupport key, byte[] data)
        {
            if (data.Length <= 0)
            {
                log.error("Es wurden keine Daten bergeben.");
                return null;
            }

            RSA rsa = new de.osci.osci12.dotcommon.RSA_Cipher(key);
            rsa.DecryptValue(data);
            byte[] cipheredData = rsa.DecryptValue(data);
            return cipheredData;
        }

        public static byte[] doRSADecryption(AsymmetricKeyParameter key, byte[] data, String mfgAlgorithm, String digestAlgorithm)
        {
            if (data.Length <= 0)
            {
                log.error("Es wurden keine Daten bergeben.");
                return null;
            }

            if (mfgAlgorithm == null)
            {
                Pkcs1Encoding pkcs12 = new Pkcs1Encoding(new RsaEngine());
                pkcs12.Init(false, key);
                return pkcs12.ProcessBlock(data, 0, data.Length);
            }
            else
            {
                IDigest gd = null;
                if (digestAlgorithm.Equals(Constants.DIGEST_ALGORITHM_SHA256))
                    gd = new Sha256Digest();
                else if (digestAlgorithm.Equals(Constants.DIGEST_ALGORITHM_SHA512))
                    gd = new Sha512Digest();

                IDigest gd_mgf = null;
                if (mfgAlgorithm.Equals(Constants.MASK_GENERATION_FUNCTION_1_SHA256))
                    gd_mgf = new Sha256Digest();
                else if (mfgAlgorithm.Equals(Constants.MASK_GENERATION_FUNCTION_1_SHA512))
                    gd_mgf = new Sha512Digest();

                OaepEncoding oaep = new OaepEncoding(new RsaEngine(), gd, gd_mgf, new byte[0]);
                oaep.Init(false, key);

                return oaep.ProcessBlock(data, 0, data.Length);
            }
        }

        public static byte[] doRSAEncryption(java.security.cert.X509Certificate encryptionCert, byte[] data)
        {
            return doRSAEncryption(encryptionCert, data, Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5);
        }

        public static byte[] doRSAEncryption(java.security.cert.X509Certificate encryptionCert, byte[] data, String algorithm)
        {
            if (algorithm.Equals(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5))
            {
                Pkcs1Encoding pkcs12 = new Pkcs1Encoding(new RsaEngine());
                pkcs12.Init(true, encryptionCert.GetPublicKey());
                return pkcs12.ProcessBlock(data, 0, data.Length);
            }
            else
            {
                IDigest gd = null;
                IDigest gd_mgf = null;
                if (DialogHandler.DigestAlgorithm.Equals(Constants.DIGEST_ALGORITHM_SHA512))
                {
                    gd = new Sha512Digest();
                    gd_mgf = new Sha512Digest();
                }
                else
                {
                    gd = new Sha256Digest();
                    gd_mgf = new Sha256Digest();
                }

                OaepEncoding oaep = new OaepEncoding(new RsaEngine(), gd, gd_mgf, null);
                oaep.Init(true,  encryptionCert.GetPublicKey());
                return oaep.ProcessBlock(data, 0, data.Length);
            }
        }
        
        /// <summary>
        /// Diese Methode prft, ob mit der verwendeten Schlssellnge und dem im DialogHandler eingestellten
        /// Hashalgorithmus eine Verschlsselung mglich ist und liefert ggf. den Identifier fr SHA-256 statt
        /// fr SHA-512. Da das Problem z.Zt. (Version 1.6) nur mit den nicht mehr zulssigen 1024-Bit-Schlsseln
        /// in Verbindung mit SHA-512 auftritt, wird diese Methode von der Bibliothek selbst nicht verwendet.
        /// </summary>
        /// <param name="encryptionCert">Verschlsselungszertifikat</param>
        /// <param name="symmetricCipherAlgo">Symmetrischer Verschlsselungsalgorithmus</param>
        /// <returns></returns>       
        public static String getDigestMethodForOAEP(java.security.cert.X509Certificate encryptionCert, String symmetricCipherAlgo)
        {
            int data_len = 24;
            if (symmetricCipherAlgo.Contains("aes"))
                data_len = Int32.Parse(symmetricCipherAlgo.Substring(symmetricCipherAlgo.IndexOf("#aes") + 4).
                    Substring(0, symmetricCipherAlgo.Length - symmetricCipherAlgo.IndexOf("-cbc") - 1));

            int keyLength = ((Org.BouncyCastle.Crypto.Parameters.RsaKeyParameters)encryptionCert.GetPublicKey()).Modulus.BitLength;
            
            String digAlgo = DialogHandler.DigestAlgorithm;

            if (keyLength < data_len + 2 + 2 * (Int32.Parse(digAlgo.Substring(digAlgo.IndexOf("#sha") + 4))))
                digAlgo = Constants.DIGEST_ALGORITHM_SHA256;

            return digAlgo;
        }
        
        public static System.String createMIMEBoundary() 
        {
			byte[] boundery = new byte[20];
			SupportClass.SecureRandomSupport.getInstance("SHA1PRNG").nextBytes(boundery);
			return de.osci.helper.Base64.encode(boundery);
		}

    }
}