namespace de.osci.osci12.encryption {
	
	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class EncryptedData:EncryptedType 
	{

		new private static Log log;
		
		/// <summary>Stream der zu verschlsselnden Daten.
		/// </summary>
		private System.IO.Stream sourceStream = null;

		/// <summary>Interner Encryption Status 
		/// </summary>
		private int encState = - 1;

		/// <summary>Temp Byte[]
		/// </summary>
		private byte[] tempData;

		public String encNS;
		protected static byte[] ns= MoreSupport.ToByteArray(
" xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:osci=\"http://www.osci.de/2002/04/osci\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
		internal String soapNSPrefix ; 
		internal String osciNSPrefix ;
		internal String dsNSPrefix  ; 
		internal String xencNSPrefix ; 
		internal String xsiNSPrefix;
	
		
		/// <summary> Erstellt ein EncryptedData Objekt mit einem CipherValue
		/// </summary>
		/// <param name="cipherValue">Cipher Value Objekt Nutzdatenverschlsselung
		/// </param>
		/// <param name="encryptionMethodAlgorithm">Algorithmus
		/// </param>
		/// <param name="iD">ID des EncryptedData Objektes
		/// </param>
		/// <param name="key">symmetrischer Schlssel fr die Nutzdatenverschlsselung
		/// </param>
		public EncryptedData(CipherValue cipherValue, System.String encryptionMethodAlgorithm, System.String iD, javax.crypto.SecretKey key) 
		{
			init();
			if ((cipherValue == null) || (key == null)) {
				throw new de.osci.osci12.encryption.OSCICipherException("Es wurden nicht die richtigen Parameter bergeben");
			}
			cipherValue.setSecretKey(key);
			CipherData cipherData = new CipherData(cipherValue);
			this.EncryptionMethodAlgorithm = encryptionMethodAlgorithm;
			this.Type = Constants.TYPE_CONTENT;
			this.MimeType = "text/xml";
			this.Id = iD;
			CipherData = cipherData;
		}
		
		/// <summary> Erstellt ein EncryptedData Objekt mit einem CipherReference
		/// </summary>
		/// <param name="cipherReference">Cipher Reference Objekt Nutzdatenverschlsselung
		/// </param>
		/// <param name="encryptionMethodAlgorithm">Algorithmus
		/// </param>
		/// <param name="iD">ID des EncryptedData Objektes
		/// </param>
		/// <param name="key">symmetrischer Schlssel fr die Nutzdatenverschlsselung
		/// </param>
		public EncryptedData(CipherReference cipherReference, System.String encryptionMethodAlgorithm, System.String iD, javax.crypto.SecretKey key) {
			init();
			if ((cipherReference == null) || (key == null)) {
				throw new de.osci.osci12.encryption.OSCICipherException("Es wurden nicht die richtigen Parameter bergeben");
			}
			CipherData cipherData = new CipherData(cipherReference);
			this.EncryptionMethodAlgorithm = encryptionMethodAlgorithm;
			this.Type = Constants.TYPE_CONTENT;
			this.MimeType = "text/xml";
			this.Id = iD;
			CipherData = cipherData;
		}
		
		/// <summary> Ein Konstruktor fr den Parser
		/// </summary>
		/// <param name="encryptionMethodAlgorithm">Algorithmus
		/// </param>
		/// <param name="iD">ID des EncryptedData Objektes
		/// </param>
		internal EncryptedData(System.String encryptionMethodAlgorithm, System.String iD) {
		init();
			this.EncryptionMethodAlgorithm = encryptionMethodAlgorithm;
			this.Type = Constants.TYPE_CONTENT;
			this.MimeType = "text/xml";
			this.Id = iD;
		}
		
		public virtual EncryptedKey findEncrypedKey(String refId){ 
			EncryptedKey[] keys = this.KeyInfo.EncryptedKeys;
			
			for (int i = 0; i < keys.Length; i++) {
				log.debug("KeyInfo Eintrag:" + keys[i].KeyInfo.RetrievalMethod.URI);
				log.debug("Vergleiche refID: refId: " + refId + " ## mit: " + keys[i].KeyInfo.RetrievalMethod.URI.EndsWith(refId));
				if (keys[i].KeyInfo.RetrievalMethod.URI.EndsWith(refId)) {
					return keys[i];
				}
			}	
			return null;
		}
				
		public void writeXML(System.IO.Stream out_Renamed)  {
		writeXML(out_Renamed, true);
	}

	public void writeXML(System.IO.Stream out_Renamed, bool inner){
			
		if (this.CipherData == null) {
				throw new de.osci.osci12.encryption.OSCICipherException("Es wurde noch kein CipherData Object eingestellt");
			}
			MoreSupport.ToStream(out_Renamed,"<" + xencNSPrefix + ":EncryptedData",Constants.CHAR_ENCODING);
            if (!(out_Renamed is System.Security.Cryptography.CryptoStream) && !inner) 
			{
                if (encNS == null)
                    out_Renamed.Write(ns, 0, ns.Length);
                else
                    MoreSupport.ToStream(out_Renamed, encNS, Constants.CHAR_ENCODING); 
			} 
			if ((Id != null) && !Id.Equals("")) {
				MoreSupport.ToStream(out_Renamed," Id=\"" + this.Id + "\"",Constants.CHAR_ENCODING);}
			MoreSupport.ToStream(out_Renamed," MimeType=\"" + this.MimeType + "\">",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,"<" + xencNSPrefix + ":EncryptionMethod Algorithm=\"" + this.EncryptionMethodAlgorithm + "\"></" + xencNSPrefix + ":EncryptionMethod>",Constants.CHAR_ENCODING);
			if (this.KeyInfo == null) {
				log.error("KeyInfo ist leider null.");
			}
			this.KeyInfo.writeXML(out_Renamed, dsNSPrefix, xencNSPrefix);
			this.CipherData.writeXML(out_Renamed, dsNSPrefix, xencNSPrefix);
			MoreSupport.ToStream(out_Renamed, "</" + xencNSPrefix + ":EncryptedData>",Constants.CHAR_ENCODING);
		}
				
		static EncryptedData() {
			log = LogFactory.getLog(typeof(EncryptedData));
		}

		public void setId(String id0) {
			Id = id0;
		}
		private void init()
		{		  soapNSPrefix = "soap"; 
		osciNSPrefix = "osci";
		dsNSPrefix = "ds"; 
		xencNSPrefix = "xenc"; 
		xsiNSPrefix = "xsi";

		}
	}
}