namespace de.osci.osci12.encryption {
	
	/// <summary> Ein EncryptedKey fr jeden Reader
	/// <EncryptedKey xmlns="http://www.w3.org/2001/04/xmlenc#" Id="job-encrypted-key">
	/// <EncryptionMethod Algorithm="http://www.w3.org/2001/04/xmlenc#rsa-1_5" />
	/// <KeyInfo xmlns="http://www.w3.org/2000/09/xmldsig#" Id="job-key-info">
	/// <X509Data>
	/// <X509Certificate>
	/// MIICiDCCAfGgAwIBAgIGAO6OKE3CMA0GCSqGSIb3DQEBBQUAMG4xCzAJBgNVBAYT
	/// AklFMQ8wDQYDVQQIEwZEdWJsaW4xJDAiBgNVBAoTG0JhbHRpbW9yZSBUZWNobm9s
	/// b2dpZXMgTHRkLjERMA8GA1UECxMIWC9TZWN1cmUxFTATBgNVBAMTDFRyYW5zaWVu
	/// dCBDQTAeFw0wMjA2MjAxNTMzNDJaFw0xMjA2MjAxNTMzMzhaMGUxCzAJBgNVBAYT
	/// AklFMQ8wDQYDVQQIEwZEdWJsaW4xJDAiBgNVBAoTG0JhbHRpbW9yZSBUZWNobm9s
	/// b2dpZXMgTHRkLjERMA8GA1UECxMIWC9TZWN1cmUxDDAKBgNVBAMTA0pvYjCBnzAN
	/// BgkqhkiG9w0BAQEFAAOBjQAwgYkCgYEA3heEqT6jgWd2Q2fFPwjck1fMwoXp2YzA
	/// jJ8PcBDEW4vdCZJYwBcuSPbuMgscqE0pNxlxRc9ZGYXrX2UPHTrxqOdRg43x1D93
	/// 7jLqb5206e+iPGaxdjKFrL1K/UQoltV6w/ibawQRIjNOnDraw4WpELFxdL4jQdjo
	/// Z6u38t5l3fECAwEAAaM6MDgwDgYDVR0PAQH/BAQDAgUgMBEGA1UdDgQKBAiFdmau
	/// 1AD5BjATBgNVHSMEDDAKgAiA4IWwNLDjUTANBgkqhkiG9w0BAQUFAAOBgQBqP1ZE
	/// btWnLPC/8kNvWFq/VHqMsALkks44QM2QOnzDF86EOVXgwkxSeOUii1UIKTwmeMev
	/// 9Kx0fNEpuvQUNP89V0fLNTqVN2BGJL12XgaKpAwoAu3SpUfTEN2WXi3taFKbrIVW
	/// 47YWz1BoHJewZdcO4dfftCHcDGcXg6W3NxW88A==
	/// </X509Certificate>
	/// </X509Data>
	/// </KeyInfo>
	/// <CipherData>
	/// <CipherValue>
	/// DmmgQWTqFhQoUkiPVh3yHpy6CMv/leHFU8qZOMuAztLrDfSTqkKFECtgdeshs4lo
	/// d1lyQ3QuZLXL6nalZHpayP6IZQJiRip54i8JPjZzhkK3sOnrmqwewSqz2CHBe7gv
	/// QWVNcNrV1q/KrjlR3S5JT/9FPQJ6tOyywt9QrQIgbZw=
	/// </CipherValue>
	/// </CipherData>
	/// </EncryptedKey>
	/// </summary>
	
	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	
	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class EncryptedKey:EncryptedType 
	{

		virtual public System.String Recipient {
			get {
				return recipient;
			}
			set {
				this.recipient = value;
			}	
		}

		virtual public System.String CarriedKeyName {
			get {
				return carriedKeyName;
			}			
			set {
				this.carriedKeyName = value;
			}
		}

		virtual public System.String ReferenceList {
			get {
				return referenceList;
			}
			set {
				this.referenceList = value;
			}	
		}
		
		new private static Log log;

		/// <summary>Empfnger der Nachricht 
		/// </summary>
		private System.String recipient;
		
		/// <summary>Name dieses Schlssels. Kann durch KeyName referenziert werden. 
		/// </summary>
		private System.String carriedKeyName;
		
		/// <summary> Referenz auf alle mit diesem Schlssel verschlsselten EncryptedType
		/// Elemente.
		/// </summary>
		private System.String referenceList;

        public String mgfAlgorithm;
  	 
        public String digestAlgorithm;
  	   
  	    private static String XENC11 = "http://www.w3.org/2009/xmlenc11#";
  	   
  	    String xenc11NSPrefix = "xenc11";
    
		private SupportClass.KeySupport key = null;

		/// <summary> Fr den Parser
		/// </summary>
		/// <param name="encryptionMethodAlgorithm">
		/// </param>
		internal EncryptedKey() 
        {
		}
		
		/// <summary> Erstellt ein neues EncryptedKey-Object.
		/// </summary>
		/// <param name="encryptionMethodAlgorithm">
		/// </param>
		/// <param name="cipherValue">
		/// </param>
		public EncryptedKey(System.String encryptionMethodAlgorithm, CipherValue cipherValue) 
        {
			this.EncryptionMethodAlgorithm = encryptionMethodAlgorithm;
            if (encryptionMethodAlgorithm.Equals(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP))
  	        {
  	             // MGF und Hashfunktionen nach DialogHandler
  	             if (DialogHandler.DigestAlgorithm.Equals(Constants.DIGEST_ALGORITHM_SHA512))
  	             {
  	                     mgfAlgorithm = Constants.MASK_GENERATION_FUNCTION_1_SHA512;
  	                     digestAlgorithm = Constants.DIGEST_ALGORITHM_SHA512;
  	             }
  	             else // default
  	             {
  	                     mgfAlgorithm = Constants.MASK_GENERATION_FUNCTION_1_SHA256;
  	                     digestAlgorithm = Constants.DIGEST_ALGORITHM_SHA256;
  	             }
  	        }
			CipherData cipherData = new CipherData(cipherValue);
			this.CipherData = cipherData;
		}

		public virtual void  addDataReference(System.String uRI) {
		}
		
		public virtual void  addKeyReference(System.String uRI) {
		}
		
		public virtual void  writeXML(System.IO.Stream out_Renamed, String ds, String xenc) {
			MoreSupport.ToStream(out_Renamed,"<" + xenc + ":EncryptedKey>",Constants.CHAR_ENCODING);
//			MoreSupport.ToStream(out_Renamed,"<" + xenc + ":EncryptionMethod Algorithm=\"" + this.EncryptionMethodAlgorithm + "\"></" + xenc + ":EncryptionMethod>",Constants.CHAR_ENCODING);
            MoreSupport.ToStream(out_Renamed,"<" + xenc + ":EncryptionMethod Algorithm=\"" + this.EncryptionMethodAlgorithm + "\">", Constants.CHAR_ENCODING);

            if (this.EncryptionMethodAlgorithm.Equals(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_OAEP))
  	        {
  	             MoreSupport.ToStream(out_Renamed,"<" + xenc11NSPrefix + ":MGF xmlns:" + xenc11NSPrefix + "=\"" + XENC11 + "\" Algorithm=\"" + mgfAlgorithm + "\"></" + xenc11NSPrefix + ":MGF>", Constants.CHAR_ENCODING);
  	             MoreSupport.ToStream(out_Renamed,"<" + ds + ":DigestMethod Algorithm=\"" + digestAlgorithm + "\"></" + ds + ":DigestMethod>", Constants.CHAR_ENCODING);
  	        }

  	        MoreSupport.ToStream(out_Renamed,"</" + xenc + ":EncryptionMethod>", Constants.CHAR_ENCODING);

			//MoreSupport.ToStream(out_Renamed,"<xenc:EncryptedKey xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\">",Constants.CHAR_ENCODING);
			this.KeyInfo.writeXML(out_Renamed,  ds,  xenc);
			this.CipherData.writeXML(out_Renamed,  ds,  xenc);
			MoreSupport.ToStream(out_Renamed,"</" + xenc + ":EncryptedKey>",Constants.CHAR_ENCODING);
		}
		
		static EncryptedKey() {
			log = LogFactory.getLog(typeof(EncryptedKey));
		}
	}
}