namespace de.osci.osci12.extinterfaces {

	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.soapheader;

	/// <summary> <p>Clients, die mehrere Dialoge gleichzeitig verwalten mssen,
	/// knnen zu diesem Zweck mit der statischen Methode
	/// DialogHandler.setDialogFinder(de.osci.osci12.common.DialogFinder)
	/// eine Instanz dieser Klasse installieren.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.common.DialogHandler">
	/// </seealso>
	public abstract class DialogFinder {

		/// <summary> Liefert eine Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public abstract System.String Version{get;}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public abstract System.String Vendor{get;}
		
		/// <summary> Diese Methode mu anhand des bergebenen ControlBlockH-Objektes das
		/// zugehrige DialogHandler-Objekt ermitteln und zurckgeben.
		/// </summary>
		/// <param name="controlBlock"> das ControlBlock-Objekt als Identifier des 
		/// gesuchten DialogHandlers
		/// </param>
		/// <returns> gefundener DialogHandler oder null, wenn er nicht gefunden wurde.
		/// </returns>
		public abstract DialogHandler findDialog(ControlBlockH controlBlock);
		
		/// <summary> Entfernt ein DialogHandler-Objekt aus der Liste der verwalteten
		/// DialogHandler-Objekte. Wird nach Beendigung eines Dialogs aufgerufen.
		/// </summary>
		/// <param name="controlBlock"> das ControlBlock-Objekt als Identifier des 
		/// gesuchten DialogHandlers
		/// </param>
		/// <returns> true, wenn der DialogHandler erfolgreich entfernt wurde.
		/// </returns>
		public abstract bool removeDialog(ControlBlockH controlBlock);
		
		/// <summary> Fgt den verwalteten DialogHandler-Objekten ein weiteres hinzu.
		/// </summary>
		/// <param name="dialog">dialog
		/// </param>
		public abstract void  addDialog(DialogHandler dialog);
	}
}