namespace de.osci.osci12.extinterfaces {

	using System;

	/// <summary> <p>Implementierungen dieser Klasse knnen mit Hilfe der Methode
	/// setDataBuffer(OSCIDataSource buffer) des DialogHandlers installiert
	/// werden, falls Inhaltsdaten nicht durch die default-Implementierung
	/// SwapBuffer im Arbeitsspeicher bzw. in temporren Dateien gepuffert werden
	/// sollen, sondern beispielsweise in einer Datenbank.</p>
	/// Dieser Puffer-Mechanismus wird von den Klassen EncryptedData, Content und Attachment
	/// genutzt.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.common.SwapBuffer">
	/// </seealso>
	public abstract class OSCIDataSource {
		
		/// <summary> Die Implementierung dieser Methode muss einen OutputStream liefern, in den
		/// die zu puffernden Daten geschrieben werden knnen.
		/// </summary>
		/// <value> OutputStream
		/// </value>
		public abstract System.IO.Stream OutputStream {
			get;
		}

		/// <summary> Die Implemetierung dieser Methode mu einen InputStream liefern, aus dem
		/// die gepufferten Daten gelesen werden knnen. Der erste Aufruf dieser
		/// Methode beendet den Schreibvorgang in diesen Puffer.
		/// <b>Achtung:</b> Der zurckgegebene InputStream mu die reset()-Methode
		/// in der Weise implementieren, da nach deren Aufruf wieder von vorn ab dem
		/// ersten Byte gelesen wird. Die markSupported()-Methode mu
		/// <b>false</b> zurckliefern.
		/// </summary>
		/// <value> InputStream
		/// </value>
		public abstract System.IO.Stream InputStream {
			get;
		}
		
		/// <summary> Diese Methode mu die Anzahl der in den Puffer geschriebenen Bytes
		/// zurckgeben.
		/// </summary>
		/// <value> Anzahl der Bytes
		/// </value>
		public abstract long Length {
			get;
		}
		
		/// <summary> Liefert eine Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public abstract System.String Version {
			get;
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public abstract System.String Vendor {
			get;
		}

		/// <summary> Die Implementierung dieser statischen Methode muss eine neue Instanz der Klasse
		/// zurckgeben.
		/// </summary>
		/// <returns> neue Instanz der implementierenden Klasse
		/// </returns>
		public abstract OSCIDataSource newInstance();
	}
}