namespace de.osci.osci12.extinterfaces {

	using System;

	/// <summary> Interface-Klasse fr das bermitteln von OSCI-Nachrichten
	/// Die OSCI-Bibliothek sieht nicht vor den Nutzer auf ein Transportprotokoll
	/// festzulegen. Aus diesem Grund wird ein Transport-Interface zur Verfgung
	/// gestellt welches es der Anwendung ermglicht die erstellten OSCI-Nachrichten
	/// mit dem gewnschtem Protokoll oder auf die gewnschte Art zu bermitteln oder
	/// zu speichern. Vorstellbare Implementierungen sind z.B. http, https, ftp,
	/// smpt/pop, Filesystem, oder jms.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public interface TransportI {

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		System.String Version {
			get;		
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		System.String Vendor {
			get;		
		}

		/// <summary> Liefert den Response Stream.
		/// </summary>
		/// <value> InputStream der eingehenden Antwortdaten.
		/// </value>
		System.IO.Stream ResponseStream {
			get;	
		}

		/// <summary> Sollte die Lnge des Response Streams liefern. Diese Methode wird
		/// von der Bibliothek z.Zt. nicht bentigt.
		/// </summary>
		/// <value> Anzahl der empfangenen Bytes.
		/// </value>
		long ContentLength {
			get;
		}

		/// <summary> Methode kann zur berprfung der Erreichbarkeit einer URL implementiert
		/// und verwendet werden. Die Bibliothek selbst ruft diese Methode nicht auf.
		/// </summary>
		/// <param name="uri">URI des Kommunikationspartners.
		/// </param>
		/// <returns>true, wenn der Kommunikationspartner erreichbar ist.
		/// </returns>
		bool isOnline(Uri uri);
			
		/// <summary>  Liefert eine konkrete Verbindung zum Versenden eines Streams.
		/// Die Methode konnektet zu der bergebenen URI und liefert als Ergebnis
		/// einen Outputstream, in den die Bibliothek dann die serialisierte OSCI-Nachricht
		/// schreibt.
		/// </summary>
		/// <param name="uri">URI des Kommunikationspartners.
		/// </param>
		/// <param name="length">Lnge der bertragungsdaten (Anzahl der Bytes).
		/// </param>
		/// <returns>Output-Stream, in den die Daten geschrieben werden knnen.
		/// </returns>
		TransportI newInstance();
		/** 
		* Die Implementierung dieser statischen Methode muss eine neue Instanz der Klasse 
		* zurckgeben. 
		* @return neue Instanz der implementierenden Klasse 
		* @throws IOException wenn ein Fehler auftritt 
		*/ 
		System.IO.Stream getConnection(Uri uri, long length);
	}
}