namespace de.osci.osci12.extinterfaces.crypto {

	using System;
	using java.security.cert;
	using de.osci.osci12.encryption;

	/// <summary>Diese abstrakte Klasse stellt die Schnittstelle der Bibliothek fr
	/// die Anbindung von Entschlsselungs-Modulen (Crypto-Token) dar. Anwendungen, die
	/// OSCI-Nachrichten entschlsseln wollen, mssen an das entschlsselnde Rollenobjekt
	/// eine Implementation dieser Klasse bergeben.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public abstract class Decrypter 
    {

		/// <summary> Liefert die Versionsnummer.
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public abstract System.String Version 
        {
			get;
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public abstract System.String Vendor 
        {
			get;
		}

		/// <summary> Die Implementierung dieser Methode muss das Verschlsselungszertifikat zurckgeben.
		/// </summary>
		/// <value> Verschlsselungszertifikat
		/// </value>
		public abstract X509Certificate Certificate 
        {
			get;
		}
		
		/// <summary> Die Implementierung dieser Methode muss das bergebene Byte-Array gem RSAES-PKCS1-v1_5 entschlsseln.
		/// </summary>
		/// <param name="data">Zu entschlsselndes Byte-Array.
		/// </param>
		/// <returns> Entschlsseltes Byte-Array.
		/// </returns>
		public abstract byte[] decrypt(byte[] data);

        /// <summary> Die Implementierung dieser Methode muss das bergebene Byte-Array gem RSAES-OAEP oder RSAES-PKCS1-v1_5
        /// entschlsseln. Diese Methode muss berschrieben werden, die default-Implementierung 
        /// wirft ein UnsupportedOperationException.
        /// </summary>
        /// <param name="data">Zu entschlsselndes Byte-Array
        /// </param>
        /// <param name="data">Identifier der Mask-Generation-Function (null fr RSAES-PKCS1-v1_5)
        /// </param>
        /// <param name="data">Identifier des verwendeten Hashalgorithmus (wird ignoriert fr RSAES-PKCS1-v1_5)
        /// </param>
        /// <returns> Entschlsseltes Byte-Array.
        /// </returns>
        public virtual byte[] decrypt(byte[] data, String mfgAlgorithm, String digestAlgorithm)
        {
            throw new UnsupportedOperationException("Entschlsselung mit RSAES-OAEP in diesem Decrypter nicht implementiert.");
        }

	}
}