namespace de.osci.osci12.extinterfaces.crypto {

	using System;
	using java.security.cert;

	/// <summary> Diese abstrakte Klasse stellt die Schnittstelle der Bibliothek fr
	/// die Anbindung von Signier-Modulen (Crypto-Token) dar. Anwendungen, die
	/// OSCI-Nachrichten signieren wollen, mssen an das signierende Rollenobjekt
	/// eine Implementation dieser Klasse bergeben.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.roles.Role.SignatureAlgorithm()">
	/// </seealso>
	public abstract class Signer {

		/// <summary> Liefert die Versionsnummer
		/// </summary>
		/// <value> Versionsnummer
		/// </value>
		public abstract System.String Version {
			get;
		}

		/// <summary> Liefert den Namen des Herstellers.
		/// </summary>
		/// <value> Herstellername
		/// </value>
		public abstract System.String Vendor {
			get;
		}

		/// <summary> Die Implementierung dieser Methode muss das Signaturzertifikat zurckgeben.
		/// </summary>
		/// <value> Signaturzertifikat
		/// </value>
		public abstract X509Certificate Certificate {
			get;
		}
		
		/// <summary> Die Implementierung dieser Methode muss das bergebene Byte-Array signieren.
        /// Der algorithm-Parameter hat keine weitere Bedeutung mehr, da ab Version 1.3 die
        /// Signer-Implemetierung ber die Methode getAlgorithm() den Algorithmus festlegt.
        /// Die OSCI-Bibliothek fragt vor dem Aufruf von sign(byte[], String) den Signaturalgorithmus
        /// ab und bergibt den String wieder an die sign-Methode.
		/// </summary>
		/// <param name="hash">Zu signierendes Byte-Array.
		/// </param>
		/// <param name="algorithm">Algorithmus
		/// </param>
		/// <returns> Signatur-Wert
		/// </returns>
		public abstract byte[] sign(byte[] hash, System.String algorithm);

        /// <summary> Diese Methode sollte den Signaturalgorithmus als XML-Identifier zurckgeben,
        /// den die Implementierung bei der Erzeugung eine Signatur verwendet.
        /// 
        /// </summary>
        /// <returns> Signaturalgorithmus
        /// </returns>
        public abstract String getAlgorithm();
    }
}