namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.roles;
	using de.osci.osci12.signature;
	using System.Security.Cryptography;
	using de.osci.osci12.common;
    using System.Collections;

	/// <summary> <p>Diese Klasse ist die Basisklasse fr smtliche Header-Element und alle weiteren
	/// Elemente welche in einer OSCI-Nachricht signiert werden (Attachment, ContentContainer,
	/// Content... ).</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public abstract class MessagePart {

		protected internal   String soapNSPrefix = "soap"; 
		protected internal  String osciNSPrefix = "osci";
		protected	internal  String dsNSPrefix = "ds"; 
		protected	internal  String xencNSPrefix = "xenc";
		protected	internal  String xsiNSPrefix = "xsi"; 
  
		private static Log log;
		protected internal System.Collections.Hashtable xmlns;
		protected internal System.String id = "";
		protected internal System.String typ;
		protected internal ArrayList transformers;
		protected internal Hashtable digestValues = new Hashtable();
		public byte[] ns;
		protected  String b64;
		protected  String can;
//		protected  SHA1Managed md; 

		private void  InitBlock() 
		{
			xmlns = new System.Collections.Hashtable();
			transformers = new System.Collections.ArrayList();
		}

		/// <summary> Das refID-Attribut des MessagePart-Objekts.
		/// </summary>
		/// <value> refID-String
		/// </value>
		virtual public System.String RefID {
			get {
				return id;
			}
			
			set {
				this.id = value;
			}
			
		}

		/// <summary> Liefert den Hashwert des Message-Parts.
		/// </summary>
		/// <value> Hashwert
		/// </value>
		virtual public byte[] getDigestValue(String digestAlgorithm) 
        {
				if (digestValues[digestAlgorithm]  != null)
					return (byte[])digestValues[digestAlgorithm];

				System.IO.MemoryStream m=new System.IO.MemoryStream();
                HashAlgorithm msgDigest = MoreSupport.createMessageDigest(digestAlgorithm);
                
                CryptoStream outStream = new CryptoStream(m, msgDigest, System.Security.Cryptography.CryptoStreamMode.Write);
				writeXML(outStream);
				m.Seek(0,System.IO.SeekOrigin.Begin);
//				string s=MoreSupport.NewString(m);
				outStream.Close();
                digestValues.Add(digestAlgorithm, msgDigest.Hash);
				return (byte[])digestValues[digestAlgorithm];
		}

		/// <summary> Liefert ein Array mit den installierten Transformern.
		/// </summary>
		/// <returns> Transformer-Array
		/// </returns>
		virtual protected internal Transform[] TransformerForSignature {
			get {
				return new Transform[transformers.Count];
			}
			
		}

		protected internal MessagePart() {
			b64 = "<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_BASE64+"\"></"+dsNSPrefix+":Transform>"; 
			 can="<"+dsNSPrefix+":Transform Algorithm=\"" + Constants.TRANSFORM_CANONICALIZATION+"\"></"+dsNSPrefix+":Transform>"; 
			InitBlock();
			typ = GetType().FullName.Substring(GetType().FullName.LastIndexOf((System.Char) '.') + 1).ToLower();
	       ns = MoreSupport.ToByteArray(" xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:osci=\"http://www.osci.de/2002/04/osci\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"");
		}
		
		/// <summary> Interne Methode
		/// </summary>
		/// <param name="out_Renamed"> Stream
		/// </param>
		public abstract void  writeXML(System.IO.Stream out_Renamed);
		
		/// <summary> Fgt der Liste von Transformer-Objekten ein weiteres hinzu. Die Transformer
		/// werden in der Reihenfolge des Hinzufgens beim Signieren dieses Objektes
		/// verwendet.
		/// </summary>
		/// <param name="transform">Transformer
		/// </param>
		public virtual void  addTransformerForSignature(Transform transform) {
			transformers.Add(transform);
		}
		
		static MessagePart() {
			log = LogFactory.getLog(typeof(MessagePart));
		}
		public void setNSPrefixes(de.osci.osci12.messagetypes.OSCIMessage msg) 
		{ 
			soapNSPrefix = msg.soapNSPrefix; 
			osciNSPrefix = msg.osciNSPrefix; 
			dsNSPrefix = msg.dsNSPrefix; 
			xencNSPrefix = msg.xencNSPrefix; 
			xsiNSPrefix = msg.xsiNSPrefix; 
			ns = MoreSupport.ToByteArray(msg.ns,Constants.CHAR_ENCODING);
				} 
		public void setNSPrefixes(String soap, String osci, String ds, String xenc, String xsi) 
		{ 
			soapNSPrefix = soap;
			osciNSPrefix = osci;
			dsNSPrefix = ds;
			xencNSPrefix = xenc;
			xsiNSPrefix = xsi;
		}    
	} 
}