namespace de.osci.osci12.messageparts {
	
	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.messagetypes;

	/// <summary> Diese Klasse dient dazu, von auerhalb des Packages auf Konstruktoren und
	/// Methoden zuzugreifen, die dort nicht sichtbar sind, um die API fr den Anwender
	/// bersichtlich zu halten. Die Klasse sollte deshalb nicht dokumentiert werden.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="ContentContainer">
	/// </seealso>
	/// <seealso cref="Attachment">
	/// </seealso>
	public class MessagePartsEntry {

		private static ISO8601DateTimeFormat isodate;
		
		public static EncryptedDataOSCI encryptedDataOsci(javax.crypto.SecretKey secretKey, System.String encryptionMethodAlgorithm, Attachment att) {
			return new EncryptedDataOSCI(secretKey,  att);
		}
		
		public static ProcessCardBundle processCardBundle(System.String name, System.String messageId, System.String recentModification, Timestamp creation, Timestamp forwarding, Timestamp reception, System.String subject, Inspection[] inspections) {
			return new ProcessCardBundle(name, messageId, recentModification, creation, forwarding, reception, subject, inspections);
		}
		
		public static Timestamp timestamp(int nameID, System.String algorithm, System.String timeStamp) {
			return new Timestamp(nameID, algorithm, timeStamp);
		}
		
		public static void  setTimestampCreation(ProcessCardBundle pcb, Timestamp tm) {
			pcb.RecentModification = SupportClass.FormatDateTime(isodate.MakeFormat(), System.DateTime.Now);
			pcb.Creation = tm;
		}
		
		public static void  setTimestampForwarding(ProcessCardBundle pcb, Timestamp tm) {
			pcb.RecentModification = SupportClass.FormatDateTime(isodate.MakeFormat(), System.DateTime.Now);
			pcb.Forwarding = tm;
		}
		
		public static void  setTimestampReception(ProcessCardBundle pcb, Timestamp tm) {
			pcb.RecentModification = SupportClass.FormatDateTime(isodate.MakeFormat(), System.DateTime.Now);
			pcb.Reception = tm;
		}
		
		public static OSCISignature osciSignature(System.String enclosingElement) {
			return new OSCISignature(enclosingElement);
		}
		
		public static OSCISignatureReference osciSignatureReference(MessagePart mp, String algo) {
			return new OSCISignatureReference(mp, algo);
		}
		
		public static Attachment attachment(System.IO.Stream ins, System.String refId, long length, String digAlgo) {
            return new Attachment(ins, refId, length, digAlgo);
		}
		
		public static void  attachmentSetStream(Attachment attachment, System.IO.Stream ins, bool encrypt, long length, String digAlgo) {
            attachment.setInputStream(ins, encrypt, length, digAlgo);
		}
		
		public static void  attachmentSetState(Attachment att, int newState, bool encrypted) {
			att.stateOfAttachment = newState;
			att.encrypt = encrypted;
		}

		static MessagePartsEntry() {
			isodate = new ISO8601DateTimeFormat();
		}
		public static  AcceptDelivery newAcceptDelivery(DialogHandler dh){return new AcceptDelivery(dh);}
		public static void sign(OSCIMessage msg) {
			if (msg is OSCIRequest)	((OSCIRequest)msg).sign();
			else ((OSCIResponseTo)msg).sign();
		}
		public static Attachment NewAttachment(String refid, javax.crypto.SecretKey key,System.IO.Stream in1, String algo) {
			Attachment att=new Attachment(null,refid,0, algo);
			att.setInputStream(in1,true,0, algo);
			att.stateOfAttachment=Attachment.STATE_OF_ATTACHMENT_ENCRYPTED;
			att.secretKey=key;
			return att;
		}
		public static void setAttachmentSecretKey(de.osci.osci12.messageparts.Attachment atta, javax.crypto.SecretKey sk) {
			atta.secretKey = sk;
		}

	}
}