namespace de.osci.osci12.messageparts {

	using System;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;
	using de.osci.helper;
	using OSCIException = de.osci.osci12.OSCIException;
	using org.xml.sax;
	using DefaultHandler = org.xml.sax.helpers.DefaultHandler;

	/// <summary> Die Methoden dieser Factory-Klasse werden von Anwendungen nicht bentigt und
	/// sind deshalb nicht dokumentiert.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class MessagePartsFactory {
		
		public static ContentPackageBuilder createContentPackageBuilder(de.osci.osci12.messagetypes.OSCIMessageBuilder omb) {
			return new ContentPackageBuilder(omb);
		}
		
		public static OSCISignatureBuilder createOsciSignatureBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes atts) {
			return new OSCISignatureBuilder(xmlReader, parentHandler, atts);
		}
		
		public static ProcessCardBundle createProcessCardBundle(System.String name, System.String messageId, System.String recentModification, Timestamp creation, Timestamp forwarding, Timestamp reception, System.String subject, Inspection[] inspections) {
			return new ProcessCardBundle(name, messageId, recentModification, creation, forwarding, reception, subject, inspections);
		}
		/*
		public static OSCISignature createOSCISignature(System.String enclosingElement) {
			return new OSCISignature(enclosingElement);
		}
		
		public static OSCISignatureReference createOSCISignatureReference(MessagePart mp, String digestAlgorithm) {
            return new OSCISignatureReference(mp, digestAlgorithm);
		}
         */

		public static FeedbackObject createFeedbackObject(String[]feedback) { 
			return new FeedbackObject(feedback); 
		}

        public static Attachment attachment(System.IO.Stream ins, System.String refId, long length, String digestAlgorithm)
        {
			return new Attachment(ins, refId, length, digestAlgorithm);
		}

        public static void attachmentSetStream(Attachment attachment, System.IO.Stream ins, bool encrypt, long length, String digestAlgorithm)
        {
			attachment.setInputStream(ins, encrypt, length, digestAlgorithm);
		}
		
		public static void  attachmentSetState(Attachment att, int newState, bool encrypted) 
		{
			att.stateOfAttachment = newState;
			att.encrypt = encrypted;
		}
		
		public static void  writeXML(MessagePart mp, System.IO.Stream out_Renamed) 
		{
			mp.writeXML(out_Renamed);
		}
	}
}