namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;
    using System.Collections;
    using System.Security.Cryptography;

	/// <summary> Diese Klasse bildet eine die Grundlage fr XML-Signature Signaturen der Bibliothek.
	/// Sie wird von Anwendungen nie direkt bentigt.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class OSCISignature:MessagePart {

		private void  InitBlock() {
			refs = new ArrayList();
			signatureAlgorithm = DialogHandler.SignatureAlgorithm;
		}

		virtual public byte[] SignedInfoBytes {
			get {
					return signedInfo;
			}
		}

		new private static Log log;

		public ArrayList refs;
		public byte[] signatureValue;
		public byte[] signedInfo;
		internal System.String enclosingElement;
		public System.String signerId;
        internal String signingTime;
        internal String signingPropsId;
        internal String signingProperties;

		public System.String signatureAlgorithm;
		public de.osci.osci12.roles.Role signer = null;
        internal Hashtable refsHash, refsDigestMethods;

		internal static int cnt = 0;
		internal OSCISignature() 
        {
			InitBlock();
		}
		
		internal OSCISignature(System.String enclosingElement) 
        {
			InitBlock();
			this.enclosingElement = enclosingElement;
		}

        internal void addSignatureTime(String time, String _id, String digAlgo)
        {            
            signingPropsId = _id;
            signingTime = time;
            String nsHead = "xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:osci=\"http://www.osci.de/2002/04/osci\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:xades=\"http://uri.etsi.org/01903/v1.3.2#\" xmlns:xenc=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"";
            signingProperties = "<xades:SignedProperties " + nsHead
                + " Id=\"" + signingPropsId 
                + "\"><xades:SignedSignatureProperties><xades:SigningTime>"
                + signingTime
                + "</xades:SigningTime></xades:SignedSignatureProperties></xades:SignedProperties>";

            OSCISignatureReference osr = new OSCISignatureReference();

            HashAlgorithm msgDigest = MoreSupport.createMessageDigest(digAlgo);
            osr.digestValue = msgDigest.ComputeHash(MoreSupport.ToByteArray(signingProperties, Constants.CHAR_ENCODING));
            osr.digestMethodAlgorithm = digAlgo;
            osr.setNSPrefixes(soapNSPrefix, osciNSPrefix, dsNSPrefix,
                              xencNSPrefix, xsiNSPrefix);
            osr.id = "#" + _id;
            refs.Add(osr);
        }

		internal OSCISignatureReference[] getReferences()
		{
			return (OSCISignatureReference[])SupportClass.ICollectionSupport.ToArray(refs, new OSCISignatureReference[refs.Count]);
		}
	
		public virtual void  sign(Role signer) 
		{
			this.signer = signer;
            this.signatureAlgorithm = signer.SignatureAlgorithm;
            if (signedInfo == null)
            {
				createSignedInfo();
			}
			signatureValue = signer.Signer.sign(signedInfo, signatureAlgorithm);
			signerId =  "#" +signer.SignatureCertificateId;
		}

        /// <summary>Liefert den zu dieser Signatur gehrende Signaturzeitpunkt (ISO-8601-DateTime Format).
        /// </summary>
        /// <returns>Signaturzeitpunkt
        /// </returns>
        public String getSigningTime()
        {
            return signingTime;
        }

        public System.Collections.Hashtable getDigests() 
		{
            if (refsHash == null)
            {
                refsHash = new System.Collections.Hashtable();
                for (int i = 0; i < refs.Count; i++)
                    SupportClass.PutElement(refsHash, ((OSCISignatureReference)refs[i]).RefID, ((OSCISignatureReference)refs[i]).digestValue);
            }
			return refsHash;
		}

        public Hashtable getDigestMethods()
        {
            if (refsDigestMethods == null)
            {
                refsDigestMethods = new Hashtable();

                for (int i = 0; i < refs.Count; i++)
                    refsDigestMethods.Add(((OSCISignatureReference)refs[i]).RefID,
                        ((OSCISignatureReference)refs[i]).digestMethodAlgorithm);
            }
            return refsDigestMethods;
        }

		
		private void  createSignedInfo() {
			// Das Element SignInfo aufbauen
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
		    MoreSupport.ToStream(out_Renamed,("<" + dsNSPrefix + ":SignedInfo xmlns:" + dsNSPrefix + "=\"http://www.w3.org/2000/09/xmldsig#\" xmlns:" + osciNSPrefix + "=\"http://www.osci.de/2002/04/osci\" xmlns:" + soapNSPrefix + "=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:" + xencNSPrefix + "=\"http://www.w3.org/2001/04/xmlenc#\" xmlns:" + xsiNSPrefix + "=\"http://www.w3.org/2001/XMLSchema-instance\">\n"),Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,("<" + dsNSPrefix + ":CanonicalizationMethod Algorithm=\"http://www.w3.org/TR/2001/REC-xml-c14n-20010315\"></" + dsNSPrefix + ":CanonicalizationMethod>\n<" + dsNSPrefix + ":SignatureMethod Algorithm=\"" + signatureAlgorithm + "\"></" + dsNSPrefix + ":SignatureMethod>\n"),Constants.CHAR_ENCODING );
  
			for (int i = 0; i < refs.Count; i++) {
				((de.osci.osci12.messageparts.OSCISignatureReference) refs[i]).writeXML(out_Renamed);
			}
			MoreSupport.ToStream(out_Renamed,"</" + dsNSPrefix + ":SignedInfo>",Constants.CHAR_ENCODING);
			signedInfo = out_Renamed.ToArray();
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			if (enclosingElement != null) {
				MoreSupport.ToStream(out_Renamed,enclosingElement,Constants.CHAR_ENCODING);
			}
//			MoreSupport.ToStream(out_Renamed,"<" + dsNSPrefix + ":Signature>",Constants.CHAR_ENCODING);
//			int start=0, stop; 
            String prf = dsNSPrefix + ":";
            int start = 0;
            int stop;
            for (start = 0; start < signedInfo.Length; start++)
                if (signedInfo[start] == '<')
                  break;
          byte[] prfBytes = MoreSupport.ToByteArray(prf);// prf.getBytes(Constants.CHAR_ENCODING);
            int j = 0;
            for (int i = start+1; i<prf.Length; i++, j++)
            {
    	        if (prfBytes[j] != signedInfo[i])
    	        {
    		        prf = "";
    		        break;
    	        }
            }
            
            if (prf.Length == 0)
    	        MoreSupport.ToStream(out_Renamed,"<Signature " + "xmlns=\"" + MessagePartParser.DS_XMLNS + "\">",Constants.CHAR_ENCODING);
            else
    	        MoreSupport.ToStream(out_Renamed,"<" + prf + "Signature>",Constants.CHAR_ENCODING);
            //neu in126
			for (start=0; start<signedInfo.Length; start++) 
				if (signedInfo[start]==0x20) 
					break; 
			for (stop=start; stop<signedInfo.Length; stop++) 
				if (signedInfo[stop]=='>') 
					break; 
			out_Renamed.Write(signedInfo, 0, start); 
			out_Renamed.Write(signedInfo, stop, signedInfo.Length - stop);
            MoreSupport.ToStream(out_Renamed,"<" + prf + "SignatureValue>",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,de.osci.helper.Base64.encode(this.signatureValue),Constants.CHAR_ENCODING);
            //id
            MoreSupport.ToStream(out_Renamed,"</" + prf + "SignatureValue><" + prf + "KeyInfo><" + prf +
                "RetrievalMethod URI=\"",Constants.CHAR_ENCODING);
            MoreSupport.ToStream(out_Renamed,signerId,Constants.CHAR_ENCODING);

            MoreSupport.ToStream(out_Renamed, "\"></" + prf + "RetrievalMethod></" + prf + "KeyInfo>", Constants.CHAR_ENCODING);
            if (signingProperties != null)
            {
                MoreSupport.ToStream(out_Renamed, "<" + dsNSPrefix + ":Object><xades:QualifyingProperties xmlns:xades=\""
                    + MessagePartParser.XADES_XMLNS + "\">", Constants.CHAR_ENCODING);
                int nsStart = signingProperties.IndexOf("<xades:SignedProperties");
                nsStart = signingProperties.IndexOf(' ', nsStart);
                MoreSupport.ToStream(out_Renamed, signingProperties.Substring(0, nsStart), Constants.CHAR_ENCODING);
                MoreSupport.ToStream(out_Renamed, signingProperties.Substring(signingProperties.IndexOf(" Id=", nsStart)), Constants.CHAR_ENCODING);
                MoreSupport.ToStream(out_Renamed, "</xades:QualifyingProperties></" + dsNSPrefix + ":Object>", Constants.CHAR_ENCODING);
            }
            MoreSupport.ToStream(out_Renamed, "</" + prf + "Signature>", Constants.CHAR_ENCODING);

			if (enclosingElement != null) 
				MoreSupport.ToStream(out_Renamed,"</" + enclosingElement.Substring(1, (enclosingElement.IndexOf((System.Char) ' ')) - (1)) + ">",Constants.CHAR_ENCODING);
		}

		static OSCISignature() {
			log = LogFactory.getLog(typeof(OSCISignature));
		}
	}
}