namespace de.osci.osci12.messageparts {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;

	/// <summary> Diese Klasse bildet den OSCI-Laufzettel ab.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ProcessCardBundle:MessagePart {
private static long serialVersionUID = -5723258187933747170L;	
		internal Timestamp Creation;

		/// <summary> Liefert den Zeitstempel der Erstellung.
		/// </summary>
		/// <value> Zeitstempel
		/// </value>
		virtual public Timestamp creation {
			get {
				return Creation;
			}
		}

		internal Timestamp Forwarding;

		/// <summary> Liefert den Zeitstempel der Weiterleitung.
		/// </summary>
		/// <value> Zeitstempel
		/// </value>
		virtual public Timestamp forwarding {
			get {
				return Forwarding;
			}
		}

		internal Timestamp Reception;
		
		/// <summary> Liefert den Zeitstempel des Empfangs.
		/// </summary>
		/// <value> Zeitstempel
		/// </value>
		virtual public Timestamp reception {
			get {
				return Reception;
			}
		}

		internal Inspection[] Inspections;

		/// <summary> Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
		/// die im ProcessCardBundle-Objekt enthalten sind.
		/// </summary>
		/// <value> die Prfergebnisse
		/// </value>
		virtual public Inspection[] inspections {
			get {
				return Inspections;
			}
		}

		internal System.String MessageId;

		/// <summary> Liefert die Message-ID der Nachricht.
		/// </summary>
		/// <value> die Message-ID
		/// </value>
		virtual public System.String messageId {
			get {
				return MessageId;
			}
		}

		internal System.String RecentModification;

		/// <summary> Liefert das Datum der letzten nderung des Laufzettels. Das Format
		/// entspricht dem XML-Schema nach <a href="http://www.w3.org/TR/xmlschema-2/#dateTime">
		/// http://www.w3.org/TR/xmlschema-2/#dateTime</a>.
		/// </summary>
		/// <value> Datum der letzten nderung.
		/// </value>
		virtual public System.String recentModification {
			get {
				return RecentModification;
			}
		}

		internal System.String Subject;

		/// <summary> Liefert den im Laufzettel enthaltenen Betreff-Eintrag.
		/// </summary>
		/// <value> den Betreff der Zustellung
		/// </value>
		virtual public System.String subject {
			get {
				return Subject;
			}
		}

		virtual internal System.String Name {
			set {
				this.name = value;
			}
		}

		new private static Log log;
		private static int idNr = - 1;
		public System.String name;
		
		internal ProcessCardBundle(System.String name) {
		}
		
		internal ProcessCardBundle(System.String name, System.String messageId, System.String recentModification, Timestamp creation, Timestamp forwarding, Timestamp reception, System.String subject, Inspection[] inspections) {
			idNr++;
			this.id = "processcardbundle" + idNr;
			this.name = name;
			this.MessageId = messageId;
			this.RecentModification = recentModification;
			this.Creation = creation;
			this.Forwarding = forwarding;
			this.Reception = reception;
			this.Subject = subject;
			this.Inspections = inspections;
		}
		
		//@todo: protected
		/**
		 * Interne Methode.
		 * @return ProcessCardBundle als String
		 * @throws IOException
		 */
		public String writeToString() {
			System.IO.MemoryStream out_Renamed = new System.IO.MemoryStream();
			writeXML(out_Renamed);
			byte[] b = out_Renamed.GetBuffer();
			string s= MoreSupport.NewString(b,0,b.Length);
			int i=s.LastIndexOf(">");
			if (i<b.Length-1)return s.Substring(0,i+1);
			return s;
		}
		/// <summary>Interne Methode 
		/// </summary>

		public override void  writeXML(System.IO.Stream out_Renamed) 
		{writeXML(out_Renamed, false);
		}
		public void  writeXML(System.IO.Stream out_Renamed, bool writeObj) 
		{
			log.debug("Name: " + name);
			log.debug("MessageID: " + messageId);
			log.debug("recentModi: " + recentModification);
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":" +	name,Constants.CHAR_ENCODING);
			if(	writeObj)
				MoreSupport.ToStream(out_Renamed," xmlns:" + osciNSPrefix + "=\"http://www.osci.de/2002/04/osci\"" ,Constants.CHAR_ENCODING);

			MoreSupport.ToStream(out_Renamed, "><" + osciNSPrefix + ":MessageId>" + de.osci.helper.Base64.encode(messageId) +
					"</" + osciNSPrefix + ":MessageId><" + osciNSPrefix + 
						":ProcessCard RecentModification=\"" + 
					recentModification + "\">",Constants.CHAR_ENCODING);
			if (creation != null) {
				creation.writeXML(out_Renamed);
			}
			if (forwarding != null) {
				forwarding.writeXML(out_Renamed);
			}
			if (reception != null) {
				reception.writeXML(out_Renamed);
			}
			if (subject != null) {
				MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix + ":Subject>" + subject + 
					"</" + osciNSPrefix + ":Subject>",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":ProcessCard><" + osciNSPrefix + ":InspectionReport>",Constants.CHAR_ENCODING);
			if (inspections != null) {
				for (int i = 0; i < inspections.Length; i++) {
					inspections[i].writeXML(out_Renamed);
				}
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix + ":InspectionReport></" + osciNSPrefix + ":" + name + ">",Constants.CHAR_ENCODING);
		}

		static ProcessCardBundle() {
			log = LogFactory.getLog(typeof(ProcessCardBundle));
		}
		internal static String encode(String text) { 
			text = text.Replace("&", "&amp;"); 
			text = text.Replace("<", "&lt;"); 
			text = text.Replace(">", "&gt;"); 
    
			return text.Replace("\r", "&#xD;"); 
		} 
    

	}
}