namespace de.osci.osci12.messagetypes {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;

	/// <summary><H4>AcceptDelivery-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class AcceptDeliveryBuilder:OSCIMessageBuilder {

		new private static Log log;
		
		// ProcessCards liegen im soap:Header und mssen beim parsen ausgewertet werden.
		private ProcessCardBundleBuilder processCardBuilder = null;
		
		/// <summary> Konstruktor fr den aufrufenden Mime-Parser.
		/// </summary>
		/// <param name="parentBuilder"> Parent Object in den meisten Fllen der OSCIEnvelopeBuilder.
		/// </param>
		public AcceptDeliveryBuilder(OSCIEnvelopeBuilder parentBuilder):base(parentBuilder) {
			msg = new de.osci.osci12.messagetypes.AcceptDelivery();
		}
		
		/// <summary> berschreibt die character Methode des DefaultHandler
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			
			if (insideBody) 
			{
				if (localName.ToUpper().Equals("ContentPackage".ToUpper())&& uri.Equals(OSCI_XMLNS))
					parentBuilder.xmlReader.setContentHandler(new ContentPackageBuilder(this));
			}
			else if (insideHeader) 
			{
				log.debug("inside Header");
				
				// ### Auswerten des ControlBlock Headers ###
				if (localName.ToUpper().Equals("ControlBlock".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{0, 1, 0, 0}));
					// ### Auswerten des ClientSignature Headers ###
				}
				else if (localName.ToUpper().Equals("ClientSignature".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.ToUpper().Equals("DesiredLanguages".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					msg.desiredLanguagesH = new DesiredLanguagesH(attributes.getValue("LanguagesList"));
					msg.desiredLanguagesH.RefID=attributes.getValue("Id");
					msg.dialogHandler.LanguageList=attributes.getValue("LanguagesList");
				}
				else if (localName.ToUpper().Equals("acceptDelivery".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					;
				}
				else if (localName.ToUpper().Equals("ProcessCardBundle".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{1, 1, 0, - 1};
					processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(processCardBuilder);
				}
				else if (localName.ToUpper().Equals("IntermediaryCertificates".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else if (localName.ToUpper().Equals("NonIntermediaryCertificates".ToUpper())&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{- 1, - 1, - 1, - 1, - 1, - 1, 0};
					parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
					log.debug("Accept Delivery Element: " + localName);
				}
			}
			else {
				base.startElement(uri, localName, qName, attributes);
			}
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName"> 
		/// </param>
		/// <param name="qName"> 
		/// </param>
		/// <exception cref="org.xml.sax.SAXException"> 
		/// </exception>
		public override void endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
				
			if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS)) {
				((AcceptDelivery) msg).ProcessCardBundle=processCardBuilder.ProcessCardBundleObject;
				insideHeader = false;
			}
			else
				base.endElement(uri, localName, qName);
			
			currentElement = null;
		}
		static AcceptDeliveryBuilder() {
			log = LogFactory.getLog(typeof(AcceptDeliveryBuilder));
		}
	}
}