namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;
	using java.util;
	using de.osci.osci12.messageparts;

	/// <summary> Diese Klasse ist die Superklasse aller OSCI-Antwortnachrichtenobjekte.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	abstract public class OSCIResponseTo:OSCIMessage {

		internal String Language;
		FeedbackObject[] feedbackObjects;

		/// <summary> Liefert das Krzel der verwendeten Sprache.
		/// </summary>
		/// <value> Sprachkrzel
		/// </value>
		virtual public System.String language {
			get {
				return Language;
			}
			
		}

		internal void  setFeedback(String[] code) {
			feedBack = new System.String[code.Length][];
			for (int i = 0; i < code.Length; i++) {
				feedBack[i] = new System.String[3];
			}
			for (int i = 0; i < code.Length; i++) {
				feedBack[i][0] = dialogHandler.LanguageList;
				feedBack[i][1] = code[i];
				feedBack[i][2] = text.getString(code[i]);
			}
		}

		/// <summary> Liefert die Rckmeldung (Feedback-Eintrag) als String-Array zurck.
		/// Der erste Index des Arrays entspricht dem Index des Entry-Elementes.
		/// Beim zweiten Index bezeichnet
		/// <p>0 - das Sprachkrzel (z.B. "de", "en-US", optional)</p>
		/// <p>1 - den Code</p>
		/// <p>2 - den Text</p>
		/// </summary>
		/// <value> Rckmeldung
		/// </value>
		virtual public System.String[][] Feedback {
			set {
				feedBack = value;
			}
			get {
				return feedBack;
			}
		}
		public FeedbackObject[] FeedbackObjects { get {
			if (feedBack == null) return null; 
			if (feedbackObjects == null) { 
				feedbackObjects = new FeedbackObject[feedBack.Length]; 
				for (int i = 0; i < feedBack.Length; i++) 
					feedbackObjects[i] = MessagePartsFactory.createFeedbackObject((String[])feedBack[i]); 
			} 
			return feedbackObjects; }
		}      
 

		new private static Log log;
		
		protected internal System.String[][] feedBack;	
		internal ResourceBundle text;	
		private System.String defaultLanguage = "de";
		
		/// <summary> Fr Weiterleitungsauftrag und Annahmeauftrag.
		/// Hier wird der Stream gleich mit in den Konstruktor geschrieben, 
		/// fraglich ist ob ein Dialog weitergereicht werden muss / kann.
		/// </summary>
		internal OSCIResponseTo(DialogHandler dh, System.IO.Stream in_Renamed):base(dh) {
		}
		
		internal OSCIResponseTo() {
		}
		
		internal OSCIResponseTo(DialogHandler dh):base(dh) {
            this.Language = defaultLanguage;
            text = ResourceBundle.getBundle("de.osci.osci12.extinterfaces.language.Text", new System.Globalization.CultureInfo(defaultLanguage));
            SupportClass.Tokenizer lang = new SupportClass.Tokenizer(dialogHandler.LanguageList.Trim(), " ");
			while (lang.HasMoreTokens()) 
            {
				this.Language = lang.NextToken();
				if (language.IndexOf((System.Char) '-') == - 1) 
					text = ResourceBundle.getBundle("de.osci.osci12.extinterfaces.language.Text", new System.Globalization.CultureInfo(language));
				else 
					text = ResourceBundle.getBundle("de.osci.osci12.extinterfaces.language.Text", new System.Globalization.CultureInfo(language.Substring(0,2) + "-" + language.Substring(3)));
                break;
            }
		}
		
		internal OSCIResponseTo(DialogHandler dh, OSCIRequest req):base(dh) {
		}
		
		internal virtual System.String writeFeedBack() 
	{ 
			System.Text.StringBuilder sb = new System.Text.StringBuilder("<"); 
			sb.Append(osciNSPrefix); 
			sb.Append(":Feedback>"); 
			for (int i = 0; i < feedBack.Length; i++) { 
				sb.Append("<"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Entry xml:lang=\""); 
				sb.Append(((String[]) feedBack[i])[0]); 
				sb.Append("\"><"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Code>"); 
				sb.Append(((String[]) feedBack[i])[1]); 
				sb.Append("</"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Code><"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Text>"); 
				sb.Append(((String[]) feedBack[i])[2]); 
				sb.Append("</"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Text></"); 
				sb.Append(osciNSPrefix); 
				sb.Append(":Entry>"); 
			} 
			sb.Append("</"); 
			sb.Append(osciNSPrefix); 
			sb.Append(":Feedback>"); 
			return sb.ToString(); 
		}    
	
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
		}
		
		/// <summary> Bringt eine Supplier-Signatur an.
		/// </summary>
		public virtual void  sign() {
			base.sign(dialogHandler.Supplier);
		}
		
		/// <summary> Nachricht verschlsseln
		/// </summary>
		internal virtual void  encrypt() {
		}

		static OSCIResponseTo() {
			log = LogFactory.getLog(typeof(OSCIResponseTo));
		}
	}
}