namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.helper;
	using de.osci.osci12.common;
	using de.osci.osci12.dotcommon;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.roles;
	using de.osci.osci12.soapheader;

	/// <summary><p><H4>Annahmeantwort</H4></p>
	/// Mit dieser Klasse werden Nachrichtenobjekte fr Annahmeantworten
	/// angelegt. Ein passiver Client, der als Supplier fungiert, mu nach Empfang
	/// eines Annahmeauftrags eine Instanz dieser Klasse aufbauen und an den Intermedir
	/// zurcksenden. Die Nachricht enthlt inhaltlich lediglich eine Rckmeldung ber
	/// den Empfang der Nachricht (Feedback).
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	/// <seealso cref="de.osci.osci12.messagetypes.AcceptDelivery">
	/// </seealso>
	public class ResponseToAcceptDelivery:OSCIResponseTo {

		new private static Log log;

		/// <summary> Setzt die Rckmeldungen (Fehler und Warnungen) auf Auftragsebene
		/// </summary>
		/// <param name="code">Array mit Fehlercodes
		/// </param>
		public void setFeedback(String[] code) 
        {
			base.setFeedback(code);
		}

        /// <summary> Setzt die feedback-Eintrge mit frei whlbaren Texten und Sprachen.
        /// Der erste Index des Arrays entspricht dem Index des Entry-Elementes.
        /// Beim zweiten Index bezeichnet
        /// 0- das Sprachkrzel
        /// 1- den Code
        /// 2- den Text
        /// </summary>
        override public System.String[][] Feedback
        {
			set 
            {
				base.Feedback = value;
			}
		}

		/// <summary> Legt ein Nachrichtenobjekt fr einen Annahmeantwort an.
		/// </summary>
		/// <param name="req">Auftragsnachricht</param>
		public ResponseToAcceptDelivery(AcceptDelivery req):base(req.DialogHandler) {			
			dialogHandler.Controlblock.Response = dialogHandler.prevChallenge;
			dialogHandler.Controlblock.Challenge = Tools.createRandom(10);
			// Hier wurde die Spec korrigiert, in der vorigen Zeile sollte auch null gesetzt werden
			dialogHandler.Controlblock.ConversationID = null;
			dialogHandler.Controlblock.SequenceNumber = - 1;
			messageType = RESPONSE_TO_ACCEPT_DELIVERY;
		}
		
		internal ResponseToAcceptDelivery(DialogHandler dh):base(dh) {
			addressee = ((Addressee) dh.Supplier);
			messageType = RESPONSE_TO_ACCEPT_DELIVERY;
		}
		
		public override void  compose() 
		{
			base.compose();
			osciH = new OsciH("responseToAcceptDelivery", writeFeedBack());
			body = new Body("");
			stateOfMsg |= STATE_COMPOSED;
			body.setNSPrefixes(this);
			stateOfMsg |= STATE_COMPOSED;
		}
		
		public virtual void  getMessageAsStream(System.IO.Stream out_Renamed) {
			this.writeXML(out_Renamed);
		}
		
		/// <summary> Bringt eine Supplier-Signatur an.
		/// </summary>
		/// @throws IOException bei Schreib-/Leseproblemen 
		/// @throws OSCIRoleException wenn dem Rollenobjekt, das als Client fungiert
		/// kein Signer-Objekt zugeordnet wurde.
		/// @throws java.security.SignatureException bei Signatur-Problemen
		/// @throws de.osci.osci12.common.OSCICancelledException bei Abbruch durch den
		/// Benutzer
		public override void  sign() {
			base.sign(dialogHandler.Supplier);
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			base.writeXML(out_Renamed);
			// Supplier-Signatur
			if (signatureHeader != null) {
				signatureHeader.writeXML(out_Renamed);
				nonIntermediaryCertificatesH.writeXML(out_Renamed);
			}
			
			osciH.writeXML(out_Renamed);
			completeMessage(out_Renamed);
		}
		
		/// <summary> Serialisiert und schreibt die Nachricht - ggf. verschlsselt - in den bergebenen Stream.
		/// Die ausgehende Nachricht kann zu Debug- oder Archivierungszwecken (in jedem Fall
		/// unverschlsselt) in den zweiten bergebenen Stream geschrieben werden.
		/// Dieser Parameter kann null sein.
		/// </summary>
		/// <param name="out_Renamed">Stream, in den die Antwortnachricht geschrieben werden soll.
		/// </param>
		/// <param name="storeOutput">Stream, in dem die (unverschlsselte) Antwortnachricht gespeichert werden soll.
		/// (NOT AVAILABLE!)
		/// </param>
		/// @throws OSCIRoleException wenn erforderliche Zertifikate fehlen
		/// @throws IOException bei Schreibproblemen
		/// @throws java.security.NoSuchAlgorithmException wenn ein bentigter
		/// Algorithmus nicht untersttzt wird
		public virtual void  writeToStream(System.IO.Stream out_Renamed, System.IO.Stream storeOutput) {
			if (dialogHandler.Encryption) {
				new SOAPMessageEncrypted(this, storeOutput).writeXML(out_Renamed);
			} else 
				if (storeOutput != null) {
					StoreOutputStream sos = new StoreOutputStream(out_Renamed, storeOutput);
					writeXML(sos);
					sos.Close();
				} 
			else 
				writeXML(out_Renamed);
		}
		
		static ResponseToAcceptDelivery() {
			log = LogFactory.getLog(typeof(ResponseToAcceptDelivery));
		}
	}
}