namespace de.osci.osci12.messagetypes {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;

	/// <summary><H4>ResponseToFetchDelivery-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ResponseToFetchDeliveryBuilder:OSCIMessageBuilder
	{
		new private static Log log;

		/// <summary> Objekt ProcessCardBundle fr ProcessCard Information
		/// </summary>
		private ProcessCardBundleBuilder processCardBuilder = null;

		/// <summary> Objekt Feedback fr Feedback Information
		/// </summary>
		private FeedbackBuilder feedbackBuilder = null;
		
		/// <summary> Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		/// <param name="parentHandler">
		/// </param>
		public ResponseToFetchDeliveryBuilder(OSCIEnvelopeBuilder parentHandler):base(parentHandler)
		{
			msg = new de.osci.osci12.messagetypes.ResponseToFetchDelivery(parentHandler.dhHandler);
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes)
		{
			log.debug("Start Element RspFwdDel: " + qName);
			if (insideBody)
				{
				if (localName.Equals("ContentPackage")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new ContentPackageBuilder(this));
				}
			}
			else if (insideHeader)
			{
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{1, 1, 1, 1}));
					// ### Auswerten des ClientSignature Headers###
				}
				else if (localName.Equals("SupplierSignature")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.Equals("responseToFetchDelivery")&& uri.Equals(OSCI_XMLNS))
				{
					;
					//nothing to do
				}
				else if (localName.Equals("ProcessCardBundle")&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{1, - 1, - 1, - 1};
					processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(processCardBuilder);
				}
				else if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS))
				{
					feedbackBuilder = new FeedbackBuilder(this);
					parentBuilder.xmlReader.setContentHandler(feedbackBuilder);
				}
				else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchDelivery) msg).SelectionMode = OSCIMessage.SELECT_BY_MESSAGE_ID;
					currentElement = new System.Text.StringBuilder();
				}
				else if (localName.Equals("ReceptionOfDelivery")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchDelivery) msg).SelectionMode = OSCIMessage.SELECT_BY_DATE_OF_RECEPTION;
					currentElement = new System.Text.StringBuilder();
				}
				else if (localName.Equals("RecentModification")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchDelivery) msg).SelectionMode = OSCIMessage.SELECT_BY_RECENT_MODIFICATION;
					currentElement = new System.Text.StringBuilder();
				}
				else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{0, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else if (localName.ToUpper().Equals("NonIntermediaryCertificates".ToUpper())&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{- 1, - 1, - 1, - 1, - 1, - 1, 0};
					parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
					log.debug("Accept Delivery Element: " + qName);
				}
				else if (!(localName.Equals("fetchDelivery")&& uri.Equals(OSCI_XMLNS)) && !(qName.Equals("osci:SelectionRule")&& uri.Equals(OSCI_XMLNS)))
				{
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			}
			else
			{
				base.startElement(uri, localName, qName, attributes);
			}
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri"> 
		/// </param>
		/// <param name="localName"> 
		/// </param>
		/// <param name="qName"> 
		/// </param>
		/// <exception cref="org.xml.sax.SAXException"> 
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName)
		{
			log.debug("End-Element: " + qName);

			if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
				((ResponseToFetchDelivery) msg).selectionRule = MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
			}
			else if ((localName.Equals("ReceptionOfDelivery")&& uri.Equals(OSCI_XMLNS)) ||
				(localName.Equals("RecentModification")&& uri.Equals(OSCI_XMLNS))) {
				((ResponseToFetchDelivery) msg).selectionRule = currentElement.ToString();
			}
			else if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS)) {
				ResponseToFetchDelivery rtmd = ((ResponseToFetchDelivery) msg);

				if (processCardBuilder != null) {
					rtmd.processCardBundle = processCardBuilder.ProcessCardBundleObject;
					rtmd.messageId = processCardBuilder.ProcessCardBundleObject.MessageId;
				}

				insideHeader = false;
			}
			else
				base.endElement(uri, localName, qName);

			currentElement = null;
		}
		static ResponseToFetchDeliveryBuilder()
		{
			log = LogFactory.getLog(typeof(ResponseToForwardDeliveryBuilder));
		}
	}
}