namespace de.osci.osci12.messagetypes {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;

	/// <summary><H4>ResponseToFetchProcessCard-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ResponseToFetchProcessCardBuilder:OSCIMessageBuilder
	{
		private void  InitBlock()
		{
			processCardBundles = new System.Collections.ArrayList();
		}
		new private static Log log;
		/// <summary>  Objekt ProcessCardBundle fr ProcessCard Information
		/// </summary>
		private ProcessCardBundleBuilder processCardBuilder = null;
		
		private System.Collections.ArrayList processCardBundles;
		/// <summary>  Objekt Feedback fr Feedback Information
		/// </summary>
		private FeedbackBuilder feedbackBuilder = null;
private System.Text.StringBuilder messageIds = new System.Text.StringBuilder();		
		/// <summary> Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		/// <param name="parentHandler">
		/// </param>
		public ResponseToFetchProcessCardBuilder(OSCIEnvelopeBuilder parentHandler):base(parentHandler)
		{
			InitBlock();
			msg = new de.osci.osci12.messagetypes.ResponseToFetchProcessCard(parentHandler.dhHandler);
		}
		private void setSelectionAttributes(Attributes att) {
			if ("true".Equals(att.getValue("NoReception")))
				((ResponseToFetchProcessCard) msg).selectNoReceptionOnly=true;

			if ("Addressee".Equals(att.getValue("Role")))
				((ResponseToFetchProcessCard) msg).roleForSelection=OSCIMessage.SELECT_ADDRESSEE;
			else if ("Originator".Equals(att.getValue("Role")))
				((ResponseToFetchProcessCard) msg).roleForSelection=OSCIMessage.SELECT_ORIGINATOR;
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes)
		{
			if (log.DebugEnabled)
				log.debug("Start Element RspFtProc: " + localName);
			
			if (insideBody)
			{
				if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS))
				{
					feedbackBuilder = new FeedbackBuilder( this);
					parentBuilder.xmlReader.setContentHandler(feedbackBuilder);
				}
				else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchProcessCard) msg).SelectionMode = OSCIMessage.SELECT_BY_MESSAGE_ID;
					setSelectionAttributes(attributes);
					currentElement= new System.Text.StringBuilder();
				}
				else if (localName.Equals("ReceptionOfDelivery")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchProcessCard) msg).SelectionMode = OSCIMessage.SELECT_BY_DATE_OF_RECEPTION;
					setSelectionAttributes(attributes);
					currentElement= new System.Text.StringBuilder();
				}
				else if (localName.Equals("RecentModification")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchProcessCard) msg).SelectionMode = OSCIMessage.SELECT_BY_RECENT_MODIFICATION;
					currentElement= new System.Text.StringBuilder();
				}
				else if (localName.Equals("Quantity")&& uri.Equals(OSCI_XMLNS))
				{
					((ResponseToFetchProcessCard) msg).QuantityLimit=System.Int64.Parse(attributes.getValue("Limit"));
				}
				else if (localName.Equals("responseToFetchProcessCard")&& uri.Equals(OSCI_XMLNS))
				{
					;
					//nothing to do
				}
				else if (localName.Equals("ProcessCardBundle")&& uri.Equals(OSCI_XMLNS))
				{
					if (processCardBuilder != null)
					{
						processCardBundles.Add(processCardBuilder.ProcessCardBundleObject);
					}
					int[] check = new int[]{1, - 1, - 1, - 1};
					processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(processCardBuilder);
				}
			}
			else if (insideHeader)
			{
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{1, 1, 1, 1}));
					// ### Auswerten des ClientSignature Headers###
				}
				else if (localName.Equals("SupplierSignature")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{0, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else
				{
					//		new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + qName);
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			}
			else
			{
				base.startElement(uri, localName, qName, attributes);
			}
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName)
		{
			if (log.DebugEnabled) log.debug("End-Element: " + localName);

			if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
//				((ResponseToFetchProcessCard) msg).SelectionRule =MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
				if (messageIds.Length > 0)
					messageIds.Append('&');
				try {
					messageIds.Append(MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING));
				}
				catch (System.IO.IOException ex) {
					throw new SAXException(ex);
				}
			}
			else if ((localName.Equals("ReceptionOfDelivery")&& uri.Equals(OSCI_XMLNS)) ||
				(localName.Equals("RecentModification"))&& uri.Equals(OSCI_XMLNS)) {
				((ResponseToFetchProcessCard) msg).SelectionRule=currentElement.ToString();
			}
			else if (localName.Equals("SelectionRule")&& uri.Equals(OSCI_XMLNS)) {
				if (((ResponseToFetchProcessCard) msg).selectionMode == OSCIMessage.SELECT_BY_MESSAGE_ID)
((ResponseToFetchProcessCard) msg).SelectionRule =messageIds.ToString();
				//((ResponseToFetchProcessCard) msg).SelectionRule=messageIds.ToString();
			}
			else if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) {
				ResponseToFetchProcessCard rtmd = ((ResponseToFetchProcessCard) msg);

				if (processCardBuilder != null)
					processCardBundles.Add(processCardBuilder.ProcessCardBundleObject);

				rtmd.processCardBundles = (ProcessCardBundle[])processCardBundles.ToArray(typeof(ProcessCardBundle));
				insideBody = false;
			}
			else
				base.endElement(uri, localName, qName);

			currentElement = null;
		}
		static ResponseToFetchProcessCardBuilder()
		{
			log = LogFactory.getLog(typeof(ResponseToForwardDeliveryBuilder));
		}
	}
}