namespace de.osci.osci12.messagetypes{

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.common;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;
 
	/// <summary><H4>ResponseToGetMessageId-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ResponseToGetMessageIdBuilder:OSCIMessageBuilder {

		new private static Log log;

		/// <summary>  Objekt Feedback fr Feedback Information
		/// </summary>
		private FeedbackBuilder feedbackBuilder = null;
		
		/// <summary>  Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		/// <param name="parentHandler">
		/// </param>
		public ResponseToGetMessageIdBuilder(OSCIEnvelopeBuilder parentHandler):base(parentHandler) {
			msg = new de.osci.osci12.messagetypes.ResponseToGetMessageId(parentHandler.dhHandler, true);
		}
		
		/// <summary> 
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start Element RspFwdDel: " + qName);
			
			if (localName.Equals("Header")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = false;
				insideHeader = true;
			}
			else if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = true;
				insideHeader = false;
			}
			else if (insideBody) {
				if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS)) {
					feedbackBuilder = new FeedbackBuilder(this);
					parentBuilder.xmlReader.setContentHandler(feedbackBuilder);
				}
				else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
					currentElement = new System.Text.StringBuilder();//("".ToCharArray());
					//nothing to do
				}
			}
			else if (insideHeader) {
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{1, - 1, 1, 1}));
					// ### Auswerten des ClientSignature Headers###
				}
				else if (localName.Equals("SupplierSignature")&& uri.Equals(OSCI_XMLNS)) {
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.Equals("responseToGetMessageId")&& uri.Equals(OSCI_XMLNS)) {
					//nothing to do
				}
				else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
					int[] check = new int[]{0, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else {
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			}
			else {
				new org.xml.sax.SAXException("Nicht vorgesehens Element! Element Name:" + qName);
			}
		}
		
		/// <summary>  
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			if (log.DebugEnabled) log.debug("End-Element: " + qName);

			if (localName.Equals("Body")&& uri.Equals(SOAP_XMLNS)) {
				insideBody = false;
			}
			else if (localName.Equals("MessageId")&& uri.Equals(OSCI_XMLNS)) {
				((ResponseToGetMessageId) msg).messageId = 
					MoreSupport.NewString(de.osci.helper.Base64.decode(currentElement.ToString()), Constants.CHAR_ENCODING);
			}
			else if (!(localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)))
				base.endElement(uri, localName, qName);
			currentElement = null;
		}
		static ResponseToGetMessageIdBuilder() {
			log = LogFactory.getLog(typeof(ResponseToGetMessageIdBuilder));
		}
	}
}