namespace de.osci.osci12.messagetypes {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.soapheader;
	
	/// <summary><H4>ResponseToStoreDelivery-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ResponseToStoreDeliveryBuilder:OSCIMessageBuilder
	{
		new private static Log log;

		/// <summary>  Objekt ProcessCardBundle fr ProcessCard Information
		/// </summary>
		private ProcessCardBundleBuilder processCardBuilder = null;

		/// <summary>  Objekt Feedback fr Feedback Information
		/// </summary>
		private FeedbackBuilder feedbackBuilder = null;
		
		/// <summary>  Constructor for the ResponseToForwardDeliveryBuilder object
		/// </summary>
		/// <param name="xmlReader">Description of Parameter
		/// </param>
		/// <param name="parentHandler"> Description of Parameter
		/// </param>
		/// <param name="dh">Description of Parameter
		/// </param>
		public ResponseToStoreDeliveryBuilder(OSCIEnvelopeBuilder parentHandler):base(parentHandler)
		{
			msg = new de.osci.osci12.messagetypes.ResponseToStoreDelivery(parentHandler.dhHandler, true);
		}
		
		/// <summary>  Description of the Method
		/// </summary>
		/// <param name="uri">Description of Parameter
		/// </param>
		/// <param name="localName">Description of Parameter
		/// </param>
		/// <param name="qName">Description of Parameter
		/// </param>
		/// <param name="attributes">Description of Parameter
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">Description of Exception
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes)
		{
			log.debug("Start Element RspStrDel: " + qName + insideHeader + localName + uri);

			if (insideBody)
			{
			}
			else if (insideHeader)
			{
				// ### Auswerten des ControlBlock Headers###
				if (localName.Equals("ControlBlock")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[]{1, - 1, 1, 1}));
				}
					// ### Auswerten des ClientSignature Headers###
				else if (localName.Equals("SupplierSignature")&& uri.Equals(OSCI_XMLNS))
				{
					parentBuilder.xmlReader.setContentHandler(new OSCISignatureBuilder(parentBuilder.xmlReader, this, attributes));
				}
				else if (localName.Equals("responseToStoreDelivery")&& uri.Equals(OSCI_XMLNS))
				{
					;
					//nothing to do
				}
				else if (localName.Equals("ProcessCardBundle")&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{1, - 1, - 1, - 1};
					processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, attributes, check);
					parentBuilder.xmlReader.setContentHandler(processCardBuilder);
				}
				else if (localName.Equals("Feedback")&& uri.Equals(OSCI_XMLNS))
				{
					feedbackBuilder = new FeedbackBuilder( this);
					parentBuilder.xmlReader.setContentHandler(feedbackBuilder);
				}
				else if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS))
				{
					int[] check = new int[]{0, - 1};
					parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
				}
				else
				{
					startCustomSoapHeader(uri, localName, qName, attributes);
				}
			}
		    else
				base.startElement(uri, localName, qName, attributes);
		}
		
		/// <summary>  Description of the Method
		/// </summary>
		/// <param name="uri">Description of Parameter
		/// </param>
		/// <param name="localName">Description of Parameter
		/// </param>
		/// <param name="qName">Description of Parameter
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">Description of Exception
		/// </exception>
		public override void  endElement(System.String uri, System.String localName, System.String qName)
		{
			if (log.DebugEnabled) log.debug("End-Element: " + localName);
			if (localName.Equals("Header") && uri.Equals(SOAP_XMLNS))
			{
				if (processCardBuilder != null)
				{
					((ResponseToStoreDelivery) msg).processCardBundle = processCardBuilder.ProcessCardBundleObject;
				}
			}
			else
				base.endElement(uri, localName, qName);
			currentElement = null;
		}
		static ResponseToStoreDeliveryBuilder()
		{
			log = LogFactory.getLog(typeof(ResponseToForwardDeliveryBuilder));
		}
	}
}