namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.messagetypes;
	using de.osci.osci12.common;
 
	/// <summary>Diese Klasse reprsentiert eine SOAP-Fehlermeldung auf Nachrichtenebene.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class SOAPFault:OSCIResponseTo {

		new private static Log log = LogFactory.getLog(typeof(SOAPFault));

		private static String soapFaultIntro = "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:osci=\"http://www.osci.de/2002/04/osci\"><soap:Body><soap:Fault><faultcode>soap:",
			soapFaultExtro = "</osci:Code></detail></soap:Fault></soap:Body></soap:Envelope>";

		private String oscicode, soapfault;

		/// <summary>Legt ein SOAP-Fehlerobjekt fr den genannten OSCI-Code an.
		/// </summary>
		/// <param name="oscicode">OSCI-Fehlercode (s. Spezifikation)
		/// </param>
		public SOAPFault(String oscicode):base() {
			this.oscicode = oscicode;
			messageType = RESPONSE_TO_EXIT_DIALOG;
		}

		protected void compose() {
			soapfault = soapFaultIntro;
			if (oscicode.Equals("9000") || oscicode.Equals("9503"))
				soapfault += "Server";
			else soapfault += "Client";
			soapfault += "</faultcode><faultstring>" + DialogHandler.text.getString(oscicode)
				+ "</faultstring><detail><osci:Code>" + oscicode + soapFaultExtro;

			stateOfMsg |= STATE_COMPOSED;
		}

		protected void writeXML(dotcommon.OutputStream out0){
			compose();
			MoreSupport.ToStream(out0,"\r\nMIME-Version: 1.0\r\nContent-Type: Multipart/Related; boundary=" + DialogHandler.boundary +	"; type=text/xml\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out0,"\r\n--" + DialogHandler.boundary + "\r\nContent-Type: text/xml; charset=UTF-8\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out0,"Content-Transfer-Encoding: 8bit\r\nContent-ID: <" + contentID + ">\r\n\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out0,"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n\r\n",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out0,soapfault,Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out0,"\r\n--" + DialogHandler.boundary + "--\r\n",Constants.CHAR_ENCODING);
		}

		/// <summary>Mit dieser Methode knnen passive Empfnger SOAP-Fehlernachrichten an
		/// den Intermedir zurckschicken.
		/// </summary>
		/// <param name="out0">OutputStream
		/// </param>
		public void writeToStream(dotcommon.OutputStream out0) {
			writeXML(out0);
		}
	}
}
