namespace de.osci.osci12.messagetypes {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.messagetypes;
	using de.osci.osci12.common;
	using org.xml.sax;

	/// <summary><H4>SOAPFault-Parser</H4>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class SOAPFaultBuilder:OSCIMessageBuilder {

		new private static Log log = LogFactory.getLog(typeof(SOAPFaultBuilder));
		private String faultcode, faultstring, oscicode;

		public SOAPFaultBuilder(OSCIEnvelopeBuilder parentBuilder):base(parentBuilder) {
		msg = new SOAPFault("");
		}

		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="SAXException">
		/// </exception>
		public override void startElement(String uri, String localName, String qName, Attributes attributes){
			if (log.DebugEnabled) log.debug("Start Element SOAPFaultBuilder: " + localName+"-"+localName);
			if (localName.Equals("Body")){insideBody = true;}
			else if (insideBody) {
				if (localName.Equals("faultcode") || localName.Equals("faultstring") || 
					localName.Equals("Code")&& uri.Equals(OSCI_XMLNS))
					currentElement = new System.Text.StringBuilder();
			}
			else if (!localName.Equals("detail"))
				throw new SAXException(DialogHandler.text.getString("unexpected_entry")+": "+localName);
		}

		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="SAXException">
		/// </exception>
		public override void endElement(String uri, String localName, String qName){
			if (log.DebugEnabled) log.debug("End-Element: " + localName);
			if (localName.Equals("faultcode"))
				faultcode = currentElement.ToString();
			else if (localName.Equals("faultstring"))
				faultstring = currentElement.ToString();
			else if (localName.Equals("Code")&& uri.Equals(OSCI_XMLNS))
				oscicode = currentElement.ToString();
			else if (localName.Equals("Envelope") && uri.Equals(SOAP_XMLNS)) {
				if (faultcode.Equals(this.OSCIMessage.soapNSPrefix +":Server") && uri.Equals(SOAP_XMLNS))
					throw new SAXException(new SoapServerException(oscicode, faultstring));
				else
					throw new SAXException(new SoapClientException(oscicode, faultstring));
 
			}
			currentElement = null;
		}
	}
}
