namespace de.osci.osci12.roles {

	using System;
	using java.security.cert;
	using de.osci.osci12.extinterfaces.crypto;

	/// <summary> Diese Klasse stellt einen OSCI-Empfnger dar.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Addressee:Role {

		/// <summary> Konstruktor fr den Empfang einer Nachricht.
		/// </summary>
		/// <param name="signer">Signer-Objekt, welches die Signatur der Rckantwort erstellen
		/// soll (null, wenn keine Signatur gewnscht).
		/// </param>
		/// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Nachricht entschlsseln
		/// soll.
		/// </param>
		public Addressee(Signer signer, Decrypter decrypter) {
			this.signer = signer;
			this.decrypter = decrypter;
		}
	
		/// <summary> Konstruktor fr das Versenden einer Nachricht.
		/// </summary>
		/// <param name="signatureCertificate">Zertifikat, mit dem die Signatur der Rckantwort geprft wird.
		/// </param>
		/// <param name="cipherCertificate">Zertifikat, mit dem die Nachricht verschlsselt werden soll.
		/// </param>
		public Addressee(X509Certificate signatureCertificate, X509Certificate cipherCertificate) {
			this.signatureCertificate = signatureCertificate;
			this.cipherCertificate = cipherCertificate;
		}
	}
}