namespace de.osci.osci12.roles {

	using System;
	using java.security.cert;
	using de.osci.osci12.extinterfaces.crypto;

	/// <summary> Diese Klasse stellt einen OSCI-Autor dar.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Author:Role {

		private static int idNr = - 1;
		
		/// <summary> Konstruktor fr den Versand einer Nachricht. Wird als zweiter Parameter
		/// null bergeben, weil keine Verschlsselung der Rckantwort gewnscht wird,
		/// mu trotzdem mit CipherCertificate(X509Certificate) ein
		/// Verschlsselungszertifikat gesetzt werden, weil dies Voraussetzung fr
		/// die Teilnahme an OSCI ist.
		/// </summary>
		/// <param name="signer">Signer-Objekt, welches die Signatur der Nachricht erstellen
		/// soll (null, wenn keine Signatur gewnscht).
		/// </param>
		/// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Rckantwort entschlsseln
		/// soll (null, wenn die Nachricht nicht verschlsselt wird).
		/// </param>
		public Author(Signer signer, Decrypter decrypter) {
			this.signer = signer;
			this.decrypter = decrypter;
			idNr++;
			id += idNr;
		}
		
		/// <summary> Konstruktor fr den Empfang einer Nachricht.
		/// </summary>
		/// <param name="signatureCertificate">Zertifikat, mit dem die Signatur der Nachricht geprft wird.
		/// </param>
		/// <param name="cipherCertificate">Zertifikat, mit dem die Rckantwort verschlsselt werden soll.
		/// </param>
		public Author(X509Certificate signatureCertificate, X509Certificate cipherCertificate) {
			this.signatureCertificate = signatureCertificate;
			this.cipherCertificate = cipherCertificate;
			idNr++;
			id += idNr;
		}
	}
}