namespace de.osci.osci12.roles {

	using System;
	using java.security.cert;
	using de.osci.osci12.extinterfaces.crypto;

	/// <summary> Diese Klasse stellt einen OSCI-Intermedir dar.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class Intermed:Role {

		/// <summary> Ruft die URI des Intermedirs ab, oder legt diese fest.
		/// </summary> 
		/// <value> URI
		/// </value>
		virtual public System.Uri Uri {
			get {
				return uri;
			}			
			set {
				this.uri = value;
			}
		}

		private System.Uri uri = null;
		
		/// <summary> Konstruktor fr ein Intermedirs-Objekt (intermedirsseitig).
		/// </summary>
		/// <param name="signer">Signer-Objekt, welches die Signaturen der Nachrichten
		/// erstellen soll (null, wenn keine Signaturen gewnscht).
		/// </param>
		/// <param name="decrypter">Decrypter-Objekt, welches den Inhalt der Nachrichten
		/// entschlsseln soll.
		/// </param>
		public Intermed(Signer signer, Decrypter decrypter) {
			this.signer = signer;
			this.decrypter = decrypter;
		}
		
		/// <summary> Konstruktor fr ein Intermedirs-Objekt (clientseitig).
		/// Das Signaturzertifikat des Intermedirs muss nicht bergeben werden.
		/// Die verwendete URL muss durch die Transportimplementierung aufgelst werden knnen.
		/// </summary>
		/// <param name="signatureCertificate">Zertifikat, mit dem die Signatur einer Antwort
		/// geprft werden kann.
		/// </param>
		/// <param name="cipherCertificate">Zertifikat, mit dem die Nachricht verschlsselt
		/// werden soll.
		/// </param>
		/// <param name="uri">URL des Intermedirs.
		/// </param>
		public Intermed(X509Certificate signatureCertificate, X509Certificate cipherCertificate, System.Uri uri) {
			this.signatureCertificate = signatureCertificate;
			this.cipherCertificate = cipherCertificate;
			this.uri = uri;
		}

	}
}