namespace de.osci.osci12.signature {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;
	using de.osci.osci12.encryption;

	/// <exclude/>
	/// <summary> Builder, der ein Element ds:KeyInfo bearbeitet.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: PPI Financial Systems GmbH</p> 
	/// </summary>
	public class KeyInfoBuilder:DefaultHandler 
	{ 
		protected const String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#"; 
		protected const String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#"; 

		private void  InitBlock() 
		{
			x509v3Buffer = new System.Text.StringBuilder();
		}

		/// <summary> Ruft das aufgebaute KeyInfo ab.
		/// </summary>
		/// <value> KeyInfo.
		/// </value>
		virtual public KeyInfo KeyInfo {
			get {
				return keyInfo;
			}
			
		}

		internal DefaultHandler parentHandler = null;
		internal XMLReader xmlReader = null;
		protected internal static Log log;

		/// <summary> Wird gesetzt, sobald bekannt ist, dass eine Instanz
		/// von KeyInfoX509 aufzubauen ist.
		/// </summary>
		private bool isX509er = false;

		private System.String currentElement = null;

		/// <summary> StringBuffer, der das X.509v3-Zertifikat hlt.
		/// </summary>
		private System.Text.StringBuilder x509v3Buffer;

		private X509DataBuilder x509Builder = null;

		/// <summary> ID.
		/// </summary>
		private System.String keyId = "";

		/// <summary>Aufgebautes KeyInfo. 
		/// </summary>
		internal KeyInfo keyInfo = null;

		/// <summary> Gibt an, ob die Methode
		/// bereits aufgerufen worden ist.
		/// </summary>
		private bool internalEndElementCalled = false;
		
		/// <summary> Konstruktor.
		/// </summary>
		/// <param name="parentHandler">DefaultCursorHandler, der diesen Builder erzeugt hat.
		/// </param>
		/// <param name="xmlReader">Aktueller CursorXMLReader.
		/// </param>
		/// <param name="attributes">Attribute des Elements ds:KeyInfo.
		/// </param>
		public KeyInfoBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes) {
			InitBlock();
			this.xmlReader = xmlReader;
			this.parentHandler = parentHandler;
			keyInfo = new KeyInfo();
			if (attributes != null && attributes.getValue("Id") != null) {
				keyId = attributes.getValue("Id");
				keyInfo.Id = keyId;
			}
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			//      else if (qName.Equals("ds:RetrievalMethod")) {
			//      //        ((KeyInfo)keyInfoStack.firstElement()).setRetrievalMethod(new RetrievalMethod().setURI(attributes.getValue("URI")));
			//      try {
			//        RetrievalMethod ret = new RetrievalMethod();
			//        ret.setURI(attributes.getValue("URI"));
			//        ( (KeyInfo) keyInfoStack.firstElement()).setRetrievalMethod(ret);
			//      }
			//      catch (Exception ex) {
			//        log.error("Fehler beim setzen der RetrievalMethod.");
			//      }
			if (localName.Equals("KeyName") && uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("KeyValue") && uri.Equals(DS_XMLNS)) {
				throw new SAXException("KeyValue - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("RetrievalMethod") && uri.Equals(DS_XMLNS)) {
				RetrievalMethodBuilder retrievalMethodBuilder = new RetrievalMethodBuilder(xmlReader, this, attributes);
				xmlReader.setContentHandler(retrievalMethodBuilder);
			}
			else if (localName.Equals("X509Data") && uri.Equals(DS_XMLNS)) {
				isX509er = true;
				x509Builder = new X509DataBuilder(xmlReader, this, attributes);
				xmlReader.setContentHandler(x509Builder);
			}
			else if (localName.Equals("PGPData") && uri.Equals(DS_XMLNS)) {
				throw new SAXException("PGPData - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("SPKIData") && uri.Equals(DS_XMLNS)) {
				throw new SAXException("SPKI-Data - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("MgmtData") && uri.Equals(DS_XMLNS)) {
				currentElement = "";
			}
			else if (localName.Equals("EncryptedKey") && uri.Equals(XENC_XMLNS)) {
				xmlReader.setContentHandler(new EncryptedKeyBuilder(xmlReader, this, attributes));
			}
				//@todo wirklich nicht untersttzen?
			else if (localName.Equals("AgreementMethod") && uri.Equals(XENC_XMLNS)) {
				throw new SAXException("AgreementMethod- Elemente werden nicht untersttzt");
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("Start-Element :" + localName);
			if (localName.Equals("KeyInfo")&& uri.Equals(DS_XMLNS)) {
				if (parentHandler is EncryptedDataBuilder) {
					((EncryptedDataBuilder) parentHandler).EncryptedData.KeyInfo = keyInfo;
				}
				else if (parentHandler is EncryptedKeyBuilder) {
					((EncryptedKeyBuilder) parentHandler).EncKey.KeyInfo = keyInfo;
				}
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("KeyName")&& uri.Equals(DS_XMLNS)) {
				keyInfo.KeyName = currentElement;
			}
			else if (localName.Equals("KeyValue")&& uri.Equals(DS_XMLNS)) {
				throw new SAXException("KeyValue - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("RetrievalMethod")) {
			}
			else if (localName.Equals("X509Data")&& uri.Equals(DS_XMLNS)) {
			}
			else if (localName.Equals("PGPData")&& uri.Equals(DS_XMLNS)) {
				throw new SAXException("PGPData - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("SPKIData")&& uri.Equals(DS_XMLNS)) {
				throw new SAXException("SPKI-Data - Elemente werden nicht untersttzt");
			}
			else if (localName.Equals("MgmtData")&& uri.Equals(DS_XMLNS)) {
				keyInfo.MgmtData = currentElement;
			}
			else if (localName.Equals("EncryptedData") && uri.Equals(XENC_XMLNS)) {
			}
				//@todo wirklich nicht untersttzen?
			else if (localName.Equals("AgreementMethod") && uri.Equals(XENC_XMLNS)) {
				throw new SAXException("AgreementMethod- Elemente werden nicht untersttzt");
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
			
			// alles wieder herein!!
			//            KeyInfoX509 keyX509 = new KeyInfoX509();
			//            keyX509.setId(new OSCIID(keyId));
			//            StringBuffer x509v3Buffer = x509Builder.getX509DataBuffer();
			//            keyX509.setX509Data(x509v3Buffer.toString());
			//            keyInfo = keyX509;
			//        } else {
			//            KeyInfoRetrieval keyRetrieval = new KeyInfoRetrieval();
			//            keyRetrieval.setId(new OSCIID(keyId));
			//            keyRetrieval.setRetrievalMethod(retMeth);
			//            keyInfo = keyRetrieval;
			//        }
			//        if (managementDataBuilder != null) {
			//            mData = managementDataBuilder.getMgmtData();
			//            keyInfo.setMgmtData(mData);
			//        }
			//        keyInfo.setInput(getInput());
			//        keyInfo.setStartIndex(getParent().getCursor());
			//        keyInfo.setEndIndex(getCursor());
			currentElement = null;
		}
		
		
		public override void  characters(char[] ch, int start, int length) {
			log.debug("Character: " + new System.String(ch, start, length));
			if (currentElement == null) {
				for (int i = 0; i < length; i++) {
if (ch[start + i] > ' '){
	//if ((ch[start + i] == '\r') || (ch[start + i] == '\n') || (ch[start + i] == ' '))
	//					i++;
	//				else {
						throw new SAXException(de.osci.osci12.common.DialogHandler.text.getString("unexpected_char"));
					}
				}
			}
			else currentElement += new System.String(ch, start, length);
		}

		static KeyInfoBuilder() {
			log = LogFactory.getLog(typeof(KeyInfoBuilder));
		}
	}
}