namespace de.osci.osci12.signature {

	using System;
	using org.apache.commons.logging;
	using org.xml.sax;
	using org.xml.sax.helpers;

	/// <exclude/>
	/// <summary> Builder, der ein Element ds:RetrievalMethod bearbeitet.
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: PPI Financial Systems GmbH</p> 
	/// </summary>
	class RetrievalMethodBuilder:DefaultHandler 
	{ 
		protected const String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#";
		private void  InitBlock() {
			retrievalObject = new RetrievalMethod();
		}

		/// <summary> Liefert die aufgebaute RetrievalMethod.
		/// </summary>
		/// <value> RetrievalMethod.
		/// </value>
		virtual public RetrievalMethod RetrievalMethod {
			get {
				return retrievalObject;
			}
			
		}

		internal DefaultHandler parentHandler = null;
		internal XMLReader xmlReader = null;
		protected internal static Log log;

		/// <summary>Aufgebaute RetrievalMethod. 
		/// </summary>
		private RetrievalMethod retrievalObject;
		
		/// <summary> Konstruktor
		/// </summary>
		/// <param name="parentHandler">DefaultCursorHandler, der diesen Builder erzeugt hat.
		/// </param>
		/// <param name="xmlReader">Aktueller CursorXMLReader.
		/// </param>
		/// <param name="attributes">Attribute des Elements ds:RetrievalMethod.
		/// </param>
		/// <exception cref="System.ArgumentException">
		/// </exception>
		public RetrievalMethodBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes) {
			InitBlock();
			this.xmlReader = xmlReader;
			this.parentHandler = parentHandler;
			if (attributes == null) {
				throw new System.ArgumentException("Dem Konstruktor der Klasse RetrievalMethodBuilder wird null bergeben !");
			}
			System.String uri = attributes.getValue("URI");
			retrievalObject.URI = uri;
		}
		
		public override void  endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("RetrievalMethod")&& uri.Equals(DS_XMLNS)) {
				if (parentHandler is KeyInfoBuilder) {
					((KeyInfoBuilder) parentHandler).keyInfo.RetrievalMethod = retrievalObject;
				}
				
				xmlReader.setContentHandler(parentHandler);
			}
			else if (localName.Equals("Transforms")&& uri.Equals(DS_XMLNS)) {
				; //nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				; //nothing to do
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
		}
		
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start Element: " + localName);
			if (localName.Equals("Transforms")&& uri.Equals(DS_XMLNS)) {
				; //nothing to do
			}
			else if (localName.Equals("Transform")&& uri.Equals(DS_XMLNS)) {
				retrievalObject.addTransformer(attributes.getValue("Algorithm"));
			}
			else {
				throw new SAXParseException("Nicht vorgesehenes Element: " + localName, null);
			}
		}
		
		static RetrievalMethodBuilder() {
			log = LogFactory.getLog(typeof(RetrievalMethodBuilder));
		}
	}
}