namespace de.osci.osci12.soapheader {
	
	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;

	/// <exclude/>
	/// <summary> <p> Die Elemente eines ControlBlocks.</p>
	/// 
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class ControlBlockH:HeaderEntry 
	{
	
		new private static Log log;

		/// <summary>Challenge der Nachricht. 
		/// </summary>
		private System.String challenge = null;
		
		/// <summary>Response der Nachricht. 
		/// </summary>
		private System.String response = null;
		
		/// <summary>Sequence Nummer der Nachricht. Sequenz beginnt mit dem Value 0 
		/// </summary>
		private int sequenceNumber = - 1;
		
		/// <summary>Die durch den Supplier vergebene DialogID. 
		/// </summary>
		private System.String conversationID = null;
		
		/// <summary> Creates a new ControlBlock object.
		/// </summary>
		public ControlBlockH() {
		}

		virtual public System.String Challenge {
			get {
				return challenge;
			}
			set {
				this.challenge = value;
			}	
		}

		virtual public System.String ConversationID {
			get {
				return conversationID;
			}
			set {
				this.conversationID = value;
			}	
		}

		virtual public System.String Response {
			get {
				return response;
			}
			set {
				this.response = value;
			}
		}

		virtual public int SequenceNumber {
			get {
				return sequenceNumber;
			}
			set {
				this.sequenceNumber = value;
			}
		}

		override public byte[] getDigestValue(String digestAlgorithm)
        {
			// Im ControlBlockH drfen keine temporren Buffer verwendet werden, weil dieses
			// Objekt als Bestandteil des DialogHandlers immer wieder verndert wird.		
            digestValues.Remove(digestAlgorithm);
			return base.getDigestValue(digestAlgorithm);
		}

		override public long Length {
			get {
				tmpBuffer = null;
				return base.Length;
			}
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix +":ControlBlock",Constants.CHAR_ENCODING);
			out_Renamed.Write(ns,0,ns.Length);
			if (conversationID != null) {
				MoreSupport.ToStream(out_Renamed," ConversationId=\"" + conversationID + "\"",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed," Id=\"controlblock\"",Constants.CHAR_ENCODING);
			
			if (sequenceNumber != - 1) {
				MoreSupport.ToStream(out_Renamed," SequenceNumber=\"" + sequenceNumber + "\"",Constants.CHAR_ENCODING);
			}

			MoreSupport.ToStream(out_Renamed," " + soapNSPrefix + ":actor=\"http://schemas.xmlsoap.org/soap/actor/next\" " + soapNSPrefix + ":mustUnderstand=\"1\"",Constants.CHAR_ENCODING);
			MoreSupport.ToStream(out_Renamed,">",Constants.CHAR_ENCODING);
			
			if (response != null) {
				MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix +":Response>" + response + "</" + osciNSPrefix +":Response>",Constants.CHAR_ENCODING);
			}
			if (challenge != null) {
				MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix +":Challenge>" + challenge + "</" + osciNSPrefix +":Challenge>",Constants.CHAR_ENCODING);
			}
			MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix +":ControlBlock>",Constants.CHAR_ENCODING);
		}
		static ControlBlockH() {
			log = LogFactory.getLog(typeof(ControlBlockH));
		}
	}
}