namespace de.osci.osci12.soapheader {

	using System;
	using java.security.cert;
	using org.apache.commons.logging;
	using org.xml.sax;
	using de.osci.osci12.messageparts;
	using de.osci.osci12.messagetypes;
	using de.osci.osci12.roles;
	
	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class IntermediaryCertificatesHBuilder:de.osci.osci12.messageparts.MessagePartParser 
	{

		new private static Log log;
		
		private const int CIPHER_CERTIFICATE_INTERMEDIARY = 0;
		private const int SIGNATURE_CERTIFICATE_INTERMEDIARY = 1;
		
		private int[] check;
		private IntermediaryCertificatesH ich;
		private int typ;
        private String refId;
		
		private Intermed intermed;
		
		/// <summary> Constructor for the IntermediaryCertificatesHBuilder object
		/// </summary>
		/// <param name="xmlReader">
		/// </param>
		/// <param name="parentHandler">
		/// </param>
		/// <param name="atts">
		/// </param>
		/// <param name="check">
		/// </param>
		public IntermediaryCertificatesHBuilder(OSCIMessageBuilder parentHandler, Attributes atts, int[] check):base(parentHandler) {
			this.check = check;
			ich = new IntermediaryCertificatesH();
			OSCIMessage msg = parentHandler.OSCIMessage; 
			ich.setNSPrefixes(msg); 
			
			if (parentHandler.OSCIMessage.dialogHandler.Supplier is Intermed) {
				intermed = (Intermed) parentHandler.OSCIMessage.dialogHandler.Supplier;
			} else {
				intermed = (Intermed) parentHandler.OSCIMessage.dialogHandler.Client;
				if (intermed == null) {
					intermed = new Intermed((X509Certificate) null, (X509Certificate) null, null);
					parentHandler.OSCIMessage.dialogHandler.client = intermed;
				}
			}
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <param name="attributes">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void  startElement(System.String uri, System.String localName, System.String qName, Attributes attributes) {
			log.debug("Start-Element: " + localName);
			if (localName.Equals("CipherCertificateIntermediary")&& uri.Equals(OSCI_XMLNS)) {
				typ = CIPHER_CERTIFICATE_INTERMEDIARY;
			} else if (localName.Equals("SignatureCertificateIntermediary")&& uri.Equals(OSCI_XMLNS)) {
				typ = SIGNATURE_CERTIFICATE_INTERMEDIARY;
			} else if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				currentElement = new System.Text.StringBuilder();
			} else if (!(localName.Equals("X509Data") && uri.Equals(DS_XMLNS))) {
				throw new org.xml.sax.SAXException("Unerwartetes Element im IntermediaryCertificate-Header: " + localName);
			}
			if (check[typ] == 0) {
				throw new org.xml.sax.SAXException("Unerlaubtes Zertifikat im IntermediaryCertificate-Header, Typ: " + typ);
			}
            if (attributes.getValue("Id") != null)
                refId = attributes.getValue("Id");
		}
		
		/// <summary>
		/// </summary>
		/// <param name="uri">
		/// </param>
		/// <param name="localName">
		/// </param>
		/// <param name="qName">
		/// </param>
		/// <exception cref="org.xml.sax.SAXException">
		/// </exception>
		public override void endElement(System.String uri, System.String localName, System.String qName) {
			log.debug("End-Element: " + localName);
			if (localName.Equals("IntermediaryCertificates")&& uri.Equals(OSCI_XMLNS)) {
				log.debug("Setze IntermedCertificates");
				msg.intermediaryCertificatesH = ich;
				xmlReader.setContentHandler(parentHandler);
			} else if (localName.Equals("X509Certificate")&& uri.Equals(DS_XMLNS)) {
				int i;
				X509Certificate cert;
					
				System.IO.MemoryStream byteStream = new System.IO.MemoryStream((MoreSupport.ToByteArray(currentElement.ToString(), de.osci.osci12.common.Constants.CHAR_ENCODING)));
				de.osci.osci12.common.Base64InputStream base64In = new de.osci.osci12.common.Base64InputStream(byteStream);
				cert = de.osci.helper.Tools.createCertificate(base64In);
					
				if (typ == CIPHER_CERTIFICATE_INTERMEDIARY)
				{
                    if (intermed.hasCipherCertificate())
                    {
                        if (!intermed.CipherCertificate.Equals(cert))
                            throw new System.SystemException("Verschlsselungszertifikat des Intermedirs in der Nachricht stimmt nicht mit dem vorhandenen Zertifikat berein.");
                    }
                    else
                        intermed.CipherCertificate = cert;

                    intermed.cipherRefId = refId;
					ich.CipherCertificateIntermediary = intermed;
				} 
				else if (typ == SIGNATURE_CERTIFICATE_INTERMEDIARY) 
				{
                    if (intermed.hasSignatureCertificate())
                    {
                        if (!intermed.SignatureCertificate.Equals(cert))
                            throw new System.SystemException("Signaturzertifikat des Intermedirs in der Nachricht stimmt nicht mit dem vorhandenen Zertifikat berein.");
                    }
                    else
                        intermed.SignatureCertificate = cert;

                    intermed.signatureRefId = refId;
					ich.SignatureCertificateIntermediary = intermed;
				}
			}
			typ = - 1;
			currentElement = null;
		}
		
		static IntermediaryCertificatesHBuilder() {
			log = LogFactory.getLog(typeof(IntermediaryCertificatesHBuilder));
		}
	}
}