namespace de.osci.osci12.soapheader {

	using System;
	using org.apache.commons.logging;
	using de.osci.osci12.common;
	using de.osci.osci12.roles;

	/// <exclude/>
	/// <summary>
	/// <p>Copyright  2014 Freie Hansestadt Bremen</p>
	/// <p>Erstellt von Governikus GmbH & Co.  KG

	/// <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
	/// freie Softwarebibliotheken genutzt werden.</p><p>Die Lizenzbestimmungen knnen
	/// unter der URL www.osci.de abgerufen oder bei der
	/// Koordinierungsstelle fr IT-Standards (KoSIT), Schillerstr. 1,
	/// 28195 Bremen schriftlich angefordert werden.</p>
	/// 
	/// <p>Author: P. Ricklefs, N. Bngener</p> 
	/// <p>Version: 1.6</p>
	/// </summary>
	public class NonIntermediaryCertificatesH:CertificateH 
	{

		virtual public Addressee CipherCertificateAddressee {
			get {
				return cipherCertificateAddressee;
			}
			set {	
				java.security.cert.X509Certificate c=value.CipherCertificate;
				SupportClass.PutElement(certificates, value.CipherCertificateId, value.CipherCertificate);
				this.cipherCertificateAddressee = value;
			}	
		}

		virtual public Originator CipherCertificateOriginator {
			get {
				return cipherCertificateOriginator;
			}
			set {
				SupportClass.PutElement(certificates, value.CipherCertificateId, value.CipherCertificate);
				this.cipherCertificateOriginator = value;
			}			
		}

		virtual public Author[] CipherCertificatesOtherAuthors {
			get {
				return cipherCertificatesOtherAuthors;
			}
			set {
				System.Collections.Hashtable hs = System.Collections.Hashtable.Synchronized(new System.Collections.Hashtable());
				
				for (int i = 0; i < value.Length; i++) 
				{
					// Nach Import von Rollen aus anderen Nachrichten knnte es vorkommen,
					// dass verschiedene Authors gleiche Ids haben.
					if (hs.ContainsKey(value[i].CipherCertificateId)) {
						if (!hs[value[i].CipherCertificateId].Equals(value[i].CipherCertificate)) {
							throw new OSCIRoleException(DialogHandler.text.getString("id_conflict_cipher_author") + " " + value[i].CipherCertificateId);
						}
					}
					else {
						hs[value[i].CipherCertificateId] = value[i];
						certificates[value[i].CipherCertificateId] = value[i].CipherCertificate;
					}
				}
				this.cipherCertificatesOtherAuthors = (de.osci.osci12.roles.Author[]) SupportClass.ICollectionSupport.ToArray(hs.Values, new de.osci.osci12.roles.Author[hs.Count]);
			}
		}

		virtual public de.osci.osci12.roles.Reader[] CipherCertificatesOtherReaders {
			get {
				return cipherCertificatesOtherReaders;
			}
			set {
				System.Collections.Hashtable hs = System.Collections.Hashtable.Synchronized(new System.Collections.Hashtable());
				
				for (int i = 0; i < value.Length; i++) 
				{
					// Nach Import von Rollen aus anderen Nachrichten knnte es vorkommen,
					// dass verschiedene Readers gleiche Ids haben.
					if (hs.ContainsKey(value[i].CipherCertificateId)) {
						if (!hs[value[i].CipherCertificateId].Equals(value[i].CipherCertificate)) {
							throw new OSCIRoleException(DialogHandler.text.getString("id_conflict_reader") + " " + value[i].CipherCertificateId);
						}
					}
					else {
						hs[value[i].CipherCertificateId] = value[i];
						certificates[value[i].CipherCertificateId] = value[i].CipherCertificate;
					}
				}
				
				this.cipherCertificatesOtherReaders = (de.osci.osci12.roles.Reader[]) SupportClass.ICollectionSupport.ToArray(hs.Values, new de.osci.osci12.roles.Reader[hs.Count]);
			}	
		}

		virtual public Originator SignatureCertificateOriginator {
			get {
				return signatureCertificateOriginator;
			}	
			set {
				Object dumm= value.SignatureCertificate;
				SupportClass.PutElement(certificates, value.SignatureCertificateId, value.SignatureCertificate);
				this.signatureCertificateOriginator = value;
			}
		}

		virtual public Addressee SignatureCertificateAddressee {
			get {
				return signatureCertificateAddressee;
			}			
			set {
				Object dumm=value.SignatureCertificate;
				SupportClass.PutElement(certificates, value.SignatureCertificateId, value.SignatureCertificate);
				this.signatureCertificateAddressee = value;
			}	
		}

		virtual public Author[] SignatureCertificatesOtherAuthors {
			get {
				return signatureCertificatesOtherAuthors;
			}			
			set {
				System.Collections.Hashtable hs = System.Collections.Hashtable.Synchronized(new System.Collections.Hashtable());
				
				for (int i = 0; i < value.Length; i++) {
					// Nach Import von Rollen aus anderen Nachrichten knnte es vorkommen,
					// dass verschiedene Authors gleiche Ids haben.
					if (hs.ContainsKey(value[i].SignatureCertificateId)) {
						if (!hs[value[i].SignatureCertificateId].Equals(value[i].SignatureCertificate)) {
							throw new OSCIRoleException(DialogHandler.text.getString("id_conflict_signer_author") + " " + value[i].SignatureCertificateId);
						}
					}
					else {
						hs[value[i].SignatureCertificateId] = value[i];
						certificates[value[i].SignatureCertificateId] = value[i].SignatureCertificate;
					}
				}
				
				this.signatureCertificatesOtherAuthors = (Author[]) SupportClass.ICollectionSupport.ToArray(hs.Values, new Author[hs.Count]);
			}	
		}
		
		new private static Log log;
		private Originator cipherCertificateOriginator;
		private Author[] cipherCertificatesOtherAuthors;
		private Addressee cipherCertificateAddressee;
		private de.osci.osci12.roles.Reader[] cipherCertificatesOtherReaders;
		private Originator signatureCertificateOriginator;
		private Addressee signatureCertificateAddressee;
		private Author[] signatureCertificatesOtherAuthors;
		
		/// <summary> Creates a new NonIntermediaryCertificatesH object.
		/// </summary>
		/// <param name="parentMessage">
		/// </param>
		public NonIntermediaryCertificatesH() {
		}
		
		public override void  writeXML(System.IO.Stream out_Renamed) {
			if (tmpBuffer != null) {
				byte[] temp_byteArray;
				temp_byteArray = tmpBuffer;
				out_Renamed.Write(temp_byteArray, 0, temp_byteArray.Length);;;;
			} else {
   MoreSupport.ToStream(out_Renamed,"<" + osciNSPrefix +":NonIntermediaryCertificates",Constants.CHAR_ENCODING);
   out_Renamed.Write(ns,0,ns.Length);
   MoreSupport.ToStream(out_Renamed," Id=\"nonintermediarycertificates\" " + soapNSPrefix + ":actor=\"http://www.w3.org/2001/12/soap-envelope/actor/none\" " + soapNSPrefix + ":mustUnderstand=\"1\">",Constants.CHAR_ENCODING);
				
				if (cipherCertificateOriginator != null) {
					addCipherCertificate(cipherCertificateOriginator, out_Renamed);
				}
				if (cipherCertificatesOtherAuthors != null) {
					for (int i = 0; i < cipherCertificatesOtherAuthors.Length; i++) {
						addCipherCertificate(cipherCertificatesOtherAuthors[i], out_Renamed);
					}
				}
				if (cipherCertificateAddressee != null) {
					addCipherCertificate(cipherCertificateAddressee, out_Renamed);
				}
				if (cipherCertificatesOtherReaders != null) {
					for (int i = 0; i < cipherCertificatesOtherReaders.Length; i++) {
						addCipherCertificate(cipherCertificatesOtherReaders[i], out_Renamed);
					}
				}
				if (signatureCertificateOriginator != null) {
					addSignatureCertificate(signatureCertificateOriginator, out_Renamed);
				}
				if (signatureCertificateAddressee != null) {
					addSignatureCertificate(signatureCertificateAddressee, out_Renamed);
				}
				if (signatureCertificatesOtherAuthors != null) {
					for (int i = 0; i < signatureCertificatesOtherAuthors.Length; i++) {
						addSignatureCertificate(signatureCertificatesOtherAuthors[i], out_Renamed);
					}
				}
				MoreSupport.ToStream(out_Renamed,"</" + osciNSPrefix +":NonIntermediaryCertificates>",Constants.CHAR_ENCODING);
			}
		}
		
		static NonIntermediaryCertificatesH() {
			log = LogFactory.getLog(typeof(NonIntermediaryCertificatesH));
		}
	}
}